@@ -92,7 +92,6 @@ typedef enum
   MYX_DBM_OT_FUNCTION
 } MYX_DBM_OBJECT_TYPE;
 
-  
 
 // Custom MySQL error codes
 typedef enum MYX_MYSQL_ERROR_CODES
@@ -100,6 +99,12 @@ typedef enum MYX_MYSQL_ERROR_CODES
   MYX_UTF8_OUT_OF_RANGE=50000
 } MYX_MYSQL_ERROR_CODES;
 
+
+typedef enum
+{
+  MYX_QEL_NOTE= 1, MYX_QEL_WARNING, MYX_QEL_ERROR
+} MYX_QUERY_ERROR_LEVEL;
+
 /*
  * Structs
  */
@@ -118,6 +123,18 @@ typedef struct {
   char *value;
 } MYX_NAME_VALUE_PAIR;
 
+/// [SCRIPT::LibInterfaceMapper] -generate_classes MYX_MYSQL_ERROR_MSG
+typedef struct {
+  MYX_QUERY_ERROR_LEVEL level;
+  int error;
+  char *text;
+} MYX_MYSQL_ERROR_MSG;
+
+/// [SCRIPT::LibInterfaceMapper] -generate_classes MYX_MYSQL_ERROR_MSGS
+typedef struct {
+  unsigned int errors_num;
+  MYX_MYSQL_ERROR_MSG *errors;
+} MYX_MYSQL_ERROR_MSGS;
 
 // Definition of user connections
 
@@ -429,6 +446,10 @@ typedef enum
   MYX_QT_EXPLAIN,
   MYX_QT_CALL,
   MYX_QT_UNION,
+  MYX_QT_CHECK,
+  MYX_QT_ANALYZE,
+  MYX_QT_REPAIR,
+  MYX_QT_OPTIMIZE,
   MYX_QT_EMPTY= 254,
   MYX_QT_UNKNOWN= 255
 } MYX_Q_TYPE;
@@ -494,8 +515,6 @@ typedef enum
 {
   MYX_RSAS_NEW, MYX_RSAS_APPLIED, MYX_RSAS_FAILED, MYX_RSAS_DISCARDED
 } MYX_RS_ACTION_STATUS;
-    
-
 
 // --------------------------
 // Query
@@ -657,6 +676,7 @@ typedef struct MYX_RS_ACTIONS
 
 
 typedef struct {
+  MYX_QUERY_ERROR_LEVEL level;
   int error;
   char *error_text;
   MYX_RS_ACTION *action;
@@ -811,6 +831,7 @@ typedef struct {
   unsigned char auto_inc;
 
   char *default_value;
+  int default_value_is_null;
 
   char *comment;
 
@@ -1026,7 +1047,13 @@ MYX_PUBLIC_FUNC int myx_free_stringlist(MYX_STRINGLIST *stringlist);
 MYX_PUBLIC_FUNC char * get_value_from_text_ex(const char *txt, int txt_length, const char *regexpr, unsigned int substring_nr);
 
 //MySQL connect functions
-MYX_PUBLIC_FUNC MYSQL* myx_mysql_init();
+MYX_PUBLIC_FUNC int myx_mysql_embedded_start();
+MYX_PUBLIC_FUNC void myx_mysql_embedded_prevent_start();
+MYX_PUBLIC_FUNC void myx_mysql_embedded_shutdown();
+MYX_PUBLIC_FUNC MYSQL * myx_mysql_embedded_init();
+
+MYX_PUBLIC_FUNC MYSQL * myx_mysql_init();
+
 MYX_PUBLIC_FUNC void myx_mysql_set_query_hooks(MYSQL *mysql, 
                                                void (*pre_hook)(MYSQL *mysql, void *cdata, const char *query, unsigned int length),
                                                void (*post_hook)(MYSQL *mysql, void *cdata, const char *query, unsigned int length),
@@ -1040,12 +1067,20 @@ MYX_PUBLIC_FUNC char * myx_get_default_schema(MYSQL *mysql);
 MYX_PUBLIC_FUNC unsigned long myx_get_thread_id(MYSQL *mysql);
 MYX_PUBLIC_FUNC int myx_get_mysql_quote_char(MYSQL *mysql, int detect);
 
+MYX_PUBLIC_FUNC char * myx_mysql_esc(MYSQL *mysql, const char *cmd, ...);
 MYX_PUBLIC_FUNC int myx_mysql_query(MYSQL *mysql, const char *cmd);
+MYX_PUBLIC_FUNC int myx_mysql_query_esc(MYSQL *mysql, const char *cmd, ...);
 MYX_PUBLIC_FUNC int myx_mysql_real_query(MYSQL *mysql, const char *cmd, unsigned int len);
+MYX_PUBLIC_FUNC bigint myx_mysql_affected_rows(MYSQL *mysql);
 
 MYX_PUBLIC_FUNC char * myx_get_mysql_full_version(MYSQL *mysql);
 MYX_PUBLIC_FUNC int myx_get_mysql_major_version(MYSQL *mysql);
 MYX_PUBLIC_FUNC int myx_get_mysql_minor_version(MYSQL *mysql);
+MYX_PUBLIC_FUNC int mysql_version_is_later_or_equal_than(MYSQL *mysql,
+                                                         int major_version, int minor_version);
+MYX_PUBLIC_FUNC int mysql_full_version_is_later_or_equal_than(MYSQL *mysql,
+                                                              int major_version, int minor_version,
+                                                              int patchlevel);
 
 MYX_PUBLIC_FUNC char * myx_get_server_variable(MYSQL *mysql, const char *name);
 /*MYX_PUBLIC_FUNC const char * myx_get_server_charset_name(MYSQL *mysql);*/
@@ -1055,7 +1090,11 @@ MYX_PUBLIC_FUNC char * myx_convert_utf8_dbstr(MYSQL *mysql, const char* str);
 
 MYX_PUBLIC_FUNC int myx_mysql_errno(MYSQL *mysql);
 MYX_PUBLIC_FUNC char * myx_mysql_error(MYSQL *mysql);
+MYX_PUBLIC_FUNC void myx_set_mysql_error(char *error, int errn);
 
+MYX_PUBLIC_FUNC MYX_MYSQL_ERROR_MSGS * myx_mysql_error_msgs_fetch(MYSQL *mysql);
+MYX_PUBLIC_FUNC void myx_mysql_error_msgs_free(MYX_MYSQL_ERROR_MSGS *errors);
+  
 //Network functions
 MYX_PUBLIC_FUNC unsigned long myx_resolve_network_name(const char *hostname);
 MYX_PUBLIC_FUNC int myx_ping_host(unsigned long ip, int ping_timeout, MYX_PING_RESULT *ping_result);
@@ -1106,7 +1145,7 @@ MYX_PUBLIC_FUNC const char * myx_tg(MYX_TRANS *trans, const char *id_textgroup,
 
 //Various
 //Check if a string matches a given search string
-MYX_PUBLIC_FUNC int myx_match_pattern(const char *text, const char *search, int case_sensitive);
+MYX_PUBLIC_FUNC int myx_match_pattern(const char *text, const char *search, int case_insensitive, int strict);
 
 //Check whether a given string can be properly converted to an encoding
 MYX_PUBLIC_FUNC int myx_check_utf8_convertible(const char *str, const char *encoding);
@@ -1119,6 +1158,7 @@ MYX_PUBLIC_FUNC int myx_free_syn(MYX_SYN *syn);
 MYX_PUBLIC_FUNC MYX_SQL_HIGHLIGHTING* myx_init_sql_parsing(MYX_SYN* syn);
 MYX_PUBLIC_FUNC int myx_free_sql_highlighting(MYX_SQL_HIGHLIGHTING *h);
 MYX_PUBLIC_FUNC int myx_highlight(MYX_SQL_HIGHLIGHTING *sql_highlighting, const char *text, unsigned int line_no);
+MYX_PUBLIC_FUNC MYX_SYN_TYPE myx_get_identifier_type(MYX_SYN *syn, const char *word);
 
 /* auto-completion */
 MYX_PUBLIC_FUNC MYX_SYN_SUGGESTIONS* myx_lookup_line(MYX_SYN* syn, const char *line);
@@ -1142,6 +1182,7 @@ MYX_PUBLIC_FUNC int myx_query_add_action(MYX_RESULTSET *resultset, MYX_RS_ACTION
 MYX_PUBLIC_FUNC int myx_query_delete_action(MYX_RESULTSET *resultset, unsigned int action);
 MYX_PUBLIC_FUNC MYX_RS_ACTION_ERRORS * myx_query_apply_actions(MYX_RESULTSET *resultset);
 MYX_PUBLIC_FUNC int myx_query_update_resultset(MYX_RESULTSET *resultset);
+MYX_PUBLIC_FUNC int myx_query_pack_resultset(MYX_RESULTSET *resultset);
 MYX_PUBLIC_FUNC int myx_query_discard_actions(MYX_RESULTSET *resultset);
 MYX_PUBLIC_FUNC int myx_query_free_action_errors(MYX_RS_ACTION_ERRORS *errors);
 
