@@ -19,11 +19,12 @@
 
 #include <stdlib.h>
 #include <stdio.h>
-#include <glib.h>
+//#include <glib.h>
 
 #if defined(__WIN__) || defined(_WIN32) || defined(_WIN64)
   #include <winsock2.h>
   #include <windows.h>
+  #include <io.h>
 #endif
 
 FILE* myx_fopen(const char *filename, const char *mode);
