@@ -377,6 +377,53 @@ char *mask_out_string(char *str, const char open_trigger,
   return str;
 }
 
+
+
+static void __sappend(char **str, int *ressize, int *reslen, const char *sbegin, int count)
+{
+  if (*reslen + count + 1 >= *ressize)
+  {
+    *ressize+= count + 100;
+    *str= g_realloc(*str, *ressize);
+  }
+  strncpy(*str+*reslen, sbegin, count);
+  *reslen+= count;
+  (*str)[*reslen]= 0;
+}
+
+
+
+
+char *str_g_subst(const char *str, const char *search, const char *replace)
+{
+  int ressize, reslen;
+  int replsize= (int)strlen(replace);
+  int searchlen= (int)strlen(search);
+  char *res;
+  const char *bptr, *ptr;
+  
+  g_return_val_if_fail(str != NULL && *str, g_strdup(str));
+  g_return_val_if_fail(search != NULL && *search, g_strdup(str));
+  g_return_val_if_fail(replace != NULL, g_strdup(str));
+
+  ressize= (int)strlen(str)+1;
+  reslen= 0;
+  res= g_malloc(ressize);
+
+  bptr= str;
+
+  while ((ptr= strstr(bptr, search)))
+  {
+    __sappend(&res, &ressize, &reslen, bptr, (int)(ptr-bptr));
+    bptr= ptr+searchlen;
+    __sappend(&res, &ressize, &reslen, replace, replsize);
+  }
+  __sappend(&res, &ressize, &reslen, bptr, (int)strlen(bptr));
+
+  return res;
+}
+
+
 /** 
  * Replaces the search string with the replace string
  * in the given str
@@ -388,15 +435,17 @@ char *mask_out_string(char *str, const char open_trigger,
  **/
 char *str_g_replace(char *str, const char *search, const char *replace)
 {
-  char **splited= g_strsplit(str, search, -1);
-  char *replaced= g_strjoinv(replace, splited);
+  char *res;
 
-  g_strfreev(splited);
+  if (!replace)
+    replace= "";
+
+  res= str_g_subst(str, search, replace);
   g_free(str);
-
-  return replaced;
+  return res;
 }
 
+
 /**
  * Appends the @addon string to the end of
  * @base_str. @base_str is expected to have
@@ -1117,22 +1166,29 @@ static int _get_hardware_info(char **cpu, char **clock, int *cpu_count, unsigned
   {
     char *ptr, *end;
     
-    fgets(line, sizeof(line), proc); // header
-    fgets(line, sizeof(line), proc); // meminfo
+    *mem_kb= 0;
+    while (fgets(line, sizeof(line), proc))
+    {
+        if (strncasecmp(line, "MemTotal:", sizeof("MemTotal:")-1)==0)
+        {
+          char *line_end= line+strlen(line);
+          ptr= strchr(line, ':')+1;
+          while (*ptr && *ptr==' ') ptr++;
+          end= strchr(ptr, ' ');
+          if (end)
+            *end= 0;
+          if (end < line_end)
+            end++;
+          if (strstr(end, "mB") || strstr(end, "MB"))
+            *mem_kb= strtoul(str_trim(ptr), NULL, 10)*1024;
+          else if (strstr(end, "kB") || strstr(end, "KB"))
+            *mem_kb= strtoul(str_trim(ptr), NULL, 10);
+          else
+            *mem_kb= strtoul(str_trim(ptr), NULL, 10)/1024;
+          break;
+        }
+    }
     fclose(proc);
-    
-    ptr= strchr(line, ':')+1;
-    while (*ptr && *ptr==' ') ptr++;
-    end= strchr(ptr, ' ');
-    if (end)
-      *end= 0;
-    
-    if (strstr(end, "mB") || strstr(end, "MB"))
-      *mem_kb= strtoul(str_trim(ptr), NULL, 10)*1024;
-    else if (strstr(end, "kB") || strstr(end, "KB"))
-      *mem_kb= strtoul(str_trim(ptr), NULL, 10);
-    else
-      *mem_kb= strtoul(str_trim(ptr), NULL, 10)/1024;
   }
   return 0;
 }
@@ -1443,19 +1499,6 @@ void strlist_g_append(char ***list, char *value)
 }
 
 
-static void __sappend(char **str, int *ressize, int *reslen, const char *sbegin, int count)
-{
-  if (*reslen + count + 1 >= *ressize)
-  {
-    *ressize+= count + 100;
-    *str= g_realloc(*str, *ressize);
-  }
-  strncpy(*str+*reslen, sbegin, count);
-  *reslen+= count;
-  (*str)[*reslen]= 0;
-}
-
-
 const char *strfindword(const char *str, const char *word)
 {
   const char *ptr;
@@ -1831,7 +1874,7 @@ int myx_mkdir(const char *filename, int mode)
 
 int get_str_index(char **string_list, unsigned int string_list_num, const char *search)
 {
-  unsigned int i= 0, index= -1;
+  unsigned int i= 0;
 
   while (i<string_list_num)
   {
@@ -1841,5 +1884,27 @@ int get_str_index(char **string_list, unsigned int string_list_num, const char *
     i++;
   }
 
-  return i;
+  return -1;
+}
+
+///////////////////////////////////////////////////////////////////////////////
+/** @brief checks if filename really exists
+    @param filename path to file
+    @return 0 if the file doesn't exist else 1
+    \b vva_todo why don't it use access function?
+*//////////////////////////////////////////////////////////////////////////////
+int check_file_exists(const char *filename)
+{
+  FILE *f;
+
+  /*check if we can open the file */    
+  if ((f= myx_fopen(filename, "r")) == NULL)
+  {
+    return 0;
+  }
+  else
+  {
+    fclose(f);
+    return 1;
+  }
 }
