@@ -152,7 +152,12 @@ MYX_SCHEMA_TABLES * myx_get_schema_tables(MYSQL * mysql,
     return NULL;
   }
 
-  if (!(tables_res= mysql_list_tables(mysql, (char*) NULL)))
+  if (!((mysql_full_version_is_later_or_equal_than(mysql, 5, 0, 3)
+         && !myx_mysql_query(mysql, "SHOW FULL TABLES")
+         && (tables_res= mysql_store_result(mysql)))
+        || 
+        (!mysql_full_version_is_later_or_equal_than(mysql, 5, 0, 3)
+         && (tables_res= mysql_list_tables(mysql, (char*) NULL)))))
   {
     g_free(schema_tables);
     restore_old_schema(mysql, old_db);
@@ -667,7 +672,7 @@ MYX_SCHEMA_TABLE_STATUS * myx_get_schema_table_status(MYSQL *mysql,
     @return new allocated header or NULL
 *//////////////////////////////////////////////////////////////////////////////
 
-char *filter_sp_header(char *sp_code)
+static char *filter_sp_header(char *sp_code)
 {
   unsigned int i= 0, len= (unsigned int)strlen(sp_code);
   unsigned int b_count= 0;
