@@ -31,12 +31,16 @@ extern "C"{
 #include <string>
 #include <vector>
 
+#include "keywords.h"
+
 using namespace std;
 
 #define SYSTEM_SCHEME "mysql"
 #define SYSTEM_CATALOG "system"
 #define DEFAULT_CATALOG "def"
 
+static bool is_sql_keyword(const char *str);
+
 const char * default_catalog(const char * schema)
 {
   return (!schema || strcmp(schema,SYSTEM_SCHEME))
@@ -153,8 +157,13 @@ Query_type_description * get_query_type_description(MYX_Q_TYPE query_type)
 #define ISSPACE(c) ((c)==' ' || (c)=='\t' || (c)=='\r' || (c)=='\n')
 
 ///////////////////////////////////////////////////////////////////////////////
-const char * myx_identifier_needs_quotes(const const_string & name)
+const char * myx_cs_identifier_needs_quotes(const const_string & name)
 {
+	if(is_sql_keyword(name.begin()))
+	{
+		return name.begin();
+	}
+
   for (const char * pos= name.begin(); pos!=name.end(); pos++)
   {
     if (!(*pos>='a' && *pos<='z' ||
@@ -173,10 +182,9 @@ const char * myx_identifier_needs_quotes(const const_string & name)
 int myx_identifier_needs_quotes(const char *name)
 {
   const_string n(name,strlen(name));
-  const char * pos= myx_identifier_needs_quotes(n);
-  return (int)(!pos ? 0 : pos - n.begin());
+  const char * pos= myx_cs_identifier_needs_quotes(n);
+  return (int)(!pos ? 0 : 1);
 }
-
 ///////////////////////////////////////////////////////////////////////////////
 void myx_query_execute_direct(MYSQL * mysql, const char * sql,
                               MYX_LIB_ERROR * error_code)
@@ -340,12 +348,11 @@ struct const_name : public const_string
     // always add quotes because otherwise we'd have to check for reserved
     // words (like table, from, to, select etc)
     // it's easier to just quote everything.
-/*    if (!myx_identifier_needs_quotes(src.data()))
+    if (!myx_identifier_needs_quotes(src.data()))
     {
       dst << (const_string&)src;
     }
     else
- */
     {
       dst.append(1,src.quote_symbol);
       for (const char * c= src.begin(); c!=src.end(); c++)
@@ -418,6 +425,7 @@ struct Query : public MYX_QUERY
 
   MYX_Q_TABLE   * get_table_by_alias(const char * alias,
                                      const char * column= 0);
+  MYX_Q_TABLE   * get_first_table();
   MYX_Q_TABLE   * get_table(const char * catalog,
                             const char * schema,
                             const char * table);
@@ -425,6 +433,7 @@ struct Query : public MYX_QUERY
 
   bool contains_tables_without_aliases()const;
   bool try_to_fix_table_aliases(MYSQL * mysql);
+	bool contains_unaliased_table(const char *table_name);
 
   bool can_contain_clause(MYX_Q_CLAUSE_TYPE clause_type);
 
@@ -603,53 +612,6 @@ void replace(std::string * res,
   }
 }
 
-///////////////////////////////////////////////////////////////////////////////
-bool Query::try_to_fix_table_aliases(MYSQL * mysql)
-{
-  const MYX_Q_CLAUSE * select_clause= get_clause(MYX_QCT_SELECT_CLAUSE);
-  if (!select_clause ||
-      get_clause(MYX_QCT_WHERE_CLAUSE) || get_clause(MYX_QCT_GROUP_CLAUSE) ||
-      get_clause(MYX_QCT_HAVING_CLAUSE) || get_clause(MYX_QCT_ORDER_CLAUSE) ||
-      strcmp(select_clause->clause,"*"))
-  {
-    return false;
-  }
-
-  MYX_Q_CLAUSE * from_clause= (MYX_Q_CLAUSE *)get_clause(MYX_QCT_FROM_CLAUSE);
-  if (!from_clause)
-    return true;
-
-  std::string from_clause_txt= " ";
-  from_clause_txt.append(from_clause->clause);
-  std::string fname, old_fname;
-  Type_tables::iterator it;
-  for (it= m_tables.begin(); it!=m_tables.end(); it++)
-  {
-    if (strcmp2(it->alias, "") == 0)
-    {
-      char * new_alias= myx_query_get_table_alias(this, it->name);
-      fname= " ";
-      fname << const_name(mysql,it->name) << CONST_STR(" ");
-      fname.append(new_alias);
-
-      old_fname= " ";
-      old_fname.append(it->fullname);
-
-      const_string cold_fname(old_fname.c_str(),old_fname.length());
-      const_string cfname(fname.c_str(),fname.length());
-
-      replace(&m_sql,cold_fname,cfname);
-      replace(&from_clause_txt,cold_fname,cfname);
-      from_clause->end_index+= (unsigned int)fname.length()-old_fname.length();
-
-      it->alias= new_alias;
-    }
-  }
-  g_free(from_clause->clause);
-  from_clause->clause= g_strstrip(g_strdup(from_clause_txt.c_str()));
-  return true;
-}
-
 ///////////////////////////////////////////////////////////////////////////////
 bool Query::contains_tables_without_aliases() const
 {
@@ -663,6 +625,26 @@ bool Query::contains_tables_without_aliases() const
   }
   return false;
 }
+///////////////////////////////////////////////////////////////////////////////
+bool Query::contains_unaliased_table(const char *table_name)
+{
+  for (Type_tables::iterator it= m_tables.begin(); it!=m_tables.end(); it++)
+	{
+    if((!strcmp3(it->name, table_name)) && ((!it->alias) || (strlen(it->alias) == 0)))
+    {
+      return true;
+    }
+	}
+	return false;
+}
+///////////////////////////////////////////////////////////////////////////////
+MYX_Q_TABLE *Query::get_first_table()
+{
+  if (m_tables.empty())
+    return 0;
+  else
+    return &*m_tables.begin();
+}
 
 ///////////////////////////////////////////////////////////////////////////////
 MYX_Q_TABLE *Query::get_table(const char * catalog,
@@ -673,9 +655,9 @@ MYX_Q_TABLE *Query::get_table(const char * catalog,
     catalog= default_catalog(schema);
   for (Type_tables::iterator it= m_tables.begin(); it!=m_tables.end(); it++)
   {
-    if (! strcmp3(it->catalog, catalog  ) &&
-        ! strcmp3(it->schema,  schema   ) &&
-        ! strcmp3(it->name,    table    ) )
+    if (! strcmp3(it->catalog, catalog) &&
+        ((!schema) || (! strcmp3(it->schema, schema))) &&
+        ! strcmp3(it->name, table))
     {
       return &*it;
     }
@@ -695,7 +677,6 @@ void Query::analyze(MYSQL *mysql, const char *sql)
   query_type= calc_query_type();
 
   s_query_build_clauses(this);
-
   switch (query_type)
   {
   case MYX_QT_SELECT:
@@ -1033,6 +1014,10 @@ MYX_Q_TYPE Query::calc_query_type()
           start_with(sql,       CONST_STR("DESC"))    ? MYX_QT_DESCRIBE :
           start_with(sql,       CONST_STR("EXPLAIN")) ? MYX_QT_EXPLAIN  :
           start_with(sql,       CONST_STR("CALL"))    ? MYX_QT_CALL     :
+          start_with(sql,       CONST_STR("CHECK"))   ? MYX_QT_CHECK    :
+          start_with(sql,       CONST_STR("ANALYZE")) ? MYX_QT_ANALYZE  :
+          start_with(sql,       CONST_STR("REPAIR"))  ? MYX_QT_REPAIR   :
+          start_with(sql,       CONST_STR("OPTIMIZE"))? MYX_QT_OPTIMIZE :
           sql==CONST_STR("")                          ? MYX_QT_EMPTY    :
                                                         MYX_QT_UNKNOWN;
 }
@@ -1323,15 +1308,18 @@ const char * s_get_table_charset(MYSQL * mysql, MYX_Q_TABLE * q_table)
     const char *err;
     
     G_string masked= g_strdup(q.Create_stmt.get_value());
-    mask_quotas(masked);
-    pcre * pcre_exp= pcre_compile("DEFAULT CHARSET=(\\w+)",
-                                  0, &err, &erroff, NULL);
-    if ((sc= pcre_exec(pcre_exp, NULL, masked, (int)strlen(masked),
-                       0, 0, matched, sizeof(matched)/sizeof(*matched)))>=0)
+    if (masked)
     {
-      res= g_strdup(PCRE_sub_string(masked, matched, sc, 1));
+      mask_quotas(masked);
+      pcre * pcre_exp= pcre_compile("DEFAULT CHARSET=(\\w+)",
+                                  0, &err, &erroff, NULL);
+      if ((sc= pcre_exec(pcre_exp, NULL, masked, (int)strlen(masked),
+                         0, 0, matched, sizeof(matched)/sizeof(*matched)))>=0)
+      {
+        res= g_strdup(PCRE_sub_string(masked, matched, sc, 1));
+      }
+      pcre_free(pcre_exp);
     }
-    pcre_free(pcre_exp);
   }
   return res;
 }
@@ -1429,10 +1417,10 @@ MYX_QUERY * query_get_query_columns(MYX_QUERY * query)
 // an identifier, with optional ` or " quotes, but grouping without the quotes
 #define RE_NAME "(?:\\w+|`[^`]+`|\"[^\"]+\")"
 const char * query_column_regexp_mask=
-  "((("RE_NAME")\\.)?(\\*))|"
-  "((("RE_NAME")\\.)+("RE_NAME")(\\s+as\\s+(\\w+))?)|"
-  "((.*)\\s+as\\s+(\\w+))|"
-  "(.*)";
+  "((("RE_NAME")\\.)?(\\*))|" // [X].Y
+  "((("RE_NAME")\\.)+("RE_NAME")(\\s+as\\s+(\\w+))?)|" // [X].Y as Z
+  "((.*)\\s+as\\s+(\\w+))|" // ? as Y
+  "(.*)"; // X
 #undef RE_NAME
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -1491,7 +1479,12 @@ void add_regexp_columns_v_14(Query *query,
   PCRE_sub_string column_alias  (columns, matched, rc, 13);
 
   G_string column_unq= unquote_identifier(g_strdup(column));
-  query_add_table_column(query, NULL, column_unq, column_alias);
+
+  // this column has no table specification, so it should mean that
+  // there's only one table in the query. 
+  // we'll pick the 1st table, which is supposed to be the only one
+
+  query_add_table_column(query, query->get_first_table(), column_unq, column_alias);
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -1501,7 +1494,12 @@ void add_regexp_columns_v_15(Query *query,
 {
   PCRE_sub_string column(columns, matched, rc, 14);
   G_string column_unq= unquote_identifier(g_strdup(column));
-  query_add_table_column(query, NULL, column_unq, column_unq);
+  
+  // this column has no table specification, so it should mean that
+  // there's only one table in the query. 
+  // we'll pick the 1st table, which is supposed to be the only one
+  
+  query_add_table_column(query, query->get_first_table(), column_unq, column_unq);
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -1643,6 +1641,25 @@ bool query_contains_distinct_distinctrow_option(Query *query)
   return false;
 }
 
+///////////////////////////////////////////////////////////////////////////////
+bool query_contains_nested_query(Query *query)
+{
+  for(Query::Type_clauses::iterator it = query->m_clauses.begin();
+      it != query->m_clauses.end(); it++)
+  {
+    MYX_Q_CLAUSE c= *it;
+    if(c.clause_type == MYX_QCT_FROM_CLAUSE)
+    {
+      char *t= g_strdup(c.clause);
+      mask_quotas_and_brackets(t);
+      bool b= (strchr(t, ',') != NULL);
+      g_free(t);
+      return b;
+    }
+  }
+  return false;
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 void build_primary_keys_for_table(Query * query,
                                   MYX_Q_TABLE * q_tbl, int enforce_editable)
@@ -1686,14 +1703,23 @@ void build_primary_keys_for_table(Query * query,
   }
 }
 
-///////////////////////////////////////////////////////////////////////////////
+/**
+ * @brief check whether the resuklt of query can be edited
+ * in QB.
+ *
+ * If the query looks editable this function may modify it
+ * by adding PKs.
+ *
+ */
 unsigned int s_query_is_editable(Query *query, int enforce_editable)
 {
   // if a column contains a '(', indicating it uses a function,
   // do not allow it to be editable. check that here because
   // we dont want it to have PK columns added either
   if (query_contains_bracket_in_a_column(query))
+  {
     return 0;
+  }
   
   // if the query contains DISTINCT or DISTINCTROW option, we can't edit it
   if (query_contains_distinct_distinctrow_option(query) ||
@@ -1702,7 +1728,13 @@ unsigned int s_query_is_editable(Query *query, int enforce_editable)
   {
     return 0;
   }
-  
+
+  // if query contains a nested query we can't edit it
+  if(query_contains_nested_query(query))
+  {
+    return 0;
+  }
+
   //only a query with one table is editable
   if (query->tables_num!=1)
     return 0;
@@ -1812,7 +1844,6 @@ Query * s_query_set_params(Query * query, MYX_STRINGLIST * parameters)
       }
     }
   }
-
   return query;
 }
 
@@ -2089,66 +2120,222 @@ char * myx_query_build_from_clauses(MYX_QUERY *query)
   return sql;
 }
 
-///////////////////////////////////////////////////////////////////////////////
+/**
+ * @brief copy the first utf8 symbol of src to the dst
+ *
+ * @param src a valid utf8 string (@see g_utf8_validate)
+ * @param dst output buffer
+ * @max_size dst size
+ *
+ * @return - size of copied symbol
+ */
+
 size_t copy_utf8_symbol(char * dst, const char * src, size_t max_size)
 {
-  size_t len= strlen(src);
-  if (max_size<len)
-    len= max_size;
-  unsigned char *invalid_chars= 0;
-  for (size_t size= 1; size<max_size; size++)
-  {
-    if (g_utf8_validate(src,(gssize)size,(const gchar**)&invalid_chars))
-    {
-      memcpy(dst,src,size);
-      dst[size]= 0;
-      return size;
-    }
-  }
-  dst[0]= 'x';
-  dst[1]= 0;
-  return 1;
+	char *next= g_utf8_next_char(src);
+	size_t sz= next - src;
+	if(sz < max_size)
+	{
+		memcpy(dst, src, sz);
+		dst[sz] = 0;
+		return sz;
+	}
+	return 0;
 }
 
-///////////////////////////////////////////////////////////////////////////////
-// Returns the alias of the first query-table that is still empty
-// if it is not empty, it returns the alias of the new_tablename
+/**
+ * @brief strip all non-alpha chars from the string
+ *
+ * Strip all non-alpha chars from the string, so for example for identifier '_my_table123'
+ * result will be 'mytable'
+ *
+ * @param table_name string
+ *
+ * @return dynamically allocated string (should be free() -d)
+ */
+
+char *get_canonical_table_name(const char *table_name)
+{
+	const char *eos, *next, *curr;
+
+	if(!g_utf8_validate(table_name, -1, &eos))
+	{
+		return NULL;
+	}
+	
+	char *cname= (char *)g_malloc(((gulong)(eos - table_name + 1)));
+	memset(cname, 0, eos - table_name + 1);
+	char *cnamenext= cname;
+
+	for(curr= table_name, next= g_utf8_next_char(table_name); 
+			curr != eos; 
+			curr= next, next= g_utf8_next_char(next))
+	{
+		if(g_unichar_isalpha(g_utf8_get_char(curr)))
+		{
+			size_t charsize= next - curr;
+			memcpy(cnamenext, curr, charsize);
+			cnamenext+= charsize;
+		}
+	}
+	
+	return cname;
+}
+
+/**
+ * @brief g_tree comparator call-back function
+ */
+static gint is_comp_func (gconstpointer a, gconstpointer b)
+{
+	return g_strcasecmp((char *) a, (char *) b);
+}
+
+/**
+ * @brief checks whether a string is a MySQL keyword (caseinsensitive)
+ *
+ * Checks whether a string is a MySQL keyword (caseinsensitive). MySQL 5.0 keyword set is used.
+ * Warning: this function uses static data and therefore is not thread safe
+ *
+ * @param str string to check
+ */
+
+static bool is_sql_keyword(const char *str)
+{
+	static GTree *tree= 0;
+	
+	if(0==tree)
+	{
+		tree= g_tree_new(is_comp_func);
+		for(size_t i= 0; i < sizeof(symbols)/sizeof(SYMBOL); i++)
+		{
+			g_tree_insert(tree, strdup(symbols[i].name), strdup(symbols[i].name));
+		}
+		for(size_t i= 0; i < sizeof(sql_functions)/sizeof(SYMBOL); i++)
+		{
+			g_tree_insert(tree, strdup(sql_functions[i].name), strdup(sql_functions[i].name));
+		}
+	}
+	
+	return g_tree_lookup(tree, str) != 0;
+}
+
+/**
+ * @brief generates table alias
+ *
+ * returns alias for the new_tablename or empty string if 
+ * usage of table name is preferrable to alias
+ *
+ * @param query query, the new table is to be added to
+ * @param new_tablename table name to be added
+ *
+ * @return alias name string (should be g_free() -d)
+ */
+
 char * myx_query_get_table_alias(Query *query, const char * new_tablename)
 {
-  char alias_project[40];
+	int i= 1;
+	gulong alias_size= 16;
+	char *alias= (char *)g_malloc(alias_size);
+	char *alias_next= alias;
 
-  size_t first_symbol_size= copy_utf8_symbol(alias_project,
-                                             new_tablename,
-                                             sizeof(alias_project)-1);
-  MYX_Q_TABLE *q_table1= query->get_table_by_alias(alias_project);
-  if (!q_table1)
-    return g_strdup(alias_project);
+	const char *new_cname= get_canonical_table_name(new_tablename);
+	MYX_Q_TABLE *q_table;
+	const char *pos1, *pos2;
 
-  const char * pos1, * pos2;
-  for (pos1= q_table1->name, pos2= new_tablename;
-       *pos1 && *pos2 && *pos1==*pos2; pos1++, pos2++);
+  alias_next+= copy_utf8_symbol(alias, new_cname, sizeof(alias)-1);
 
-  size_t max_size2= sizeof(alias_project)-first_symbol_size-1;
-  size_t second_symbol_size= *pos2
-                  ? copy_utf8_symbol(alias_project+first_symbol_size,pos2,
-                                     max_size2)
-                  : copy_utf8_symbol(alias_project+first_symbol_size,
-                                     new_tablename+first_symbol_size,
-                                     max_size2);
+	// try to find an unique alphabetical table name alias
+	do
+	{
+		char *table_cname= 0;
+		if((q_table= query->get_table_by_alias(alias)))
+		{
+			table_cname= get_canonical_table_name(q_table->name);
+		}
+		else if(is_sql_keyword(alias))
+		{
+			table_cname= strcpy((char *)g_malloc(((gulong)strlen(alias)+1)), alias);
+		}
+		else if(query->contains_unaliased_table(alias))
+		{
+			table_cname= strcpy((char *)g_malloc(((gulong)strlen(alias)+1)), alias);
+		}
+		else
+		{
+			goto ret;
+		}
+	
+		for(pos2= new_cname, pos1= table_cname; 
+				*pos1 && *pos2 && *pos1 == *pos2; 
+				pos1++, pos2++);
+		
+		g_free(table_cname);
+		
+		if (*pos2)
+		{
+			if((alias + alias_size - 1) < (g_utf8_next_char(alias_next)))
+			{
+				size_t offs= alias_next - alias;
+				char *alias_old = alias;
+				alias= (char *)memcpy(g_malloc(alias_size<<1), alias, alias_size);
+				g_free(alias_old);
+				alias_size <<= 1;
+				alias_next= alias + offs;
+			}
 
-  MYX_Q_TABLE *q_table2= query->get_table_by_alias(alias_project);
-  if (!q_table2)
-    return g_strdup(alias_project);
+			alias_next+= 
+				copy_utf8_symbol(alias_next, pos2, alias + alias_size - alias_next);
+		}
+	}
+	while(*pos2);
+	
+	// didn't find an unique alphabetical alias
+	
+	if((alias + alias_size - 1) < alias_next+16)
+	{
+		size_t offs= alias_next - alias;
+		char *alias_old = alias;
+		alias= (char *)memcpy(g_malloc(alias_size+16), alias, alias_size);
+		g_free(alias_old);
+		alias_size+= 16;
+		alias_next= alias + offs;
+	}
 
-  for (unsigned int i= 0; i<100; i++)
+	// create number-based alias
+	// this expected to succeed always
+  while(true)
   {
-    sprintf(alias_project+first_symbol_size,"%u",i);
-    MYX_Q_TABLE *q_tablei= query->get_table_by_alias(alias_project);
-    if (!q_tablei)
-      return g_strdup(alias_project);
+    sprintf(alias_next, "%u", i++);
+
+    if(query->get_table_by_alias(alias) || query->get_table(0, 0, alias))
+		{
+			continue;
+		}
+		else
+		{
+      goto ret;
+		}
   }
 
-  return g_strdup(new_tablename);
+	// here it makes decision whether to use alias
+	// or its better to use table name itself w/o an alias
+ret:
+	if(g_utf8_collate(alias, new_tablename) == 0)
+	{
+		return strcpy((char *)g_malloc(1), "");
+	}
+	
+	if((strlen(alias) < strlen(new_tablename)) 
+	|| query->get_table_by_alias(new_tablename)
+	|| query->contains_unaliased_table(new_tablename)
+	|| is_sql_keyword(new_tablename)
+	|| (strlen(new_cname) < strlen(new_tablename)))	// tablename contains weird symbols
+	{
+		return alias;
+	}
+
+	return strcpy((char *)g_malloc(1), "");
+
 }
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -2165,8 +2352,13 @@ struct full_table_name
 
   friend std::string & operator << (std::string & s, const full_table_name & t)
   {
-    return s << t.schema_name << CONST_STR(".") << t.table_name;
+		return s << t.schema_name << CONST_STR(".") << t.table_name;
   }
+
+	size_t table_name_length() const
+	{
+		return table_name.length();
+	}
 };
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -2192,13 +2384,18 @@ struct full_table_name_with_def
     : table(mysql,const_string(schema_name_arg,strlen(schema_name_arg)),
             const_string(table_name_arg,strlen(table_name_arg))),
       default_schema_name(defalut_schema_name_arg,
-                          strlen(defalut_schema_name_arg))
+                          defalut_schema_name_arg?strlen(defalut_schema_name_arg):0)
   {}
 
+	size_t table_name_length() const
+	{
+		return table.table_name_length();
+	}
+
   friend std::string & operator << (std::string & s, 
                                     const full_table_name_with_def & t)
   {
-    return !t.default_schema_name.empty() && 
+    return t.default_schema_name.empty() ||
            t.default_schema_name!=t.table.schema_name
                 ? s << t.table
                 : s << t.table.table_name;
@@ -2224,7 +2421,7 @@ struct aliased_full_table_name_with_def
   friend std::string & operator << (std::string & s, 
                                     const aliased_full_table_name_with_def & t)
   {
-    return s << t.table << CONST_STR(" ") << t.alias;
+		return s << t.table << CONST_STR(" ") << t.alias;
   }
 };
 ///////////////////////////////////////////////////////////////////////////////
@@ -2247,6 +2444,58 @@ struct table_name_with_created_alias : public aliased_full_table_name_with_def
 };
 ///////////////////////////////////////////////////////////////////////////////
 
+bool Query::try_to_fix_table_aliases(MYSQL * mysql)
+{
+  const MYX_Q_CLAUSE * select_clause= get_clause(MYX_QCT_SELECT_CLAUSE);
+  if (!select_clause ||
+      get_clause(MYX_QCT_WHERE_CLAUSE) || get_clause(MYX_QCT_GROUP_CLAUSE) ||
+      get_clause(MYX_QCT_HAVING_CLAUSE) || get_clause(MYX_QCT_ORDER_CLAUSE) ||
+      strcmp(select_clause->clause,"*"))
+  {
+    return false;
+  }
+
+  MYX_Q_CLAUSE * from_clause= (MYX_Q_CLAUSE *)get_clause(MYX_QCT_FROM_CLAUSE);
+  if (!from_clause)
+    return true;
+
+  std::string from_clause_txt= " ";
+  from_clause_txt.append(from_clause->clause);
+  std::string fname, old_fname;
+  Type_tables::iterator it;
+  for (it= m_tables.begin(); it!=m_tables.end(); it++)
+  {
+    if (strcmp2(it->alias, "") == 0)
+    {
+      char * new_alias= myx_query_get_table_alias(this, it->name);
+			
+			aliased_full_table_name_with_def aftn(
+				full_table_name_with_def(mysql, it->schema, it->schema, it->name), 
+				const_string(new_alias, strlen(new_alias)));
+			fname= " ";
+			fname << aftn;
+			
+      //fname << const_name(mysql,it->name) << CONST_STR(" ");
+      //fname.append(new_alias);
+
+      old_fname= " ";
+      old_fname.append(it->fullname);
+
+      const_string cold_fname(old_fname.c_str(),old_fname.length());
+      const_string cfname(fname.c_str(),fname.length());
+
+      replace(&m_sql,cold_fname,cfname);
+      replace(&from_clause_txt,cold_fname,cfname);
+      from_clause->end_index+= (unsigned int)fname.length()-old_fname.length();
+
+      it->alias= new_alias;
+    }
+  }
+  g_free(from_clause->clause);
+  from_clause->clause= g_strstrip(g_strdup(from_clause_txt.c_str()));
+  return true;
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 const char * look_for_last_breakline(const const_string & buffer)
 {
@@ -2301,15 +2550,26 @@ MYX_STRINGLIST * myx_check_all_relationships
   const char * schema= table_ref.table.table.schema_name.data();
   const char * table= table_ref.table.table.table_name.data();
   const char * alias= table_ref.alias.data();
+	
+	if(strlen(alias) == 0)
+	{
+		alias = table;
+	}
 
   MYX_STRINGLIST * rels= 0;
   Query::Type_tables::iterator it;
   for (it= query.m_tables.begin(); it!=query.m_tables.end(); it++)
   {
-    if ((rels= myx_check_table_relationship(mysql, schema,table, alias,
-                                            it->name, it->alias)) ||
+    const char *it_alias = it->alias;
+		if(strlen(it_alias) == 0)
+		{
+			it_alias = it->name;
+		}
+			
+		if ((rels= myx_check_table_relationship(mysql, schema,table, alias,
+                                            it->name, it_alias)) ||
         (rels= myx_check_table_relationship(mysql, schema,
-                                            it->name, it->alias,
+                                            it->name, it_alias,
                                             table, alias)))
     {
       break;
@@ -2345,7 +2605,7 @@ int insert_query(const full_table_name_with_def & table, std::string * res)
 int new_select_query(const full_table_name_with_def & table,
                      std::string * res)
 {
-  Query query;
+	Query query;
   table_name_with_created_alias table_ref(query,table);
   *res << CONST_STR("SELECT * FROM ") << table_ref;
   return (int)res->length();
@@ -2442,12 +2702,12 @@ void query_add_table_to_select(MYSQL *mysql, const const_string & prepared_sql,
             query.query_type == MYX_QT_DELETE && a_type != MYX_QTAT_SELECT &&
             (from_clause= query.get_clause(MYX_QCT_FROM_CLAUSE)))
   {
-    if (query.contains_tables_without_aliases() && 
-        !query.try_to_fix_table_aliases(mysql))
-    {
-      *error= MYX_QC_TABLES_WITHOUT_ALIAS;
-    }
-    else
+    //if (query.contains_tables_without_aliases() && 
+    //    !query.try_to_fix_table_aliases(mysql))
+    //{
+    //  *error= MYX_QC_TABLES_WITHOUT_ALIAS;
+    //}
+    //else
     {
       MYX_STRINGLIST *rels;
       const_string sql(query.m_sql.c_str(),query.m_sql.length());
@@ -2526,7 +2786,7 @@ char * myx_query_add_table_to_sql(MYSQL *mysql, const char *def_schema,
   else
   {
     *pos= (int)prepared_sql.length();
-    return (char*)prepared_sql.data();
+		return (char*)prepared_sql.data();
   }
 }
 
