@@ -144,7 +144,11 @@ MYX_RESULTSET * myx_query_execute(MYSQL *mysql, const char *sql, int enforce_edi
         (resultset->query->query_type != MYX_QT_DESCRIBE) &&
         (resultset->query->query_type != MYX_QT_EXPLAIN) &&
         (resultset->query->query_type != MYX_QT_CALL) &&
-        (resultset->query->query_type != MYX_QT_UNION))
+        (resultset->query->query_type != MYX_QT_UNION) &&
+        (resultset->query->query_type != MYX_QT_CHECK) &&
+        (resultset->query->query_type != MYX_QT_ANALYZE) &&
+        (resultset->query->query_type != MYX_QT_REPAIR) &&
+        (resultset->query->query_type != MYX_QT_OPTIMIZE))
     {
       g_free(resultset);
 
@@ -585,6 +589,9 @@ int myx_query_update_resultset(MYX_RESULTSET *resultset)
     unsigned int c;
     unsigned int j;
 
+    if (row >= resultset->rows_num)
+      continue;
+
     if (resultset->actions)
     {
       // Move actions up if the are below a deleted row
@@ -619,6 +626,26 @@ int myx_query_update_resultset(MYX_RESULTSET *resultset)
 }
 
 
+int myx_query_pack_resultset(MYX_RESULTSET *resultset)
+{
+  unsigned int row;
+  for (row= 0; row < resultset->rows_num; row++)
+  {
+    if (resultset->rows[row].fields == NULL)
+    {
+      if (row < resultset->rows_num-1)
+      {
+        memmove(&resultset->rows[row],
+                &resultset->rows[row+1],
+                (resultset->rows_num-row-1)*sizeof(MYX_RS_ROW));
+      }
+      resultset->rows_num--;
+    }
+  }
+  return 0;
+}
+
+
 int myx_query_free_resultset(MYX_RESULTSET *resultset)
 {
   MYX_RS_COLUMN *current_column;
@@ -817,7 +844,7 @@ static char *prepare_pks(MYX_RESULTSET *rs, MYX_RS_ACTION *action)
   g_assert(rs->columns_num == query->columns_num);
 
   //if this is not an added row, take value from RS
-  if(action->row <= rs->rows_num)
+  if(action->row < rs->rows_num)
   {
     // iterate over list of columns
     for (i= 0; i < query->columns_num; i++)
@@ -880,11 +907,52 @@ static void append_error(MYX_RS_ACTION_ERRORS **errors, MYSQL *mysql,
   
   error= (*errors)->errors + (*errors)->errors_num-1;
   
+  error->level= MYX_QEL_ERROR;
   error->error= myx_mysql_errno(mysql);
-  error->error_text= g_strdup(myx_mysql_error(mysql));
+  error->error_text= myx_mysql_error(mysql); // myx_mysql_error() already strdup()s
   error->action= action;
 }
 
+#if MYSQL_VERSION_ID >= 50000
+static void append_warnings(MYX_RS_ACTION_ERRORS **errors, MYSQL *mysql, 
+                            MYX_RS_ACTION *action)
+{
+  MYX_RS_ACTION_ERROR *error;
+  MYSQL_RES *res;
+  MYSQL_ROW row;
+  int c;
+
+  c= mysql_warning_count(mysql);
+  if (c > 0 && !myx_mysql_query(mysql, "SHOW WARNINGS") &&
+      (res= mysql_use_result(mysql)))
+  {
+    if (!*errors)
+      *errors= g_malloc0(sizeof(MYX_RS_ACTION_ERRORS));
+
+    (*errors)->errors= g_realloc((*errors)->errors,
+                                 sizeof(MYX_RS_ACTION_ERROR)*((*errors)->errors_num+c));
+
+    while ((row= mysql_fetch_row(res)))
+    {
+      error= (*errors)->errors + (*errors)->errors_num++;
+
+      if (strcasecmp(row[0], "Note")==0)
+        error->level= MYX_QEL_NOTE;
+      else
+        error->level= MYX_QEL_WARNING;
+      error->error= atoi(row[1]);
+      error->error_text= g_strdup(row[2]);
+      error->action= action;
+    }
+
+    mysql_free_result(res);
+  }
+}
+#else // mysql < 5.0
+// nothing
+# define append_warnings(a,b,c) do {} while (0)
+#endif // mysql < 5.0
+
 #if 0
 static void append_charset_error(MYX_RS_ACTION_ERRORS **errors,
                                  const char *charset,
@@ -966,10 +1034,10 @@ static int add_same_row_actions_of_type(MYX_RESULTSET *resultset, int i, int add
   int count= 1;
 
   //Prepare insert
-  if (myx_identifier_needs_quotes(action->column->name))
+//  if (myx_identifier_needs_quotes(action->column->name))
     strlist_g_append(&columns_arr, quote_identifier(action->column->name));
-  else
-    strlist_g_append(&columns_arr, g_strdup(action->column->name));
+//  else
+//    strlist_g_append(&columns_arr, g_strdup(action->column->name));
 
   if (action->column->column_type == MYX_RSCT_BLOB && exclude_blobs)
   {
@@ -1005,10 +1073,10 @@ static int add_same_row_actions_of_type(MYX_RESULTSET *resultset, int i, int add
         }
       if (!columns_arr[q])
       {
-        if (myx_identifier_needs_quotes(action->column->name))
+//        if (myx_identifier_needs_quotes(action->column->name))
           strlist_g_append(&columns_arr, quote_identifier(action2->column->name));
-        else
-          strlist_g_append(&columns_arr, g_strdup(action2->column->name));
+//        else
+//          strlist_g_append(&columns_arr, g_strdup(action2->column->name));
         
         if (action2->column->column_type == MYX_RSCT_BLOB && exclude_blobs)
         {
@@ -1116,6 +1184,7 @@ static int do_update_value(MYX_RESULTSET *resultset, const char *pks,
                            MYX_RS_ACTION_ERRORS **errors, char *buffer)
 {
   char *query;
+  int mysql_is_50= mysql_version_is_later_or_equal_than(resultset->mysql,5,0);
 
   prepare_action_value(resultset->mysql, action, buffer);
 
@@ -1144,10 +1213,16 @@ static int do_update_value(MYX_RESULTSET *resultset, const char *pks,
   {
     action->status= MYX_RSAS_FAILED;
     append_error(errors, resultset->mysql, action);
+    
+    if (mysql_is_50)
+      append_warnings(errors, resultset->mysql, action);
   }
   else
   {
     action->status= MYX_RSAS_APPLIED;
+    
+    if (mysql_is_50)
+      append_warnings(errors, resultset->mysql, action);    
   }
   g_free(query);
         
@@ -1170,6 +1245,7 @@ MYX_RS_ACTION_ERRORS *myx_query_apply_actions(MYX_RESULTSET *resultset)
   char *buffer= NULL;
   char already_deleted;
   int mysql_is_41= mysql_version_is_later_or_equal_than(resultset->mysql,4,1);
+  int mysql_is_50= mysql_version_is_later_or_equal_than(resultset->mysql,5,0);
 
   buffer_size= 100;
   // go through all values to make a big enough buffer
@@ -1256,12 +1332,17 @@ MYX_RS_ACTION_ERRORS *myx_query_apply_actions(MYX_RESULTSET *resultset)
         {
           result= MYX_RSAS_FAILED;
           append_error(&errors, resultset->mysql, action);
+          if (mysql_is_50)
+            append_warnings(&errors, resultset->mysql, action);
         }
         else
         {
           unsigned int c;
           MYX_RS_COLUMN *auto_inc_column= NULL;
           
+          if (mysql_is_50)
+            append_warnings(&errors, resultset->mysql, action);
+          
           /* check if the table has an auto-inc column that wasn't mentioned
            in the INSERT. in that case, we will get the value assigned to that
            column and add an action to the resultset with that value to signal
@@ -1403,9 +1484,16 @@ MYX_RS_ACTION_ERRORS *myx_query_apply_actions(MYX_RESULTSET *resultset)
           {
             action->status= MYX_RSAS_FAILED;
             append_error(&errors, resultset->mysql, action);
+            
+            if (mysql_is_50)
+              append_warnings(&errors, resultset->mysql, action);
           }
           else
+          {
             action->status= MYX_RSAS_APPLIED;
+            if (mysql_is_50)
+              append_warnings(&errors, resultset->mysql, action);
+          }
           g_free(query);
         }
         else
@@ -1447,7 +1535,7 @@ MYX_RS_ACTION_ERRORS *myx_query_apply_actions(MYX_RESULTSET *resultset)
       str= NULL;
       for (j= 0; keys_arr[j]; j++)
       {
-        char *tmp= g_strdup_printf("%s`%s`=%s", j>0?", ":"",keys_arr[j], values_arr[j]);
+        char *tmp= g_strdup_printf("%s%s=%s", j>0?", ":"", keys_arr[j], values_arr[j]);
 
         str= str_g_append_and_free(str, tmp);
       }
@@ -1471,10 +1559,16 @@ MYX_RS_ACTION_ERRORS *myx_query_apply_actions(MYX_RESULTSET *resultset)
       {
         action->status= MYX_RSAS_FAILED;
         append_error(&errors, resultset->mysql, action);
+        
+        if (mysql_is_50)
+          append_warnings(&errors, resultset->mysql, action);
       }
       else
       {
         action->status= MYX_RSAS_APPLIED;
+        
+        if (mysql_is_50)
+          append_warnings(&errors, resultset->mysql, action);
       }
       
       // go over all the involved records and update their status
@@ -1491,6 +1585,7 @@ MYX_RS_ACTION_ERRORS *myx_query_apply_actions(MYX_RESULTSET *resultset)
   return errors;
 }
 
+
 int myx_query_discard_actions(MYX_RESULTSET *resultset)
 {
   unsigned int i;
@@ -2106,8 +2201,7 @@ int myx_export_resultset(MYSQL *mysql, MYX_TABLE_EXPORTER_INFO *info,
       }
     }
 
-    // check if this row was deleted
-    if (!skip)
+    if (!skip)     // check if this row was deleted
     {
       MYX_RESULTSET *detail_resultset= NULL;
 
