@@ -37,7 +37,7 @@
 #endif
 
 
-static MYX_GRT_VALUE *unserialize_from_xml(xmlNodePtr node, GHashTable *objects_by_id);
+static MYX_GRT_VALUE *unserialize_from_xml(MYX_GRT *grt, xmlNodePtr node, GHashTable *objects_by_id);
 static xmlNodePtr serialize_to_xml(xmlNodePtr parent, MYX_GRT_VALUE *value, GHashTable *saved_ids);
 
 
@@ -52,6 +52,17 @@ static void default_log(MYX_GRT *grt, int code, const char *msg, const char *det
   printf("%i: %s: %s\n", code, msg, detail?detail:"");
 }
 
+/**
+ **************************************************************************** 
+ * @brief Returns the version of the GRT environment
+ *
+ * @return a const char with the version information
+ ****************************************************************************/
+const char * myx_grt_version()
+{
+  return GRT_VERSION;
+}
+
 
 /**
  **************************************************************************** 
@@ -67,16 +78,20 @@ static void default_log(MYX_GRT *grt, int code, const char *msg, const char *det
  *
  * @return A newly created GRT.
  ****************************************************************************/
-MYX_GRT *myx_grt_initialize()
+MYX_GRT *myx_grt_initialize(int options)
 {
   MYX_GRT *grt= g_new0(MYX_GRT, 1);
 
+  g_static_rec_mutex_init(&grt->global_mutex);
+
+  grt->options= options;
+  
   grt->print= default_print;
   grt->print_data= NULL;
   
   grt->logfunc= default_log;
   
-  grt->root= myx_grt_dict_new(NULL);
+  grt->root= myx_grt_dict_new(NULL, NULL);
 
   grt->struct_icon_cache= g_hash_table_new_full(g_str_hash, g_str_equal,
                                                 g_free, g_free);
@@ -124,8 +139,56 @@ void myx_grt_finalize(MYX_GRT *grt)
  ****************************************************************************/
 void myx_grt_set_output_callback(MYX_GRT *grt, void *user_data, MYX_GRT_PRINT_CALLBACK process_output_func)
 {
+  GRT_ENTER(grt);
+
   grt->print= process_output_func;
   grt->print_data= user_data;
+  
+  GRT_LEAVE(grt);
+}
+
+/**
+ ****************************************************************************
+ * @brief Sets the function to be used to process messages
+ * 
+ *  Sets the function that is used when GRT needs to process messages.
+ *
+ * @param grt      the GRT environment
+ * @param user_data a pointer to be passed to the callback function. 
+ * @param process_output_func the callback function which should take 
+ *        the text and the userdata pointer as arguments
+ ****************************************************************************/
+void myx_grt_set_message_callback(MYX_GRT *grt, void *user_data, 
+                                  MYX_GRT_MESSAGE_CALLBACK process_messages_func)
+{
+  GRT_ENTER(grt);
+
+  grt->process_messages= process_messages_func;
+  grt->process_messages_data= user_data;
+  
+  GRT_LEAVE(grt);
+}
+
+/**
+ ****************************************************************************
+ * @brief Sets the function to be used to process messages
+ * 
+ *  Sets the function that is used when GRT needs to process messages.
+ *
+ * @param grt      the GRT environment
+ * @param user_data a pointer to be passed to the callback function. 
+ * @param process_output_func the callback function which should take 
+ *        the text and the userdata pointer as arguments
+ ****************************************************************************/
+void myx_grt_set_input_callback(MYX_GRT *grt, void *user_data,
+                                MYX_GRT_INPUT_CALLBACK process_input_func)
+{
+  GRT_ENTER(grt);
+
+  grt->process_input= process_input_func;
+  grt->process_input_data= user_data;
+  
+  GRT_LEAVE(grt);
 }
 
 
@@ -141,7 +204,9 @@ void myx_grt_set_output_callback(MYX_GRT *grt, void *user_data, MYX_GRT_PRINT_CA
  ****************************************************************************/
 void myx_grt_module_set_log_callback(MYX_GRT *grt, MYX_GRT_LOG_CALLBACK log_func)
 {
+  GRT_ENTER(grt);
   grt->logfunc= log_func;
+  GRT_LEAVE(grt);
 }
 
 
@@ -164,12 +229,14 @@ MYX_GRT_ERROR myx_grt_register_module_loader(MYX_GRT *grt, MYX_GRT_MODULE_LOADER
   g_return_val_if_fail(grt != NULL, MYX_GRT_INTERNAL_ERROR);
   g_return_val_if_fail(loader != NULL, MYX_GRT_INTERNAL_ERROR);
 
+  GRT_ENTER(grt);
+
   grt->loaders_num++;
   grt->loaders= g_realloc(grt->loaders, sizeof(MYX_GRT_MODULE_LOADER*)*grt->loaders_num);
 
   grt->loaders[grt->loaders_num-1]= loader;
 
-  return MYX_GRT_NO_ERROR;
+  GRT_RETURN(grt, MYX_GRT_NO_ERROR, MYX_GRT_ERROR);
 }
 
 /**
@@ -186,10 +253,12 @@ MYX_GRT_STRUCTS * myx_grt_structs_get(MYX_GRT *grt)
 
   g_return_val_if_fail(grt != NULL, NULL);
 
+  GRT_ENTER(grt);
+
   structs->structs_num= grt->structs_num;
   structs->structs= g_memdup(grt->structs, sizeof(MYX_GRT_STRUCT)*structs->structs_num);
 
-  return structs;
+  GRT_RETURN(grt, structs, MYX_GRT_STRUCTS*);
 }
 
 MYX_STRINGLIST * myx_grt_struct_packages(MYX_GRT *grt)
@@ -198,6 +267,8 @@ MYX_STRINGLIST * myx_grt_struct_packages(MYX_GRT *grt)
 
   g_return_val_if_fail(grt != NULL, NULL);
 
+  GRT_ENTER(grt);
+  
   {
     MYX_STRINGLIST *packages= g_new0(MYX_STRINGLIST, 1);
 
@@ -231,32 +302,39 @@ MYX_STRINGLIST * myx_grt_struct_packages(MYX_GRT *grt)
       g_free(str_pak);
     }
 
-    return packages;
+    GRT_RETURN(grt, packages, MYX_STRINGLIST*);
   }
 }
 
 int myx_grt_package_count(MYX_GRT *grt)
 {
   unsigned int c;
-  MYX_STRINGLIST *packages= myx_grt_struct_packages(grt);
+  MYX_STRINGLIST *packages;
+  GRT_ENTER(grt);
+  
+  packages= myx_grt_struct_packages(grt);
 
   c= packages->strings_num;
 
   myx_free_stringlist(packages);
 
-  return c;
+  GRT_RETURN(grt, c, int);
 }
 
 char * myx_grt_package_by_index(MYX_GRT *grt, unsigned int index)
 {
   char *s;
-  MYX_STRINGLIST *packages= myx_grt_struct_packages(grt);
+  MYX_STRINGLIST *packages;
+  
+  GRT_ENTER(grt);
+
+  packages= myx_grt_struct_packages(grt);
 
   s= g_strdup(packages->strings[index]);
 
   myx_free_stringlist(packages);
 
-  return s;
+  GRT_RETURN(grt, s, char*);
 }
 
 /**
@@ -270,8 +348,10 @@ char * myx_grt_package_by_index(MYX_GRT *grt, unsigned int index)
 int myx_grt_struct_get_count(MYX_GRT *grt)
 {
   g_return_val_if_fail(grt != NULL, -1);
+  
+  GRT_ENTER(grt);
 
-  return grt->structs_num;
+  GRT_RETURN(grt, grt->structs_num, int);
 }
 
 /**
@@ -289,6 +369,8 @@ int myx_grt_struct_get_child_count(MYX_GRT *grt, const char *struct_name)
   unsigned int i, c= 0;
 
   g_return_val_if_fail(grt != NULL, -1);
+  
+  GRT_ENTER(grt);
 
   for (i= 0; i<grt->structs_num; i++)
   {
@@ -300,7 +382,7 @@ int myx_grt_struct_get_child_count(MYX_GRT *grt, const char *struct_name)
     }
   }
 
-  return c;
+  GRT_RETURN(grt, c, int);
 }
 
 int myx_grt_package_struct_count(MYX_GRT *grt, const char *package_name)
@@ -310,6 +392,8 @@ int myx_grt_package_struct_count(MYX_GRT *grt, const char *package_name)
   g_return_val_if_fail(grt != NULL, 0);
   g_return_val_if_fail(package_name != NULL, 0);
 
+  GRT_ENTER(grt);
+  
   for (i= 0; i<grt->structs_num; i++)
   {
     MYX_GRT_STRUCT *str= grt->structs + i;   
@@ -331,7 +415,7 @@ int myx_grt_package_struct_count(MYX_GRT *grt, const char *package_name)
     }
   }
 
-  return c;
+  GRT_RETURN(grt, c, int);
 }
 
 /**
@@ -346,9 +430,12 @@ int myx_grt_package_struct_count(MYX_GRT *grt, const char *package_name)
 MYX_GRT_STRUCT * myx_grt_struct_get_by_index(MYX_GRT *grt, unsigned int index)
 {
   g_return_val_if_fail(grt != NULL, NULL);
-  g_return_val_if_fail(index < grt->structs_num, NULL);
+  
+  GRT_ENTER(grt);
 
-  return grt->structs + index;
+  GRT_RETURN_VAL_IF_FAIL(grt, index < grt->structs_num, NULL);
+
+  GRT_RETURN(grt, grt->structs + index, MYX_GRT_STRUCT*);
 }
 
 /**
@@ -367,6 +454,8 @@ MYX_GRT_STRUCT * myx_grt_struct_get_child_by_index(MYX_GRT *grt, const char *str
   unsigned int i, c= 0;
 
   g_return_val_if_fail(grt != NULL, NULL);
+  
+  GRT_ENTER(grt);
 
   for (i= 0; i<grt->structs_num; i++)
   {
@@ -375,13 +464,13 @@ MYX_GRT_STRUCT * myx_grt_struct_get_child_by_index(MYX_GRT *grt, const char *str
     if (strcmp3(str->parent_struct_name, struct_name) == 0)
     {
       if (c == index)
-        return str;
+        GRT_RETURN(grt, str, MYX_GRT_STRUCT*);
 
       c++;
     }
   }
 
-  return NULL;
+  GRT_RETURN(grt, NULL, MYX_GRT_STRUCT*);
 }
 
 MYX_GRT_STRUCT * myx_grt_package_struct_by_index(MYX_GRT *grt, const char *package_name, unsigned int index)
@@ -390,6 +479,8 @@ MYX_GRT_STRUCT * myx_grt_package_struct_by_index(MYX_GRT *grt, const char *packa
 
   g_return_val_if_fail(grt != NULL, NULL);
 
+  GRT_ENTER(grt);
+  
   for (i= 0; i<grt->structs_num; i++)
   {
     MYX_GRT_STRUCT *str= grt->structs + i;
@@ -404,7 +495,7 @@ MYX_GRT_STRUCT * myx_grt_package_struct_by_index(MYX_GRT *grt, const char *packa
         if (c == index)
         {
           g_free(begin_str);
-          return str;
+          GRT_RETURN(grt, str, MYX_GRT_STRUCT*);
         }
 
         c++;
@@ -417,14 +508,14 @@ MYX_GRT_STRUCT * myx_grt_package_struct_by_index(MYX_GRT *grt, const char *packa
       if (sub_str_count(".", str->name) == 0)
       {
         if (c == index)
-          return str;
-
+          GRT_RETURN(grt, str, MYX_GRT_STRUCT*);
+        
         c++;
       }
     }
   }
 
-  return NULL;
+  GRT_RETURN(grt, NULL, MYX_GRT_STRUCT*);
 }
 
 
@@ -440,7 +531,9 @@ MYX_GRT_VALUE *myx_grt_get_root(MYX_GRT *grt)
 {
   g_return_val_if_fail(grt != NULL, NULL);
 
-  return grt->root;
+  GRT_ENTER(grt);
+  
+  GRT_RETURN(grt, grt->root, MYX_GRT_VALUE*);
 }
 
 
@@ -465,8 +558,10 @@ MYX_GRT_ERROR myx_grt_set_root(MYX_GRT *grt, MYX_GRT_VALUE *new_root)
   g_return_val_if_fail(grt != NULL, MYX_GRT_INTERNAL_ERROR);
   g_return_val_if_fail(new_root != NULL, MYX_GRT_INTERNAL_ERROR);
 
+  GRT_ENTER(grt);
+  
   if (myx_grt_value_get_type(new_root) != MYX_DICT_VALUE)
-    return MYX_GRT_BAD_VALUE;
+    GRT_RETURN(grt, MYX_GRT_BAD_VALUE, MYX_GRT_ERROR);
 
   myx_grt_reference_cache_clear(grt);
   myx_grt_value_release(grt->root);
@@ -476,7 +571,7 @@ MYX_GRT_ERROR myx_grt_set_root(MYX_GRT *grt, MYX_GRT_VALUE *new_root)
   myx_grt_value_retain(new_root);
   myx_grt_reference_cache_rescan(grt);
   
-  return MYX_GRT_NO_ERROR;
+  GRT_RETURN(grt, MYX_GRT_NO_ERROR, MYX_GRT_ERROR);
 }
 
 
@@ -493,7 +588,7 @@ MYX_GRT_ERROR myx_grt_set_root(MYX_GRT *grt, MYX_GRT_VALUE *new_root)
  *
  * @return NULL on error and the value if the file could be correctly loaded.
  ****************************************************************************/
-MYX_GRT_VALUE *myx_grt_retrieve_from_file(const char *filename)
+MYX_GRT_VALUE *myx_grt_retrieve_from_file(MYX_GRT *grt, const char *filename)
 {
   xmlDocPtr doc;
   xmlNodePtr root;
@@ -510,7 +605,7 @@ MYX_GRT_VALUE *myx_grt_retrieve_from_file(const char *filename)
     if (root)
     {
       GHashTable *objects_by_id= g_hash_table_new(g_str_hash, g_str_equal);
-      value= unserialize_from_xml(root, objects_by_id);
+      value= unserialize_from_xml(grt, root, objects_by_id);
       g_hash_table_destroy(objects_by_id);
     }
   }
@@ -581,16 +676,19 @@ MYX_GRT_ERROR myx_grt_store_to_file(MYX_GRT_VALUE *value, const char *filename)
 int myx_grt_printf(MYX_GRT *grt, const char *fmt, ...)
 {
   char *tmp;
-
+  
   va_list args;
   va_start(args, fmt);
+
+  GRT_ENTER(grt);
   
   tmp= g_strdup_vprintf(fmt, args);
   va_end(args);
   
   MYX_PRINT(grt, tmp);
   g_free(tmp);
-  return 0;
+  
+  GRT_RETURN(grt, 0, int);
 }
 
 
@@ -601,7 +699,7 @@ static xmlNodePtr serialize_to_xml(xmlNodePtr parent, MYX_GRT_VALUE *value,
   unsigned int i;
   char buffer[100];
   const char *id;
-  xmlNodePtr node;
+  xmlNodePtr node= NULL;
 
   switch (value->type)
   {
@@ -668,8 +766,11 @@ static xmlNodePtr serialize_to_xml(xmlNodePtr parent, MYX_GRT_VALUE *value,
     {
       xmlNodePtr child;
       
-      child= serialize_to_xml(node, value->value.d->items[i].value, saved_ids);
-      xmlNewProp(child, "key", value->value.d->items[i].key);
+      if (value->value.d->items[i].value)
+      {
+        child= serialize_to_xml(node, value->value.d->items[i].value, saved_ids);
+        xmlNewProp(child, "key", value->value.d->items[i].key);
+      }
     }
     break;
   case MYX_ANY_VALUE:
@@ -679,7 +780,7 @@ static xmlNodePtr serialize_to_xml(xmlNodePtr parent, MYX_GRT_VALUE *value,
 }
 
 
-static MYX_GRT_VALUE *unserialize_from_xml(xmlNodePtr node, GHashTable *objects_by_id)
+static MYX_GRT_VALUE *unserialize_from_xml(MYX_GRT *grt, xmlNodePtr node, GHashTable *objects_by_id)
 {
   xmlChar *str;
   xmlChar *node_type= xmlGetProp(node, "type");
@@ -753,7 +854,7 @@ static MYX_GRT_VALUE *unserialize_from_xml(xmlNodePtr node, GHashTable *objects_
       {
         xmlChar *struct_name= xmlGetProp(node, "struct-name");
 
-        value= myx_grt_dict_new(struct_name);
+        value= myx_grt_dict_new(grt, struct_name);
 
         xmlFree(struct_name);
       }
@@ -767,7 +868,7 @@ static MYX_GRT_VALUE *unserialize_from_xml(xmlNodePtr node, GHashTable *objects_
         {
           xmlChar *key= xmlGetProp(child, "key");
 
-          sub_value= unserialize_from_xml(child, objects_by_id);
+          sub_value= unserialize_from_xml(grt, child, objects_by_id);
           if (sub_value)
           {
             myx_grt_dict_item_set_value(value, key, sub_value);
@@ -822,7 +923,7 @@ static MYX_GRT_VALUE *unserialize_from_xml(xmlNodePtr node, GHashTable *objects_
 
         if (child->type == XML_ELEMENT_NODE && strcmp(child->name, "value")==0)
         {
-          sub_value= unserialize_from_xml(child, objects_by_id);
+          sub_value= unserialize_from_xml(grt, child, objects_by_id);
           if (sub_value)
           {
             myx_grt_list_item_add(value, sub_value);
@@ -950,6 +1051,8 @@ static MYX_GRT_VALUE *unserialize_from_xml_global_object(MYX_GRT *grt, xmlNodePt
     return NULL;
   }
 
+  GRT_ENTER(grt);
+  
   // check if we have a global object
   if ((strcmp(node_type, "dict")==0) || (strcmp(node_type, "list")==0))
   {
@@ -957,13 +1060,13 @@ static MYX_GRT_VALUE *unserialize_from_xml_global_object(MYX_GRT *grt, xmlNodePt
 
     if (global_object_path)
     {
-      value= myx_grt_value_get_by_path(myx_grt_get_root(grt), global_object_path);
+      value= myx_grt_dict_item_get_by_path(myx_grt_get_root(grt), global_object_path);
       xmlFree(global_object_path);
-      return value;
+      GRT_RETURN(grt, value, MYX_GRT_VALUE*);
     }
   }
 
-  return unserialize_from_xml(node, objects_by_id);
+  GRT_RETURN(grt, unserialize_from_xml(grt, node, objects_by_id), MYX_GRT_VALUE*);
 }
 
 /**
@@ -1022,7 +1125,7 @@ char *myx_grt_value_to_xml(MYX_GRT_VALUE *value)
  * 
  * @see myx_grt_value_to_xml
  *****************************************************************************/
-MYX_GRT_VALUE *myx_grt_value_from_xml(const char *str, size_t size)
+MYX_GRT_VALUE *myx_grt_value_from_xml(MYX_GRT *grt, const char *str, size_t size)
 {
   xmlDocPtr doc= xmlParseMemory(str, (int)size);
   xmlNodePtr root;
@@ -1047,7 +1150,7 @@ MYX_GRT_VALUE *myx_grt_value_from_xml(const char *str, size_t size)
   if (root)
   {
     GHashTable *objects_by_id= g_hash_table_new(g_str_hash, g_str_equal);
-    value= unserialize_from_xml(root, objects_by_id);
+    value= unserialize_from_xml(grt, root, objects_by_id);
     g_hash_table_destroy(objects_by_id);
   }
   else
@@ -1085,6 +1188,8 @@ MYX_GRT_VALUE *myx_grt_value_from_xml_global_object(MYX_GRT *grt, const char *st
     return NULL;
   }
 
+  GRT_ENTER(grt);
+  
   root= xmlDocGetRootElement(doc);
 
   //check if we have a <data> rootnode
@@ -1106,7 +1211,7 @@ MYX_GRT_VALUE *myx_grt_value_from_xml_global_object(MYX_GRT *grt, const char *st
 
   xmlFreeDoc(doc);
 
-  return value;
+  GRT_RETURN(grt, value, MYX_GRT_VALUE*);
 }
 
 /**
@@ -1151,6 +1256,87 @@ char *myx_grt_value_to_xml_global_object(const char *objectPath, MYX_GRT_VALUE *
 }
 
 
+/** 
+ ****************************************************************************
+ * @brief Converts messages that are stored in a GRT value to a GRT 
+ * message list
+ *
+ * @param msgs_list  the GRT list value holding the messages
+ *
+ * @return Converted messages or NULL. The returned struct must be freed 
+ * with myx_grt_messages_free()
+ *****************************************************************************/
+MYX_GRT_MSGS * myx_grt_messages_convert(MYX_GRT_VALUE *msgs_list)
+{
+  MYX_GRT_MSGS *msgs= NULL;
+
+  if (msgs_list)
+  {
+    unsigned int i;
+    msgs= g_new0(MYX_GRT_MSGS, 1);
+    msgs->msgs_num= myx_grt_list_item_count(msgs_list);
+    msgs->msgs= g_new0(MYX_GRT_MSG, msgs->msgs_num);
+
+    for (i= 0; i < msgs->msgs_num; i++)
+    {
+      MYX_GRT_MSG *msg= msgs->msgs + i;
+      MYX_GRT_VALUE *msg_value= myx_grt_list_item_get(msgs_list, i);
+      MYX_GRT_VALUE *msg_detail_value= myx_grt_dict_item_get_value(msg_value, "details");
+
+      msg->msg= g_strdup(myx_grt_dict_item_get_as_string(msg_value, "msg"));
+      msg->msg_type= myx_grt_dict_item_get_as_int(msg_value, "msgType");
+      msg->progress= myx_grt_dict_item_get_as_int(msg_value, "progress");
+
+      if (msg_detail_value)
+      {
+        int count= myx_grt_list_item_count(msg_detail_value);
+
+        if (count > 0)
+        {
+          int j;
+
+          msg->msg_detail= g_new0(MYX_STRINGLIST, 1);
+          msg->msg_detail->strings_num= count;
+          msg->msg_detail->strings= g_malloc0(sizeof(char *) * count);
+
+          for (j= 0; j < count; j++)
+          {
+            msg->msg_detail->strings[j]= g_strdup(myx_grt_list_item_get_as_string(msg_detail_value, j));
+          }
+        }
+      }
+    }
+  }
+
+  return msgs;
+}
+
+
+/** 
+ ****************************************************************************
+ * @brief 
+ *
+ * @param msgs messages to be freed
+ *****************************************************************************/
+void myx_grt_messages_free(MYX_GRT_MSGS *msgs)
+{
+  unsigned int i;
+  
+  if (msgs)
+  {
+    for (i= 0; i < msgs->msgs_num; i++)
+    {
+      g_free(msgs->msgs[i].msg);
+      if (msgs->msgs[i].msg_detail)
+        myx_free_stringlist(msgs->msgs[i].msg_detail);
+    }
+    g_free(msgs->msgs);
+    g_free(msgs);
+  }
+}
+
+
+
 /** 
  ****************************************************************************
  * @brief Add a message to the GRT message queue.
@@ -1161,13 +1347,16 @@ char *myx_grt_value_to_xml_global_object(const char *objectPath, MYX_GRT_VALUE *
  * @param details  optional string list for details of the message
  * @param copy_details if 1, the function will make a copy of the details 
  *              parameter
+ * @param progress the current progress value (1 ... 100) or -1 
  *
  *****************************************************************************/
-void myx_grt_add_msg(MYX_GRT *grt, int msg_type, const char *message, MYX_STRINGLIST *details,
-                     int copy_details)
+void myx_grt_messages_stack_add(MYX_GRT *grt, int msg_type, const char *message, MYX_STRINGLIST *details,
+                     int copy_details, int progress)
 {
   MYX_GRT_MSG *msg;
   
+  GRT_ENTER(grt);
+  
   if (!grt->msgs)
     grt->msgs= g_new0(MYX_GRT_MSGS, 1);
 
@@ -1188,12 +1377,48 @@ void myx_grt_add_msg(MYX_GRT *grt, int msg_type, const char *message, MYX_STRING
   }
   else
     msg->msg_detail= details;
+  
+  GRT_LEAVE(grt);
 }
 
+void myx_grt_messages_stack_add_message(MYX_GRT *grt, const char *message, 
+                                        MYX_STRINGLIST *details, int copy_details, ...)
+{
+  char *msg;
+  va_list args;
+
+  va_start(args, copy_details);
+
+  msg= g_strdup_vprintf(message, args);
+
+  va_end(args);
+
+
+  myx_grt_messages_stack_add(grt, 0, msg, details, copy_details, 0);
+
+  myx_grt_messages_stack_flush(grt, 0);
+}
+
+void myx_grt_messages_stack_add_error(MYX_GRT *grt, const char *message, 
+                                      MYX_STRINGLIST *details, int copy_details, ...)
+{
+  char *msg;
+  va_list args;
+
+  va_start(args, copy_details);
+
+  msg= g_strdup_vprintf(message, args);
+
+  va_end(args);
+
+  myx_grt_messages_stack_add(grt, 1, msg, details, copy_details, 0);
+
+  myx_grt_messages_stack_flush(grt, 0);
+}
 
 /** 
  ****************************************************************************
- * @brief Fetches messages from the GRT.
+ * @brief Processes the messages stored on the GRT message stack.
  *
  * @param grt  
  * @param count  number of messages to fetch. 0 will return all messages.
@@ -1201,18 +1426,17 @@ void myx_grt_add_msg(MYX_GRT *grt, int msg_type, const char *message, MYX_STRING
  * @return Fetched messages or NULL, if there's none. The returned struct
  * must be freed with myx_grt_free_msgs()
  *****************************************************************************/
-MYX_GRT_MSGS * myx_grt_get_msgs(MYX_GRT *grt, unsigned int count)
+void myx_grt_messages_stack_flush(MYX_GRT *grt, unsigned int count)
 {
   //unsigned int i;
   MYX_GRT_MSGS *msgs;
   
-  // fetch messages from modules
-  myx_grt_fetch_module_messages(grt);
+  GRT_ENTER(grt);
   
   // return what was requested
 
   if (!grt->msgs)
-    return NULL;
+    GRT_LEAVE(grt);
   
   if (count == 0)
   {
@@ -1237,29 +1461,15 @@ MYX_GRT_MSGS * myx_grt_get_msgs(MYX_GRT *grt, unsigned int count)
     grt->msgs->msgs_num-= count;
   }
 
-  return msgs;
+  // call the callback function to process the messages
+  grt->process_messages(msgs, grt->process_messages_data);
+
+  myx_grt_messages_free(msgs);
+
+  GRT_LEAVE(grt);
 }
 
 
-/** 
- ****************************************************************************
- * @brief 
- *
- * @param msgs messages to be freed
- *****************************************************************************/
-void myx_grt_free_msgs(MYX_GRT_MSGS *msgs)
-{
-  unsigned int i;
-  
-  for (i= 0; i < msgs->msgs_num; i++)
-  {
-    g_free(msgs->msgs[i].msg);
-    if (msgs->msgs[i].msg_detail)
-      myx_free_stringlist(msgs->msgs[i].msg_detail);
-  }
-  g_free(msgs->msgs);
-  g_free(msgs);
-}
 
 /** 
  ****************************************************************************
@@ -1269,20 +1479,28 @@ void myx_grt_free_msgs(MYX_GRT_MSGS *msgs)
  *****************************************************************************/
 void myx_grt_reference_cache_clear(MYX_GRT *grt)
 {
+  GRT_ENTER(grt);
+  
   // free current cache
   g_hash_table_destroy(grt->reference_cache);
 
   // reallocate cache
   grt->reference_cache= g_hash_table_new_full(g_str_hash, g_str_equal,
                                                 g_free, NULL);
+  
+  GRT_LEAVE(grt);
 }
 
 
 static void grt_rescan_dict(MYX_GRT *grt, MYX_GRT_VALUE *dict, const char *object_path)
 {
   unsigned int i;
-  const char *struct_name= myx_grt_dict_struct_get_name(dict);
+  const char *struct_name;
 
+  GRT_ENTER(grt);
+  
+  struct_name= myx_grt_dict_struct_get_name(dict);
+  
   // if this dict inherits from GrtObject, cache its _id if it not empty
   // and set the object path
   if (struct_name && myx_grt_struct_check_inherits_from(grt, struct_name, "GrtObject"))
@@ -1328,6 +1546,8 @@ static void grt_rescan_dict(MYX_GRT *grt, MYX_GRT_VALUE *dict, const char *objec
         }
       }
   }
+
+  GRT_LEAVE(grt);
 }
 
 /** 
@@ -1338,8 +1558,10 @@ static void grt_rescan_dict(MYX_GRT *grt, MYX_GRT_VALUE *dict, const char *objec
  *****************************************************************************/
 void myx_grt_reference_cache_rescan(MYX_GRT *grt)
 {
+  GRT_ENTER(grt);
   // start scan
   grt_rescan_dict(grt, myx_grt_get_root(grt), "");
+  GRT_LEAVE(grt);
 }
 
 /** 
@@ -1353,23 +1575,28 @@ void myx_grt_reference_cache_rescan(MYX_GRT *grt)
  *****************************************************************************/
 MYX_GRT_VALUE * myx_grt_reference_cache_lookup(MYX_GRT *grt, const char *ref_id)
 {
+  MYX_GRT_VALUE *retval= NULL;
+  
+  GRT_ENTER(grt);
   if(ref_id)
   {
     // try to find the value in the cache
     MYX_GRT_VALUE *value= g_hash_table_lookup(grt->reference_cache, ref_id);
 
     if (value)
-      return value;
+      retval= value;
     else
     {
       // if it is not found, rescan and search again
       myx_grt_reference_cache_rescan(grt);
 
-      return g_hash_table_lookup(grt->reference_cache, ref_id);
+      retval= g_hash_table_lookup(grt->reference_cache, ref_id);
     }
   }
   else
-    return NULL;
+    retval= NULL;
+  
+  GRT_RETURN(grt, retval, MYX_GRT_VALUE*);
 }
 
 
@@ -1400,3 +1627,46 @@ char * myx_grt_get_guid()
   }
 #endif
 }
+
+
+
+const char *myx_grt_error_string(MYX_GRT_ERROR error)
+{
+  switch (error)
+  {
+  case MYX_GRT_NO_ERROR:
+    return "Success";
+  case MYX_GRT_INTERNAL_ERROR:
+    return "Internal error";
+  case MYX_GRT_BAD_PATH:
+    return "Invalid path";
+  case MYX_GRT_CANT_OPEN_FILE:
+    return "Cannot open file";
+  case MYX_GRT_BAD_FUNCTION:
+    return "Invalid function";
+  case MYX_GRT_DUPLICATE_ENTRY:
+    return "Insertion of duplicate name";
+  case MYX_GRT_BAD_VALUE:
+    return "Invalid value";
+  case MYX_GRT_BAD_DATA:
+    return "Invalid data";
+  case MYX_GRT_VALIDATION_ERROR:
+    return "Validation error";
+  case MYX_GRT_FUNCTION_CALL_ERROR:
+    return "Error calling function";
+  case MYX_GRT_MODULE_INIT_ERROR:
+    return "Error initializing module";
+  case MYX_GRT_BAD_MODULE:
+    return "Invalid module";
+  case MYX_GRT_UNKNOWN_MODULE_TYPE:
+    return "Unknown module type";
+  case MYX_GRT_JAVA_NOT_FOUND:
+    return "Java JRE not found";
+  case MYX_GRT_JAVA_REGISTRY_CORRUPTED:
+    return "Java registry corrupted";
+  case MYX_GRT_JAVA_JRE_CANNOT_BE_LOADED:
+    return "Java JRE cannot be loaded";
+  default:
+    return "Unknown error";
+  }
+}
