@@ -63,6 +63,8 @@ MYX_GRT_MODULE_LOADER *myx_lua_init_loader(MYX_GRT *grt, MYX_GRT_ERROR *error)
   static char *file_extensions[]= {
     ".lua"
   };
+  
+  GRT_ENTER(grt);
 
   *error= MYX_GRT_NO_ERROR;
 
@@ -102,9 +104,34 @@ MYX_GRT_MODULE_LOADER *myx_lua_init_loader(MYX_GRT *grt, MYX_GRT_ERROR *error)
   lua_register(priv->lua, "logwarning", l_log_warning);
   lua_register(priv->lua, "logmessage", l_log_message);
 
+  if (check_file_exists("lua" MYX_PATH_SEPARATOR_STR "_library.lua"))
+  {
+    int status;
+
+    // load the _library
+    status= luaL_loadfile(priv->lua, "lua" MYX_PATH_SEPARATOR_STR "_library.lua");
+    if (status != 0)
+    {
+      if (getenv("GRT_VERBOSE"))
+      {
+        g_warning("Could not load lua _library: %s", lua_tostring(priv->lua, -1));
+        lua_pop(priv->lua, 1);
+      }
+    }
+
+    // execute the _library, so that function declarations in it get executed
+    status= lua_pcall(priv->lua, 0, 0, 0);
+
+    if (status != 0)
+    {
+      g_warning("error executing lua _library: %s\n", lua_tostring(priv->lua, -1));
+      lua_pop(priv->lua, 1);
+    }
+  }
+
   g_assert(lua_gettop(priv->lua) == 0);
   
-  return loader;
+  GRT_RETURN(grt, loader, MYX_GRT_MODULE_LOADER*);
 }
 
 
@@ -123,6 +150,7 @@ static MYX_GRT_ERROR lua_init_module(MYX_GRT_MODULE_LOADER *loader, const char *
   lua_State *l= loader->priv->lua;
   MYX_GRT_VALUE *module_info;
   const char *module_name, *extends;
+  char *lua_function_table_name;
   MYX_GRT_LIST *module_functions;
   int status;
   unsigned int i;
@@ -194,6 +222,7 @@ static MYX_GRT_ERROR lua_init_module(MYX_GRT_MODULE_LOADER *loader, const char *
 
   // get module data
   module_name= myx_grt_value_as_string(myx_grt_dict_item_get_value(module_info, "name"));
+  lua_function_table_name= g_strdup_printf("__%s_impl", module_name);
 
   extends= myx_grt_value_as_string(myx_grt_dict_item_get_value(module_info, "extends"));
 
@@ -209,14 +238,17 @@ static MYX_GRT_ERROR lua_init_module(MYX_GRT_MODULE_LOADER *loader, const char *
   }
 
   // rename the ___tmp module table to the definitive name
-  lua_pushstring(l, module_name);
+  lua_pushstring(l, lua_function_table_name);
   lua_getglobal(l, "___tmp");
   lua_settable(l, LUA_GLOBALSINDEX);
 
+  // !!! temporary workaround
   lua_pushstring(l, "___tmp");
   lua_pushnil(l);
   lua_settable(l, LUA_GLOBALSINDEX);
 
+  g_free(lua_function_table_name);
+
   
   // init internal module descriptor
   module= g_new0(MYX_GRT_MODULE, 1);
@@ -256,23 +288,33 @@ static MYX_GRT_ERROR lua_call_function(MYX_GRT_FUNCTION *function, MYX_GRT_VALUE
 {
   MYX_GRT_ERROR error= MYX_GRT_NO_ERROR;
   lua_State *lua= function->module->loader->priv->lua;
+  char *lua_function_table_name= g_strdup_printf("__%s_impl", function->module->name);
+  ;
   
   if (getenv("GRT_VERBOSE"))
     g_message("Calling lua function %s.%s", function->module->name, function->name);
 
+  lua_getglobal(lua, lua_function_table_name);
+  lua_pushstring(lua, function->name);
+  lua_gettable(lua, -2);
+
   if (value)
     myx_lua_push_grt_value(lua, value);
   else
     lua_pushnil(lua);
 
-  lua_getglobal(lua, function->module->name);
-  lua_pushstring(lua, function->name);
-  lua_gettable(lua, -2);
+  g_free(lua_function_table_name);
   
   if (lua_pcall(lua, 1, 1, 0) != 0)
   {
-    g_warning("Error calling lua function %s.%s", function->module->name, function->name);
-    *retval= NULL;
+    char *error_str= g_strdup_printf("Error calling lua function %s.%s", 
+      function->module->name, function->name);
+
+    *retval= myx_grt_dict_new(NULL, NULL);
+
+    myx_grt_dict_item_set_value_from_string(*retval, "error", error_str);
+
+    g_free(error_str);
     
     error= MYX_GRT_FUNCTION_CALL_ERROR;
   }
@@ -281,6 +323,8 @@ static MYX_GRT_ERROR lua_call_function(MYX_GRT_FUNCTION *function, MYX_GRT_VALUE
     *retval= myx_lua_pop_grt_value(lua);
   }
 
+  lua_pop(lua, 1);
+
   return error;
 }
 
