@@ -53,18 +53,25 @@ int myx_grt_scan_for_modules(MYX_GRT *grt, const char *directory,
 
   g_return_val_if_fail(grt != NULL, -1);
 
+  GRT_ENTER(grt);
+  
   dir= g_dir_open(directory, 0, NULL);
   if (!dir)
   {
+    myx_grt_messages_stack_add_message(grt, "Cannot find module directory %s.", NULL, 0, directory);
+
     *error= MYX_GRT_BAD_PATH;
-    return -1;
+    GRT_RETURN(grt, -1, int);
   }
 
+  if (grt->options & MYX_GRT_VERBOSE)
+    myx_grt_messages_stack_add_message(grt, "Scanning module directory %s.", NULL, 0, directory);
+
   while ((entry= g_dir_read_name(dir)) != NULL)
   {
     char *path= g_build_filename(directory, entry, NULL);
 
-    if (myx_grt_module_init(grt, path) == MYX_GRT_NO_ERROR)
+    if ((entry[0] != '_') && (myx_grt_module_init(grt, path) == MYX_GRT_NO_ERROR))
     {
       count++;
     }
@@ -74,7 +81,7 @@ int myx_grt_scan_for_modules(MYX_GRT *grt, const char *directory,
 
   g_dir_close(dir);
 
-  return count;
+  GRT_RETURN(grt, count, int);
 }
 
 int myx_grt_scan_for_structs(MYX_GRT *grt, const char *directory, MYX_GRT_ERROR *error)
@@ -87,11 +94,13 @@ int myx_grt_scan_for_structs(MYX_GRT *grt, const char *directory, MYX_GRT_ERROR
 
   g_return_val_if_fail(grt != NULL, -1);
 
+  GRT_ENTER(grt);
+  
   dir= g_dir_open(directory, 0, NULL);
   if (!dir)
   {
     *error= MYX_GRT_BAD_PATH;
-    return -1;
+    GRT_RETURN(grt, -1, int);
   }
 
   while ((entry= g_dir_read_name(dir)) != NULL)
@@ -111,7 +120,7 @@ int myx_grt_scan_for_structs(MYX_GRT *grt, const char *directory, MYX_GRT_ERROR
 
   g_dir_close(dir);
 
-  return count;
+  GRT_RETURN(grt, count, int);
 }
 
 
@@ -146,6 +155,8 @@ MYX_GRT_ERROR myx_grt_module_init(MYX_GRT *grt, const char *filename)
   MYX_GRT_MODULE *module= NULL;
   unsigned int i;
   MYX_GRT_ERROR error;
+
+  GRT_ENTER(grt);
   
   for (i= 0; i < grt->loaders_num; i++)
   {
@@ -154,18 +165,23 @@ MYX_GRT_ERROR myx_grt_module_init(MYX_GRT *grt, const char *filename)
     {
       if (g_str_has_suffix(filename, grt->loaders[i]->extensions[j]))
       {
+        if (grt->options & MYX_GRT_VERBOSE)
+          myx_grt_messages_stack_add_message(grt, "Trying to load module %s.", NULL, 0, filename);
+
         error= grt->loaders[i]->init_module(grt->loaders[i], filename, &module);
         if (error != MYX_GRT_NO_ERROR)
-          return error;
+          GRT_RETURN(grt, error, MYX_GRT_ERROR);
         break;
       }
     }
   }
 
   if (!module)
-    return MYX_GRT_UNKNOWN_MODULE_TYPE;
+    error= MYX_GRT_UNKNOWN_MODULE_TYPE;
   else
-    return myx_grt_add_module(grt, module);
+    error= myx_grt_add_module(grt, module);
+  
+  GRT_RETURN(grt, error, MYX_GRT_ERROR);
 }
 
 /**
@@ -184,6 +200,9 @@ MYX_GRT_ERROR myx_grt_module_init(MYX_GRT *grt, const char *filename)
 MYX_GRT_FUNCTION *myx_grt_function_get(MYX_GRT *grt, const char *module, const char *function_name, int search_parent)
 {
   unsigned int i;
+  MYX_GRT_FUNCTION *function= NULL;
+  
+  GRT_ENTER(grt);
   
   for (i= 0; i < grt->modules_num; i++)
   {
@@ -196,16 +215,19 @@ MYX_GRT_FUNCTION *myx_grt_function_get(MYX_GRT *grt, const char *module, const c
       {
         if (strcmp(grt->modules[i]->functions[j].name, function_name) == 0)
         {
-          return grt->modules[i]->functions+j;
+          GRT_RETURN(grt, grt->modules[i]->functions+j, MYX_GRT_FUNCTION*);
         }
       }
       /* if not found, then search in parent */
       if (search_parent && grt->modules[i]->extends)
-        return myx_grt_function_get(grt, grt->modules[i]->extends, function_name, search_parent);
+      {
+        function= myx_grt_function_get(grt, grt->modules[i]->extends, function_name, search_parent);
+        break;
+      }
     }
   }
 
-  return NULL;
+  GRT_RETURN(grt, function, MYX_GRT_FUNCTION*);
 }
 
 /**
@@ -244,18 +266,20 @@ MYX_GRT_VALUE * myx_grt_function_call(MYX_GRT *grt, MYX_GRT_FUNCTION *func,
   //g_return_val_if_fail(grt!=NULL, MYX_GRT_INTERNAL_ERROR);
   //g_return_val_if_fail(func!=NULL, MYX_GRT_INTERNAL_ERROR);
 
+  GRT_ENTER(grt);
+  
   if (func->param_struct_name && argument)
   {
     if (!myx_grt_dict_struct_validate(grt, argument, func->param_struct_name, 0))
     {
       *error= MYX_GRT_VALIDATION_ERROR;
-      return NULL;
+      GRT_RETURN(grt, NULL, MYX_GRT_VALUE*);
     }
   }
 
   *error= (*func->module->loader->call_function)(func, argument, &retval);
   if (*error != MYX_GRT_NO_ERROR)
-    return NULL;
+    GRT_RETURN(grt, NULL, MYX_GRT_VALUE*);
   
   if (retval && func->return_struct_name)
   {
@@ -263,11 +287,11 @@ MYX_GRT_VALUE * myx_grt_function_call(MYX_GRT *grt, MYX_GRT_FUNCTION *func,
     {
       myx_grt_value_release(retval);
       *error= MYX_GRT_VALIDATION_ERROR;
-      return NULL;
+      GRT_RETURN(grt, NULL, MYX_GRT_VALUE*);
     }
   }
 
-  return retval;
+  GRT_RETURN(grt, retval, MYX_GRT_VALUE*);
 }
 
 /**
@@ -291,14 +315,16 @@ MYX_GRT_VALUE * myx_grt_function_call(MYX_GRT *grt, MYX_GRT_FUNCTION *func,
 MYX_GRT_VALUE * myx_grt_function_get_and_call(MYX_GRT *grt, const char *module, const char *function_name, int search_parent,
                                          MYX_GRT_VALUE *argument, MYX_GRT_ERROR *error)
 {
-  MYX_GRT_FUNCTION *func= myx_grt_function_get(grt, module, function_name, search_parent);
+  MYX_GRT_FUNCTION *func;
+  GRT_ENTER(grt);
+  func= myx_grt_function_get(grt, module, function_name, search_parent);
   if (!func) 
   {
     *error = MYX_GRT_BAD_FUNCTION;
-    return NULL;
+    GRT_RETURN(grt, NULL, MYX_GRT_VALUE*);
   }
 
-  return myx_grt_function_call(grt, func, argument, error);
+  GRT_RETURN(grt, myx_grt_function_call(grt, func, argument, error), MYX_GRT_VALUE*);
 }
 
 /**
@@ -317,7 +343,7 @@ MYX_GRT_VALUE * myx_grt_function_create_result(MYX_GRT_VALUE *result)
   
   if (result)
   {
-    res= myx_grt_dict_new("");
+    res= myx_grt_dict_new(NULL, "");
 
     myx_grt_dict_item_set_value(res, "value", result);
     myx_grt_value_release(result);
@@ -361,7 +387,7 @@ MYX_GRT_VALUE * myx_grt_function_extract_value_from_result(MYX_GRT_VALUE *result
  ****************************************************************************/
 MYX_GRT_VALUE * myx_grt_function_create_error_result(const char *error, const char *error_details)
 {
-  MYX_GRT_VALUE *result= myx_grt_dict_new("");
+  MYX_GRT_VALUE *result= myx_grt_dict_new(NULL, "");
   MYX_GRT_VALUE *val= myx_grt_value_from_string(error);
 
   myx_grt_dict_item_set_value(result, "error", val);
@@ -394,7 +420,7 @@ char * myx_grt_function_check_error(MYX_GRT_VALUE *res, int allow_null_as_result
 
   //Check if we have a value
   if ( (res == NULL) && (allow_null_as_result == 0) )
-    return g_strdup("The function has not returned a object.");
+    return g_strdup("The function did not return an object.");
   else if ( (res == NULL) && (allow_null_as_result == 1) )
     return NULL;
 
@@ -406,13 +432,17 @@ char * myx_grt_function_check_error(MYX_GRT_VALUE *res, int allow_null_as_result
   error_val= myx_grt_dict_item_get_value(res, "error");
   if (error_val)
   {
-    return g_strdup_printf("%s" _br 
-      "Details: " _br 
-      "%s", 
-      myx_grt_value_as_string(error_val),
-      myx_grt_value_as_string(
-        myx_grt_dict_item_get_value(res, "detail"))
-      );
+    const char *detail= myx_grt_dict_item_get_as_string(res, "detail");
+
+    if (detail)
+      return g_strdup_printf("%s" _br 
+        "Details: " _br 
+        "%s", 
+        myx_grt_value_as_string(error_val),
+        detail);
+    else
+      return g_strdup_printf("%s", 
+        myx_grt_value_as_string(error_val));
   }
   else
     return NULL;
@@ -432,24 +462,29 @@ char * myx_grt_function_check_error(MYX_GRT_VALUE *res, int allow_null_as_result
 MYX_GRT_MODULE *myx_grt_module_get(MYX_GRT *grt, const char *name)
 {
   unsigned int i;
+  
+  GRT_ENTER(grt);
   for (i= 0; i < grt->modules_num; i++)
   {
     if (strcmp2(grt->modules[i]->name, name)==0)
-      return grt->modules[i];
+      GRT_RETURN(grt, grt->modules[i], MYX_GRT_MODULE*);
   }
-  return NULL;
+  GRT_RETURN(grt, NULL, MYX_GRT_MODULE*);
 }
 
 MYX_GRT_MODULE * myx_grt_module_get_by_index(MYX_GRT *grt, unsigned int index)
 {
+  GRT_ENTER(grt);
   if (index < grt->modules_num)
-    return grt->modules[index];
+    GRT_RETURN(grt, grt->modules[index], MYX_GRT_MODULE*);
   else
-    return NULL;
+    GRT_RETURN(grt, NULL, MYX_GRT_MODULE*);
 }
 
 MYX_GRT_MODULE_TYPE myx_grt_module_get_type(MYX_GRT_MODULE *module)
 {
+  //GRT_RETURN(grt, module->loader->loader_type, MYX_GRT_MODULE_TYPE);
+
   return module->loader->loader_type;
 }
 
@@ -476,8 +511,10 @@ int myx_grt_modules_get_that_extend(MYX_GRT *grt, const char *module, MYX_GRT_MO
   g_return_val_if_fail(grt!=NULL, 0);
   g_return_val_if_fail(retmodules != NULL, 0);
   
+  GRT_ENTER(grt);
+  
   if (grt->modules_num == 0)
-    return 0;
+    GRT_RETURN(grt, 0, int);
 
   *retmodules= g_new0(MYX_GRT_MODULE*,grt->modules_num);
 
@@ -498,12 +535,13 @@ int myx_grt_modules_get_that_extend(MYX_GRT *grt, const char *module, MYX_GRT_MO
   else
     *retmodules= g_realloc(*retmodules, sizeof(MYX_GRT_MODULE*)*count);
 
-  return count;
+  GRT_RETURN(grt, count, int);
 }
 
 int myx_grt_module_get_count(MYX_GRT *grt)
 {
-  return grt->modules_num;
+  GRT_ENTER(grt);
+  GRT_RETURN(grt, grt->modules_num, int);
 }
 
 MYX_STRINGLIST * myx_grt_module_get_names(MYX_GRT *grt)
@@ -511,6 +549,8 @@ MYX_STRINGLIST * myx_grt_module_get_names(MYX_GRT *grt)
   MYX_STRINGLIST *modules= g_new0(MYX_STRINGLIST, 1);
   unsigned int i;
 
+  GRT_ENTER(grt);
+  
   modules->strings_num= grt->modules_num;
   modules->strings= g_malloc0(sizeof(char *)*modules->strings_num);
 
@@ -520,7 +560,7 @@ MYX_STRINGLIST * myx_grt_module_get_names(MYX_GRT *grt)
     modules->strings[i]= g_strdup(module->name);
   }
 
-  return modules;
+  GRT_RETURN(grt, modules, MYX_STRINGLIST*);
 }
 
 int myx_grt_module_function_get_count(MYX_GRT_MODULE *module)
@@ -597,47 +637,86 @@ char * myx_grt_module_function_get_return_type(MYX_GRT_FUNCTION *func)
     return g_strdup("");
 }
 
-
-/* this is private
- ****************************************************************************
- * @brief Checks all modules for new messages to be added to the GRT msg queue.
- *
- * @param grt  
- *
- * @return 
- *****************************************************************************/
-MYX_GRT_ERROR myx_grt_fetch_module_messages(MYX_GRT *grt)
+MYX_GRT_MODULE_LOADER * myx_grt_module_loader_create(MYX_GRT *grt, MYX_GRT_MODULE_TYPE loader_type,
+                  MYX_STRINGLIST *extensions, 
+                  MYX_GRT_ERROR (*init_module)(MYX_GRT_MODULE_LOADER *loader, const char *file_name, MYX_GRT_MODULE **retmodule),
+                  MYX_GRT_ERROR (*call_function)(MYX_GRT_FUNCTION *func, MYX_GRT_VALUE *argument, MYX_GRT_VALUE **retval),
+                  void *priv)
 {
+  MYX_GRT_MODULE_LOADER *loader= g_new0(MYX_GRT_MODULE_LOADER, 1);
   unsigned int i;
-  MYX_GRT_ERROR error;
 
-  for (i= 0; i < grt->loaders_num; i++)
-  {
-    MYX_GRT_MSGS msgs;
-
-    msgs.msgs_num= 0;
-    msgs.msgs= NULL;
-    if (grt->loaders[i]->fetch_messages)
-    {
-      error= grt->loaders[i]->fetch_messages(grt->loaders[i], &msgs);
-
-      if (msgs.msgs_num > 0)
-      {
-        if (!grt->msgs)
-          grt->msgs= g_new0(MYX_GRT_MSGS, 1);
-
-        grt->msgs->msgs_num+= msgs.msgs_num;
-        grt->msgs->msgs= g_realloc(grt->msgs->msgs, grt->msgs->msgs_num*sizeof(MYX_GRT_MSG));
-        
-        memcpy(grt->msgs->msgs+(grt->msgs->msgs_num-msgs.msgs_num),
-               msgs.msgs, msgs.msgs_num*sizeof(MYX_GRT_MSG));
-      }
-      g_free(msgs.msgs);
-
-      if (error != MYX_GRT_NO_ERROR)
-        return error;
-    }
-  }
+  GRT_ENTER(grt);
   
-  return MYX_GRT_NO_ERROR;
+  loader->grt= grt;
+  loader->loader_type= loader_type;
+
+  //Copy list of extensions
+  if (extensions == NULL) 
+  {
+    loader->extensions_num= 0;
+    loader->extensions= NULL;
+  }
+  else
+  {
+    loader->extensions_num= extensions->strings_num;
+    loader->extensions= g_malloc(extensions->strings_num+1);
+    for (i= 0; i < extensions->strings_num; i++)
+      loader->extensions[i]= g_strdup(extensions->strings[i]);
+    loader->extensions[extensions->strings_num]= NULL;
+  }
+
+  loader->init_module= init_module;
+  loader->call_function= call_function;
+
+  loader->priv= priv;
+
+  GRT_RETURN(grt, loader, MYX_GRT_MODULE_LOADER*);
 }
+
+void * myx_grt_module_loader_get_private_data(MYX_GRT_MODULE_LOADER *loader)
+{
+  return loader->priv;
+}
+
+MYX_GRT_MODULE * myx_grt_module_create(MYX_GRT_MODULE_LOADER *loader, const char *name, 
+                                                       const char *path, const char *extends, void *priv)
+{
+  MYX_GRT_MODULE *mod= g_new0(MYX_GRT_MODULE, 1);
+
+  mod->loader= loader;
+  mod->name= g_strdup(name);
+  if(path)
+    mod->path= g_strdup(path);
+  if(extends)
+    mod->extends= g_strdup(extends);
+
+  mod->priv= priv;
+
+  return mod;
+}
+
+void myx_grt_module_add_function(MYX_GRT_MODULE *module, const char *name, const char *param_struct_name, 
+                                                 const char *return_struct_name, void *priv)
+{
+  MYX_GRT_FUNCTION *func;
+
+  module->functions= g_realloc(module->functions, sizeof(MYX_GRT_FUNCTION) * (module->functions_num + 1));
+  func= module->functions + module->functions_num;
+  module->functions_num++;
+  
+
+  func->module= module;
+  func->name= g_strdup(name);
+  if ((param_struct_name) && (*param_struct_name))
+    func->param_struct_name= g_strdup(param_struct_name);
+  else
+    func->param_struct_name= NULL;
+
+  if ((return_struct_name) && (*return_struct_name))
+    func->return_struct_name= g_strdup(return_struct_name);
+  else
+    func->return_struct_name= NULL;
+
+  func->priv= priv;
+}
\ No newline at end of file
