@@ -60,10 +60,15 @@ extern "C" {
  * Functions
  */
 
-MYX_PUBLIC_FUNC MYX_GRT_ERROR myx_register_builtin_grt_module_reverse_engineer_mysql(MYX_GRT *grt);
+MYX_PUBLIC_FUNC MYX_GRT_MODULE * myx_register_builtin_grt_module_base(MYX_GRT *grt);
 
-MYX_PUBLIC_FUNC MYX_GRT_ERROR myx_register_builtin_grt_module_transformation_mysql(MYX_GRT *grt);
+MYX_PUBLIC_FUNC MYX_GRT_MODULE * myx_register_builtin_grt_module_reverse_engineer_mysql(MYX_GRT *grt);
 
+MYX_PUBLIC_FUNC MYX_GRT_MODULE * myx_register_builtin_grt_module_transformation_mysql(MYX_GRT *grt);
+
+#if !defined(__WIN__) && !defined(_WIN32) && !defined(_WIN64)
+MYX_PUBLIC_FUNC MYX_GRT_MODULE * myx_register_builtin_grt_module_forms(MYX_GRT *grt);
+#endif
 
 #ifdef __cplusplus
 }
