@@ -21,8 +21,9 @@
 extern "C" {
 #endif
 
-MYX_GRT_VALUE *get_schema_list(MYX_GRT_VALUE *param, void *data);
-MYX_GRT_VALUE *get_schema_assets(MYX_GRT_VALUE *param, void *data);
+MYX_GRT_VALUE *test_connection(MYX_GRT_VALUE *param, void *data);
+MYX_GRT_VALUE *get_schemata(MYX_GRT_VALUE *param, void *data);
+MYX_GRT_VALUE *reverse_engineer(MYX_GRT_VALUE *param, void *data);
 
 #ifdef __cplusplus
 }
