@@ -200,7 +200,7 @@ public class Grt {
 			obj = (GrtObject) getCallback().getGrtGlobalById(
 					getMyxGrtPointer(), refId);
 		}
-		
+
 		return obj;
 	}
 
@@ -218,7 +218,7 @@ public class Grt {
 		try {
 			Class c = Class.forName(Grt.GrtPackagePrefix + className);
 
-			Constructor con = c.getConstructor(new Class[] { GrtObject.class });
+			Constructor con = c.getConstructor(new Class[] { Object.class });
 			GrtObject obj = (GrtObject) con
 					.newInstance(new Object[] { parent });
 
@@ -234,9 +234,29 @@ public class Grt {
 	 * 
 	 * @param msg
 	 *            message string
+	 * @return a new generated GrtMessage
 	 */
 	public GrtMessage addMsg(String msg) {
-		GrtMessage grtMsg = new GrtMessage(msg, 0);
+		GrtMessage grtMsg = new GrtMessage(msg, GrtMessage.logMessage);
+
+		msgs.add(grtMsg);
+
+		return grtMsg;
+	}
+
+	/**
+	 * Adds the given message string as a normal GRT message to the GRT message
+	 * list
+	 * 
+	 * @param msg
+	 *            message string
+	 * @param progress
+	 *            the current progress, a value between 0 and 100
+	 * @return a new generated GrtMessage
+	 */
+	public GrtMessage addProgress(String msg, int progress) {
+		GrtMessage grtMsg = new GrtMessage(msg, GrtMessage.logProgress,
+				progress);
 
 		msgs.add(grtMsg);
 
@@ -248,9 +268,10 @@ public class Grt {
 	 * 
 	 * @param msg
 	 *            message string
+	 * @return a new generated GrtMessage
 	 */
 	public GrtMessage addErr(String msg) {
-		GrtMessage grtMsg = new GrtMessage(msg, 1);
+		GrtMessage grtMsg = new GrtMessage(msg, GrtMessage.logError);
 
 		msgs.add(grtMsg);
 
@@ -288,11 +309,35 @@ public class Grt {
 	 * 
 	 * @param offset
 	 *            number of first message to return
+	 * @return the Grt message vector
 	 */
 	public Vector getMessages() {
 		return msgs;
 	}
 
+	/**
+	 * Forces the output of the GRT messages
+	 *  
+	 */
+	public int flushMessages() {
+		int res = 0;
+
+		if ((msgs.size() > 0) && (callback != null)) {
+			GrtList msgList = new GrtList("GrtMessage");
+
+			for (int i = 0; i < msgs.size(); i++) {
+				msgList.addObject(msgs.get(i));
+			}
+
+			msgs.clear();
+
+			res = getCallback().processMessages(getMyxGrtPointer(),
+					msgList.getGrtXml());
+		}
+
+		return res;
+	}
+
 	/**
 	 * Returns the numeric type of a value_type name
 	 * 
@@ -375,7 +420,8 @@ public class Grt {
 
 				// if this is a regular object
 				if (globalObjectPath.equals("")) {
-					Constructor con = c.getConstructor(new Class[] { Object.class });
+					Constructor con = c
+							.getConstructor(new Class[] { Object.class });
 					obj = (GrtObject) con.newInstance(new Object[] { null });
 				} else {
 					// if this is a global object
@@ -568,10 +614,10 @@ public class Grt {
 					list.addObject(item);
 			}
 		}
-		
+
 		if (list != null)
 			list.setOwner(parent);
-			
+
 		return list;
 	}
 
@@ -943,10 +989,13 @@ public class Grt {
 		GrtHashMap result = new GrtHashMap();
 		result.addObject("value", resultValue);
 		String xmlResult = prepareGrtXml(getObjectAsXml(result));
-		
+
 		// clear object cache
 		Grt.getInstance().clearObjectCache();
-		
+
+		// flush messages if there are any
+		Grt.getInstance().flushMessages();
+
 		return xmlResult;
 	}
 
@@ -1085,7 +1134,8 @@ public class Grt {
 	 *            the new value of the object
 	 */
 	public void setGrtGlobalFromString(String objectPath, String value) {
-		getCallback().setGrtGlobalFromString(getMyxGrtPointer(), objectPath, value);
+		getCallback().setGrtGlobalFromString(getMyxGrtPointer(), objectPath,
+				value);
 	}
 
 	/**
@@ -1098,7 +1148,8 @@ public class Grt {
 	 *            the new value of the object
 	 */
 	public void setGrtGlobalFromInt(String objectPath, int value) {
-		getCallback().setGrtGlobalFromInt(getMyxGrtPointer(), objectPath, value);
+		getCallback()
+				.setGrtGlobalFromInt(getMyxGrtPointer(), objectPath, value);
 	}
 
 	/**
@@ -1111,7 +1162,8 @@ public class Grt {
 	 *            the new value of the object
 	 */
 	public void setGrtGlobalFromReal(String objectPath, double value) {
-		getCallback().setGrtGlobalFromReal(getMyxGrtPointer(), objectPath, value);
+		getCallback().setGrtGlobalFromReal(getMyxGrtPointer(), objectPath,
+				value);
 	}
 
 	public int getGrtGlobalListSize(String objectPath) {
@@ -1148,4 +1200,14 @@ public class Grt {
 		getCallback().removeGrtGlobalDictItem(getMyxGrtPointer(), objectPath,
 				key);
 	}
+
+	public int getGrtGlobalDictKeyCount(String objectPath) {
+		return getCallback().getGrtGlobalDictKeyCount(getMyxGrtPointer(),
+				objectPath);
+	}
+
+	public String getGrtGlobalDictKey(String objectPath, int index) {
+		return getCallback().getGrtGlobalDictKey(getMyxGrtPointer(),
+				objectPath, index);
+	}
 }
\ No newline at end of file
