@@ -1,25 +1,30 @@
 package com.mysql.grt;
 
 public class GrtMessage implements GrtXml {
-	private int msgType; // 0..log message, 1..error
+	private int msgType = 1; // 0..message, 1..error, 2.. progress
 
-	private String msg;
+	private String msg = "";
 
 	private GrtStringList msgDetails = new GrtStringList();
+	
+	private int progress = -1;
 
 	final static int logMessage = 0;
 
 	final static int logError = 1;
-
-	public GrtMessage() {
-		this.msg = "";
-		this.msgType = logError;
-	}
+	
+	final static int logProgress = 2;
 
 	public GrtMessage(String msg, int msgType) {
 		this.msg = msg;
 		this.msgType = msgType;
 	}
+	
+	public GrtMessage(String msg, int msgType, int progress) {
+		this.msg = msg;
+		this.msgType = msgType;
+		this.progress = progress;
+	}
 
 	public void addMessageDetail(String msgDetail) {
 		msgDetails.add(msgDetail);
@@ -30,10 +35,14 @@ public class GrtMessage implements GrtXml {
 	}
 
 	public String getGrtXml(String keyTag) {
-		return "<value type=\"dict\" struct-name=\"GrtMessage\" key=\""
+		String xml = "<value type=\"dict\" struct-name=\"GrtMessage\" key=\""
 				+ keyTag + "\">" + "<value type=\"int\" key=\"msgType\">"
 				+ Integer.toString(msgType) + "</value>\n"
+				+ "<value type=\"int\" key=\"progress\">"
+				+ Integer.toString(progress) + "</value>\n"
 				+ "<value type=\"string\" key=\"msg\">" + msg + "</value>\n"
 				+ msgDetails.getGrtXml(" key=\"details\"") + "</value>\n";
+
+		return xml;
 	}
 }
\ No newline at end of file
