@@ -222,7 +222,7 @@ bool MCrontab::load()
   if (!g_spawn_command_line_sync(cmdline, &stdout_text, &stderr_text, &exit_status,
                                  &error) || exit_status != 0)
   {
-    if (!strstr || !strstr(stderr_text, "no crontab"))
+    if (!stderr_text || !strstr(stderr_text, "no crontab"))
     {
       got_error= true;
 
