@@ -37,7 +37,7 @@ static struct OptionsByEngine {
   char *option;
   int engines;
 } EngineOptionList[]= {
-  {"fk_page", (1<<MYX_DBM_TE_INNODB)},
+//  {"fk_page", (1<<MYX_DBM_TE_INNODB)},
 //  {"index_kind_option", (1<<MYX_DBM_TE_MEMORY)},
   {"merge_options", (1<<MYX_DBM_TE_MERGE)}
 };
@@ -102,33 +102,6 @@ static MYX_DBM_DATATYPE *find_datatype(MYX_DBM_DATATYPES *data_types, Glib::ustr
   return NULL;
 }
 
-
-static Glib::ustring quote_default_value(Glib::ustring value)
-{
-  if (value != "NULL")
-  {
-    Glib::ustring::size_type i;
-    bool isnum= true;
-    bool dot= false;
-
-    for (i= 0; i < value.size(); i++)
-    {
-      if (isdigit(value[i]) || (!dot && (value[i]=='.' || value[i]==',')))
-      {
-        if (value[i]=='.' || value[i]==',') dot= true;
-        isnum= false;
-        break;
-      }
-    }
-    if (!(value[0]=='\'' && value[value.size()-1]=='\'') && !isnum)
-    {
-      value= "'"+value+"'";
-    }
-  }
-  return value;
-}
-
-
 static Glib::ustring shorten_comment(const Glib::ustring &comment)
 {
   Glib::ustring::size_type p= comment.find('\n');
@@ -210,7 +183,6 @@ MGTableEditor::MGTableEditor(bool windowed)
   std::vector<Gtk::CellRenderer*> rends= column->get_cell_renderers();
   
   tree->signal_button_press_event().connect_notify(SigC::slot(*this,&MGTableEditor::column_tree_clicked));
-  tree->signal_key_press_event().connect_notify(SigC::slot(*this,&MGTableEditor::column_tree_key_pressed));
   tree->signal_key_release_event().connect(SigC::slot(*this,&MGTableEditor::column_tree_key_released));
 
   ((Gtk::CellRendererText*)rends[1])->property_editable()= true;
@@ -220,6 +192,9 @@ MGTableEditor::MGTableEditor(bool windowed)
   tree->append_column_editable("", _tcolumns.autoinc);
   tree->append_column(_("Flags"), _tcolumns.flags);
   tree->append_column_editable(_("Default Value"), _tcolumns.defval);
+  tree->get_column(5)->add_attribute(static_cast<Gtk::CellRendererText*>(tree->get_column(5)->get_first_cell_renderer())->property_style_set(),
+                                     _tcolumns.defnull);
+  static_cast<Gtk::CellRendererText*>(tree->get_column(5)->get_first_cell_renderer())->property_style()= Pango::STYLE_ITALIC;
   tree->append_column_editable(_("Comments"), _tcolumns.comment);
   Gtk::Widget *w;
   tree->get_column(2)->set_widget(*(w=Gtk::manage(new Gtk::Image(PIXCACHE->load("editor_table_not_null.png")))));
@@ -256,7 +231,8 @@ MGTableEditor::MGTableEditor(bool windowed)
   _xml->get_toggle("nnull_check")->signal_toggled().connect(SigC::bind<char*>(SigC::slot(*this,&MGTableEditor::column_option_changed),"nn"));
   _xml->get_toggle("autoinc_check")->signal_toggled().connect(SigC::bind<char*>(SigC::slot(*this,&MGTableEditor::column_option_changed),"autoinc"));
   _xml->get_text("comment_text")->get_buffer()->signal_changed().connect(SigC::bind<char*>(SigC::slot(*this,&MGTableEditor::column_option_changed),"comment"));
-
+  _xml->get_toggle("null_toggle")->signal_toggled().connect(SigC::bind<char*>(SigC::slot(*this,&MGTableEditor::column_option_changed),"defnull"));
+  
   // set up dragging from column list
   std::vector<Gtk::TargetEntry> types;
   types.push_back(Gtk::TargetEntry("x-mysqlgui-table-column"));
@@ -426,7 +402,11 @@ void MGTableEditor::show_column_list(MYX_DBM_TABLE_DATA *data)
         tmp= Glib::ustring(column->datatype_flags[j]);
     }
     row[_tcolumns.flags]= tmp;
-    row[_tcolumns.defval]= column->default_value?:"";
+    if (column->default_value_is_null)
+      row[_tcolumns.defval]= "NULL";
+    else
+      row[_tcolumns.defval]= column->default_value?:"";
+    row[_tcolumns.defnull]= column->default_value_is_null;
     row[_tcolumns.comment]= shorten_comment(column->comment?:"");
     row[_tcolumns.comment_full]= column->comment?:"";
     row[_tcolumns.charset]= column->charset?description_for_charset(column->charset):DEFAULT_CHARSET;
@@ -454,6 +434,7 @@ void MGTableEditor::show_column_info(MYX_DBM_TABLE_DATA *data, const Gtk::TreeIt
   _xml->get_entry("column_name_entry")->set_text(iter ? (Glib::ustring)row[_tcolumns.name] : "");
   _xml->get_combo("datatype_combo")->get_entry()->set_text(iter ? (Glib::ustring)row[_tcolumns.type] : "");
   _xml->get_entry("default_entry")->set_text(iter ? (Glib::ustring)row[_tcolumns.defval] : "");
+  _xml->get_toggle("null_toggle")->set_active(iter ? (bool)row[_tcolumns.defnull] : false);
   _xml->get_toggle("pk_check")->set_active(iter ? (bool)row[_tcolumns.pk] : false);
   _xml->get_toggle("nnull_check")->set_active(iter ? (bool)row[_tcolumns.nnull] : false);
   _xml->get_toggle("autoinc_check")->set_active(iter ? (bool)row[_tcolumns.autoinc] : false);
@@ -928,6 +909,9 @@ void MGTableEditor::show_table(MYSQL *mysql, const Glib::ustring &catalog,
   fill_aux_data();
 
   revert_changes();
+  
+  // always leave a placeholder row
+  column_add();
 }
 
 
@@ -1029,29 +1013,10 @@ void MGTableEditor::column_delete()
 }
 
 
-void MGTableEditor::column_tree_key_pressed(GdkEventKey *event)
-{
-  /* not sure if this is good...
-  if (event->keyval == GDK_Down)
-  {
-   Gtk::TreeIter iter= _column_tree->get_selection()->get_selected();
-    
-    if (iter)
-    {
-      ++iter;
-      if (iter == _column_store->children().end())
-        _column_tree->set_cursor(Gtk::TreePath(column_add()), *_column_tree->get_column(0), true);
-    }
-  }
-   */
-}
-
-
 bool MGTableEditor::column_tree_key_released(GdkEventKey *event)
 {
   _column_tree_last_key= event->keyval;
-  if (event->keyval == GDK_Tab || event->keyval == GDK_Return 
-      || event->keyval == GDK_Delete || event->keyval == GDK_Insert)
+  if (event->keyval == GDK_Tab || event->keyval == GDK_Delete)
   {
     Gtk::TreeViewColumn *column;
     Gtk::TreePath path;
@@ -1098,22 +1063,6 @@ bool MGTableEditor::column_tree_key_released(GdkEventKey *event)
       if (_column_tree == static_cast<Gtk::Window*>(_column_tree->get_toplevel())->get_focus())
         column_delete();
     }
-    else if (event->keyval == GDK_Insert)
-    {
-      Gtk::TreeIter iter= _column_tree->get_selection()->get_selected();
-      
-      if (iter)
-      {      
-        _column_tree->set_cursor(Gtk::TreePath(column_add(iter)), *_column_tree->get_column(0), true);
-      }
-      else
-        append_row= true;
-    }
-    
-    if (append_row)
-    {
-      _column_tree->set_cursor(Gtk::TreePath(column_add()), *_column_tree->get_column(0), true);
-    }
     return true;
   }
   return false;
@@ -1147,37 +1096,40 @@ void MGTableEditor::column_edited(const Glib::ustring &path, const Glib::ustring
 {
   Gtk::TreeIter iter= _column_store->get_iter(path);
   Gtk::TreeRow row= *iter;
+  bool changed= false;
 
   switch (column)
   {
   case 0: // name
-    row[_tcolumns.name]= new_text; // we need to manually set the value here
-    _xml->get_entry("column_name_entry")->set_text(new_text);
+    if (row[_tcolumns.name] != new_text)
+    {
+      row[_tcolumns.name]= new_text; // we need to manually set the value here
+      _xml->get_entry("column_name_entry")->set_text(new_text);
+      changed= true;
+    }
     break;
   case 1: // type
-    if (check_datatype(new_text))
-      _xml->get_combo("datatype_combo")->get_entry()->set_text(new_text);
-    else
-      row[_tcolumns.type]= _xml->get_combo("datatype_combo")->get_entry()->get_text();
-
+    if (row[_tcolumns.type] != new_text)
     {
-      Glib::ustring a, b;
-      split_datatype((Glib::ustring)row[_tcolumns.type], a, b);
-
-      row[_tcolumns.icon]= get_icon_for_column(find_datatype(_data_types, a),
-                                               row[_tcolumns.pk]);
+      if (check_datatype(new_text))
+        _xml->get_combo("datatype_combo")->get_entry()->set_text(new_text);
+      else
+        row[_tcolumns.type]= _xml->get_combo("datatype_combo")->get_entry()->get_text();
+      
+      {
+        Glib::ustring a, b;
+        split_datatype((Glib::ustring)row[_tcolumns.type], a, b);
+        
+        row[_tcolumns.icon]= get_icon_for_column(find_datatype(_data_types, a),
+                                                 row[_tcolumns.pk]);
+      }
     }
     break;
   case 5: // default
-    {
-      Glib::ustring a, b;
-      split_datatype((Glib::ustring)row[_tcolumns.type], a, b);
-      MYX_DBM_DATATYPE *dt= find_datatype(_data_types, a);
-      if (!dt || dt->group != MYX_DBM_DTG_NUMERIC)
-        _xml->get_entry("default_entry")->set_text(quote_default_value(new_text));
-      else
-        _xml->get_entry("default_entry")->set_text(new_text);
-    }
+    _xml->get_entry("default_entry")->set_text(new_text);
+    row[_tcolumns.defval]= new_text;
+    row[_tcolumns.defnull]= false;
+    _xml->get_toggle("null_toggle")->set_active(false);
     break;
   case 6: // comment
     {
@@ -1192,6 +1144,9 @@ void MGTableEditor::column_edited(const Glib::ustring &path, const Glib::ustring
       break;
     }
   }
+
+  if ((unsigned int)Gtk::TreePath(path).front() == _column_store->children().size()-1 && changed)
+    column_add();
 }
 
 
@@ -1303,17 +1258,24 @@ void MGTableEditor::column_option_changed(const char *option)
     }
     else if (strcmp(option, "default")==0)
     {
-      Glib::ustring value= _xml->get_entry("default_entry")->get_text();
-      Glib::ustring a, b;
-      split_datatype(row[_tcolumns.type], a, b);
-      MYX_DBM_DATATYPE *dt= find_datatype(_data_types, a);
-
-      if (!dt || dt->group != MYX_DBM_DTG_NUMERIC)
+      row[_tcolumns.defval]= _xml->get_entry("default_entry")->get_text();
+    }
+    else if (strcmp(option, "defnull")==0)
+    {
+      bool flag= _xml->get_toggle("null_toggle")->get_active();
+      
+      if (flag)
       {
-        value= quote_default_value(value);
+        row[_tcolumns.defnull]= true;
+        row[_tcolumns.defval]= "NULL";
       }
-      row[_tcolumns.defval]= value;
-      _xml->get_entry("default_entry")->set_text(value);
+      else
+      {
+        row[_tcolumns.defnull]= false;
+        row[_tcolumns.defval]= "";
+        _xml->get_entry("default_entry")->set_text("");
+      }
+      _xml->get_entry("default_entry")->set_sensitive(!flag);
     }
     else if (strcmp(option, "pk")==0)
     {
@@ -1479,15 +1441,28 @@ void MGTableEditor::save_index_changes()
 
 
 void MGTableEditor::add_fk()
-{
-  Gtk::TreeIter iter= _fk_store->append();
-  Gtk::TreeRow row= *iter;
-  
-  row[_fk_columns.name]= "new_fk_constraint";
-  row[_fk_columns.on_delete]= MYX_DBM_FA_RESTRICT;
-  row[_fk_columns.on_update]= MYX_DBM_FA_RESTRICT;
+{  
+  if (get_selected_engine() == MYX_DBM_TE_INNODB)
+  {
+    Gtk::TreeIter iter= _fk_store->append();
+    Gtk::TreeRow row= *iter;
 
-  _xml->get_tree("fk_tree")->get_selection()->select(iter);
+    row[_fk_columns.name]= "new_fk_constraint";
+    row[_fk_columns.on_delete]= MYX_DBM_FA_RESTRICT;
+    row[_fk_columns.on_update]= MYX_DBM_FA_RESTRICT;
+
+    _xml->get_tree("fk_tree")->get_selection()->select(iter);
+  }
+  else
+  {
+    if (_windowed)
+      myg_show_info(*(Gtk::Window*)_xml->get_widget("table_editor"), 
+                    _("Foreign Keys can only be defined for tables that use the InnoDB engine.\n"
+                      "Make this an InnoDB table by selecting 'InnoDB' in the 'Table Options' tab."));
+    else
+      myg_show_info(_("Foreign Keys can only be defined for tables that use the InnoDB engine.\n"
+                      "Make this an InnoDB table by selecting 'InnoDB' in the 'Table Options' tab."));
+  }
 }
 
 
@@ -1801,7 +1776,9 @@ void MGTableEditor::commit_changes()
       column->datatype_flags_num= 0;
       while (column->datatype_flags[column->datatype_flags_num])
         column->datatype_flags_num++;
-      ASSIGN_N(column->default_value, (Glib::ustring)row[_tcolumns.defval]);
+      column->default_value_is_null= row[_tcolumns.defnull];
+      ASSIGN(column->default_value, (Glib::ustring)row[_tcolumns.defval]);
+      if (!column->default_value) column->default_value= g_strdup("");
       ASSIGN_N(column->comment, (Glib::ustring)row[_tcolumns.comment_full]);
       a= row[_tcolumns.charset];
       if (a != DEFAULT_CHARSET)
@@ -2097,7 +2074,7 @@ void MGTableEditor::confirm_save_script()
       myg_show_sys_error(ufmt(_("Could not create file '%s'."), fname.c_str()), errno);
       return;
     }
-    if (write(fd, data.data(), data.size()) < 0)
+    if (write(fd, data.data(), data.bytes()) < 0)
     {
       myg_show_sys_error(ufmt(_("Could not save to file '%s'."), fname.c_str()), errno);
     }
