@@ -6,7 +6,6 @@
 
 #include <mysqld_error.h>
 
-
 @interface MConnectionPanel(Private)
 - (void)setEnabled: (BOOL)flag;
 - (void)controlTextDidChange:(NSNotification *)aNotification;
@@ -26,13 +25,13 @@
 - (void)showConnectionEditor:(id)sender;
 - (void)saveConnection:(id)sender;
 
-- (void)finish;
+- (void)finish:(id)arg;
 @end
 
 
 @implementation MConnectionPanel(Private)
 
-- (void)finish
+- (void)finish:(id)arg
 {
   int rc;
   MYX_USER_CONNECTION *info= NULL;
@@ -80,8 +79,9 @@
   }
   else
     rc= 0;
-  [[self window] close];
 
+  [[self window] close];
+  
   if (rc > 0)
     [_delegate connectionPanel:self finished:_mysql info:info];
   else
@@ -105,7 +105,7 @@
 - (void)connectionFinished: (id)result
 {  
   int err;
-  
+
   _connecting= NO;
 
   [progressIndicator stopAnimation: self];
@@ -138,9 +138,12 @@
   }
   else
   {
-    [self finish];
+	[[NSRunLoop currentRunLoop] performSelector:@selector(finish:)
+										 target:self
+									   argument:nil
+										  order:1000
+										  modes:[NSArray arrayWithObject:NSDefaultRunLoopMode]];
   }
-  [result release];
 }
 
 
@@ -151,6 +154,8 @@
   BOOL ok= YES;
   id tmp;
   
+  mysql_thread_init();
+  
   mysql= myx_mysql_init();
 
   if (myx_connect_to_instance([info pointerValue], mysql) < 0)
@@ -162,7 +167,10 @@
 
   [self performSelectorOnMainThread:@selector(connectionFinished:)
                           withObject:result
-                       waitUntilDone:NO];
+                       waitUntilDone:YES];
+  [result release];
+  
+  mysql_thread_end();
 }
 
 
@@ -203,6 +211,8 @@
     g_free(_new_connection.hostname);
     _new_connection.hostname= g_strdup([[hostname stringValue] UTF8String]);
     _new_connection.port= [port intValue];
+	_new_connection.schema= g_strdup([[schema stringValue] UTF8String]);
+	
     _new_connection.storage_type= MYX_HISTORY_USER_CONNECTION;
 
     for (i= 0; i < _new_connection.advanced_options_num; i++)
@@ -211,12 +221,15 @@
     if (_new_connection.advanced_options)
       g_free(_new_connection.advanced_options);
     
-    _new_connection.advanced_options= g_new0(char*, 1);
+    _new_connection.advanced_options= g_new0(char*, 2);
     
     i= 0;
     if ([[socketText stringValue] length] > 0)
       _new_connection.advanced_options[i++]= g_strdup_printf("SOCKET_PATH=%s", 
-                                                             [[socketText stringValue] UTF8String]);
+                                                             [[[socketText stringValue] stringByExpandingTildeInPath] UTF8String]);
+	if ([compress state] == NSOnState)
+      _new_connection.advanced_options[i++]= g_strdup("COMPRESS=1");
+	
     _new_connection.advanced_options_num= i;
     
     return &_new_connection;
@@ -231,12 +244,18 @@
     [password setStringValue: [NSString stringWithUTF8String: info->password?:""]];
     [hostname setStringValue: [NSString stringWithUTF8String: info->hostname?:""]];
     [port setIntValue: info->port];
+	[schema setStringValue: [NSString stringWithUTF8String: info->schema?:""]];
     
     [socketText setStringValue: 
       [NSString stringWithUTF8String: 
         myx_get_option_value((const char**)info->advanced_options,
                              info->advanced_options_num,
                              "SOCKET_PATH") ? : ""]];
+	
+	[compress setState: 
+        myx_get_option_value((const char**)info->advanced_options,
+                             info->advanced_options_num,
+                             "COMPRESS") ? NSOnState : NSOffState];
   }
 }
 
@@ -315,8 +334,7 @@
 
 - (void)checkConnectOK
 {
-  if ([[username stringValue] length] > 0
-      && [[hostname stringValue] length] > 0)
+  if ([[hostname stringValue] length] > 0)
     [connectButton setEnabled: YES];
   else
     [connectButton setEnabled: NO];
@@ -422,13 +440,12 @@
     return [NSString stringWithFormat:@"%s@%s", conn->username, conn->hostname];
 }
 
-
 - (id)initWithConnectionsFile:(NSString*)file
 {
   MYX_LIB_ERROR error;
 
   self= [super initWithWindowNibName: @"ConnectPanel"];
-  
+
   _connectionsFile= [file retain];
   _connections= myx_load_user_connections([file cString], &error);
   if (!_connections)
@@ -437,29 +454,50 @@
       NSLog(@"Could not load user connections file");
     _connections= g_malloc0(sizeof(MYX_USER_CONNECTIONS));
   }
-  
+
   _connecting= NO;
   _mysql= NULL;
   memset(&_new_connection, 0, sizeof(MYX_USER_CONNECTION));
-  
+
   return self;
 }
 
 - (void)awakeFromNib
-{
+{  
   [[NSNotificationCenter defaultCenter] addObserver:self
                                            selector:@selector(errorPanelWillClose:)
                                                name:NSWindowWillCloseNotification
                                              object:errorPanel];
 }
 
+
+- (void)setEditsSchema
+{
+  NSRect frame= [[self window] frame];
+  NSRect eframe= [extraView frame];
+
+  [schema setHidden:NO];
+  [[[[self window] contentView] viewWithTag:201] setHidden:NO];
+  
+  frame.origin.y-= 30;
+  frame.size.height+= 30;
+  
+  [[self window] setFrame:frame display:NO];
+  [extraView setFrame:eframe];
+  [[self window] setMinSize:frame.size];
+  
+  [password setNextKeyView:schema];
+}
+
 - (void)dealloc
 {
+  [[NSNotificationCenter defaultCenter] removeObserver: self];
+  
   [_connectionsFile release];
   if (_connections)
     myx_free_user_connections(_connections);
   myx_free_user_connection_content(&_new_connection);
-  [[NSNotificationCenter defaultCenter] removeObserver: self];
+  
   
   if (pingPipe) [pingPipe release];
   if (pingTask) 
@@ -468,7 +506,6 @@
     [pingTask release];
   }
   if (pingPipeHandle) [pingPipeHandle release];
-  
   [super dealloc];
 }
 
@@ -476,7 +513,7 @@
 {
   if (!_connecting)
   {
-    [self finish];
+    [self finish:nil];
   }
 }
 
@@ -516,27 +553,35 @@
 {
   NSRect frame= [[self window] frame];
   NSSize newSize;
-  BOOL flag;
-  
+  BOOL expanded;
+
   if ([detailsButton state])
   {
-    flag= NO;
-    newSize= [[self window] maxSize];
+    expanded= YES;
+    newSize= [[self window] minSize];
+	newSize.height+= 88-22;
   }
   else
   {
-    flag= YES;
+    expanded= NO;
     newSize= [[self window] minSize];
   }
-  
+
   frame.origin.y += (frame.size.height - newSize.height);
   frame.size= newSize;
 
-  if (flag)
-    [[self window] setFrame:frame display:YES animate:YES];
-  [extraView setHidden:flag];
-  if (!flag)
-    [[self window] setFrame:frame display:YES animate:YES];
+  if (expanded)
+  {
+	[[self window] setFrame:frame display:YES animate:YES];
+	[extraView setFrame:NSMakeRect(20, 43, 388, 88)];
+  }
+  else
+  {
+	[[self window] setFrame:frame display:YES animate:YES];
+	[extraView setFrame:NSMakeRect(20, 43, 388, 22)];
+  }
+  
+  [[[self window] contentView] setNeedsDisplay:YES];
 }
 
 - (void)windowDidLoad
