@@ -170,6 +170,22 @@
 }
 
 
+- (void)cancelRequest:(id)sender
+{
+  MYSQL *mysql= myx_mysql_init();
+  
+  if (myx_connect_to_instance(_instance, mysql) == 0)
+  {
+	MYX_LIB_ERROR error;
+	char *sql= g_strdup_printf("kill %li", mysql_thread_id(_mysql));
+	myx_query_execute_direct(mysql, sql, &error);
+	g_free(sql);
+  }
+  
+  myx_mysql_close(mysql);
+}
+
+
 - (BOOL)waitForRequestToFinish:(WaitCallbackRequest*)req window:(NSWindow*)window message:(NSString*)msg
 {  
   // if still didn't finish, then show a busy sheet
@@ -177,6 +193,9 @@
   {    
     MProgressPanel *panel= [[MProgressPanel alloc] initAndLoad];
     
+	[panel setStopAction:@selector(cancelRequest:)];
+	[panel setTarget:self];
+	
     req->dialogVisible= YES;
     
     if (window)
@@ -200,6 +219,8 @@ static void *processRequests(void *arg)
 {
   MMySQLDispatcher *disp= (MMySQLDispatcher*)arg;
   
+  mysql_thread_init();
+  
   for (;;)
   {
     Request *req;
@@ -211,16 +232,21 @@ static void *processRequests(void *arg)
 
     [disp executeRequest:req];
   }
+  
+  mysql_thread_end();
+  
   return NULL;
 }
 
 
-- initWithMySQL: (MYSQL*)mysql
+- (id)initWithMySQL: (MYSQL*)mysql
+		   instance: (MYX_USER_CONNECTION*)instance
 {
   self= [super init];
   if (self)
   {
     _mysql= mysql;
+	_instance= instance;
   
     pthread_mutex_init(&_mysql_mx, NULL);
     pthread_mutex_init(&_request_mx, NULL);
