@@ -6,6 +6,9 @@
 
 #define COLUMN_TYPE(c) ((c).datatype_pointer ?  (c).datatype_pointer->group : 0)
 
+NSString *MTableEditorTableDidChange= @"MTableEditorTableDidChange";
+NSString *MTableRowPboardType= @"MTableRowPboardType";
+
 @interface MTableEditor(Private)
 - (void)addColumn:(id)sender;
 - (void)deleteColumn:(id)sender;
@@ -25,12 +28,7 @@
                     popup:(id)popup;
 - (void)handleTextEdited:(NSNotification*)notif;
 
-- (int)numberOfRowsInTableView:(NSTableView *)aTableView;
-- (id)tableView:(NSTableView *)aTableView objectValueForTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex;
-- (void)tableView:(NSTableView *)aTableView setObjectValue:(id)anObject forTableColumn:(NSTableColumn *)aTableColumn row:(int)rowIndex;
-
 - (void)tableViewSelectionDidChange:(NSNotification *)aNotification;
-- (NSImage*)loadImage: (NSString*)file;
 - (BOOL)commitChanges;
 - (void)revertChanges;
 - (void)refreshForTableType;
@@ -59,9 +57,7 @@ static MYX_SCHEMA_TABLE *find_table(MYX_CATALOGS *cats, const char *catalog,
           {
             MYX_SCHEMA_TABLE *tab= sch->schema_tables->schema_tables+t;
             if (strcmp2(tab->table_name, table)==0)
-            {
               return tab;
-            }
           }
           break;
         }
@@ -82,6 +78,8 @@ static MYX_SCHEMA_TABLE *find_table(MYX_CATALOGS *cats, const char *catalog,
   _table->columns[i].name= g_strdup("newcolumn");
   _table->columns[i].datatype_name= g_strdup("INT");
   _table->columns[i].datatype_pointer= myx_dbm_get_datatype(_data_types, &_table->columns[i].datatype_name);
+  _table->columns[i].default_value= g_strdup("");
+  _table->columns[i].default_value_is_null= 0;
   [columnList reloadData];
   [columnList selectRow:i byExtendingSelection:NO];
   [columnList editColumn:0 row:i withEvent:nil select:YES];
@@ -321,6 +319,8 @@ static MYX_SCHEMA_TABLE *find_table(MYX_CATALOGS *cats, const char *catalog,
   {
     g_free(column->name);
     column->name= g_strdup([str UTF8String]);
+    
+    [self columnListChanged];
   }
   else if (obj == colTypeCombo)
   {
@@ -355,7 +355,17 @@ static MYX_SCHEMA_TABLE *find_table(MYX_CATALOGS *cats, const char *catalog,
     [colNameText setStringValue:NSStr(column->name)];
     
     [colTypeCombo setStringValue:[NSString stringWithFormat:@"%s%s",column->datatype_name,column->datatype_params?:""]];
-    [colDefaultText setStringValue:NSStr(column->default_value)];
+    [colDefaultNull setState:column->default_value_is_null ? NSOnState : NSOffState];
+    if (column->default_value_is_null)
+    {
+      [colDefaultText setStringValue:@""];
+      [colDefaultText setEnabled:NO];
+    }
+    else
+    {
+      [colDefaultText setStringValue:NSStr(column->default_value)];
+      [colDefaultText setEnabled:YES];
+    }
     [colFlagsList reloadData];
         
     [colCharsetPop selectItemAtIndex:[[colCharsetPop menu] indexOfItemWithRepresentedObject:NSStr(column->charset)]];
@@ -377,6 +387,7 @@ static MYX_SCHEMA_TABLE *find_table(MYX_CATALOGS *cats, const char *catalog,
     
     [colTypeCombo setStringValue:@""];
     [colDefaultText setStringValue:@""];
+    [colDefaultNull setState:NSOffState];
     [colFlagsList reloadData];
     
     [colCharsetPop selectItemWithTitle:@""];
@@ -423,7 +434,8 @@ static MYX_SCHEMA_TABLE *find_table(MYX_CATALOGS *cats, const char *catalog,
     [indNameText setStringValue:@""];
     [fkOnDeletePop selectItemAtIndex:0];
     [fkOnUpdatePop selectItemAtIndex:0];
-    [fkRefTablePop selectItemAtIndex:0];
+	if ([fkRefTablePop numberOfItems]>0)
+	  [fkRefTablePop selectItemAtIndex:0];
   }  
   [fkColumnList reloadData];  
 }
@@ -511,11 +523,19 @@ static MYX_SCHEMA_TABLE *find_table(MYX_CATALOGS *cats, const char *catalog,
       }      
       [aCell setImage:img];
     }
+    else if ([[aTableColumn identifier] isEqualTo:@"default"])
+    {
+      if (_table->columns[rowIndex].default_value_is_null)
+        [aCell setImage:_nullIcon];
+      else
+        [aCell setImage:nil];
+    }
   }
   else if (aTableView == colFlagsList)
   {
     int r= [columnList selectedRow];
-    [aCell setTitle:[NSString stringWithUTF8String:_table->columns[r].datatype_pointer->flags[rowIndex]]];
+    if (rowIndex < _table->columns[r].datatype_pointer->flags_num)
+      [aCell setTitle:[NSString stringWithUTF8String:_table->columns[r].datatype_pointer->flags[rowIndex]]];
   }
 }
 
@@ -558,7 +578,12 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
       return ms;
     }
     else if ([[aTableColumn identifier] isEqualTo:@"default"])
-      return NSStr(_table->columns[rowIndex].default_value);
+    {
+      if (_table->columns[rowIndex].default_value_is_null)
+        return @"";
+      else
+        return NSStr(_table->columns[rowIndex].default_value);
+    }
     else if ([[aTableColumn identifier] isEqualTo:@"comments"])
       return NSStr(_table->columns[rowIndex].comment);
   }
@@ -589,11 +614,16 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
   }
   else if (aTableView == indColumnList)
   {
-    int index= [indexList selectedRow];
+    int index= [indexList selectedRow];        
     if (index < 0)
-      return @"";
+      return nil;
     if ([[aTableColumn identifier] isEqualTo:@"column"])
-      return NSStr(_table->indices[index].columns[rowIndex].name);
+    {
+      int i= [[aTableColumn dataCell] indexOfItemWithTitle:NSStr(_table->indices[index].columns[rowIndex].name)];
+      if (i == NSNotFound)
+        return nil;
+      return [NSNumber numberWithInt:i];
+    }
     else
       return NSStr(_table->indices[index].columns[rowIndex].len);
   }
@@ -605,11 +635,21 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
   {
     int index= [fkList selectedRow];
     if (index < 0)
-      return 0;
+      return nil;
     if ([[aTableColumn identifier] isEqualTo:@"column"])
-      return NSStr(_table->fks[index].column_mapping[rowIndex].name);
+    {
+      int i= [[aTableColumn dataCell] indexOfItemWithTitle:NSStr(_table->fks[index].column_mapping[rowIndex].name)];
+      if (i < 0)
+        return nil;
+      return [NSNumber numberWithInt:i];
+    }
     else
-      return NSStr(_table->fks[index].column_mapping[rowIndex].value);
+    {
+      int i= [[aTableColumn dataCell] indexOfItemWithTitle:NSStr(_table->fks[index].column_mapping[rowIndex].value)];
+      if (i < 0)
+        return nil;
+      return [NSNumber numberWithInt:i];
+    }
   }
   return @"";
 }
@@ -627,6 +667,7 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
     {
       g_free(_table->columns[rowIndex].name);
       _table->columns[rowIndex].name= g_strdup([anObject UTF8String]);
+      [self columnListChanged];
     }
     else if ([[aTableColumn identifier] isEqualTo:@"type"])
     {
@@ -658,6 +699,9 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
     {
       g_free(_table->columns[rowIndex].default_value);
       _table->columns[rowIndex].default_value= g_strdup([anObject UTF8String]);
+      if ([colDefaultNull state] == NSOnState)
+        [colDefaultNull performClick:nil];
+      _table->columns[rowIndex].default_value_is_null= 0;
     }
     else if ([[aTableColumn identifier] isEqualTo:@"comments"])
     {
@@ -703,6 +747,32 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
     }
     [columnList reloadData];
   }
+  // FK Columns
+  else if (aTableView == fkColumnList)
+  {
+    int index= [fkList selectedRow];
+    int cindex= [fkColumnList selectedRow];
+    MYX_NAME_VALUE_PAIR *cdata= _table->fks[index].column_mapping+cindex;
+    int i= [anObject intValue];
+    NSString *name;
+    
+    if (i >= 0)
+    {
+      name= [[aTableColumn dataCell] itemTitleAtIndex:i];
+      
+      if ([[aTableColumn identifier] isEqualTo:@"column"])
+      {
+        g_free(cdata->name);
+        cdata->name= g_strdup([name UTF8String]);
+      }
+      else
+      {
+        g_free(cdata->value);
+        cdata->value= g_strdup([name UTF8String]);
+      }
+      [fkColumnList reloadData];
+    }
+  }  
   // Index Columns
   else if (aTableView == indColumnList)
   {
@@ -726,9 +796,124 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
       else
         cdata->len= g_strdup([anObject UTF8String]);
     }
+    [indColumnList reloadData];
   }
 }
 
+- (BOOL)tableView:(NSTableView *)tableView
+        writeRows:(NSArray *)rows
+     toPasteboard:(NSPasteboard *)pboard
+{
+  if (tableView == columnList)
+  {
+    _draggedRow= [[rows objectAtIndex:0] intValue];
+    
+    [pboard declareTypes:[NSArray arrayWithObject:MTableRowPboardType] owner:nil];
+    [pboard setData:[NSData data] forType:MTableRowPboardType];
+    
+    return YES;
+  }
+  else
+    return NO;
+}
+
+- (NSDragOperation)tableView:(NSTableView *)tableView
+                validateDrop:(id <NSDraggingInfo>)info
+                 proposedRow:(int)row
+       proposedDropOperation:(NSTableViewDropOperation)operation
+{
+  NSDragOperation op;
+  if (tableView == columnList)
+  {
+    if ([info draggingSource] == tableView && row != _draggedRow && row >= 0)
+      op= NSDragOperationPrivate;
+    else
+      op= NSDragOperationNone;
+  }
+  else if (tableView == fkColumnList && [fkColumnList isEnabled] && [fkList selectedRow]>=0)
+    op= NSDragOperationPrivate;
+  else if (tableView == indColumnList && [indColumnList isEnabled] && [indexList selectedRow]>=0)
+    op= NSDragOperationPrivate;
+  else 
+    op= NSDragOperationNone;
+  return op;
+}
+
+- (BOOL)tableView:(NSTableView *)tableView 
+       acceptDrop:(id <NSDraggingInfo>)info
+              row:(int)row 
+    dropOperation:(NSTableViewDropOperation)operation
+{
+  if (tableView == columnList)
+  {
+    MYX_DBM_COLUMN_DATA column;
+    
+    if (row >= _table->columns_num)
+      row= _table->columns_num-1;
+    
+    memcpy(&column, _table->columns+_draggedRow, sizeof(column));
+    
+    if (_draggedRow < row)
+    {
+      // I1
+      // I2 ---
+      // I3   |
+      // I4 <--
+      // I5
+      memmove(_table->columns+_draggedRow, _table->columns+_draggedRow+1,
+              (row-_draggedRow)*sizeof(column));
+      memcpy(_table->columns+row, &column, sizeof(column));
+    }
+    else
+    {
+      // I1
+      // I2 <--
+      // I3   |
+      // I4 ---
+      // I5    
+      memmove(_table->columns+row+1, _table->columns+row,
+              (_draggedRow-row)*sizeof(column));
+      memcpy(_table->columns+row, &column, sizeof(column));    
+    }
+    [tableView reloadData];
+    return YES;
+  }
+  else if (tableView == fkColumnList)
+  {
+    int index= [fkList selectedRow];
+    MYX_NAME_VALUE_PAIR *fdata;
+    int findex;
+    
+    [self addFKColumn:nil];
+    findex= [fkColumnList selectedRow];
+    fdata= _table->fks[index].column_mapping+findex;
+    
+    g_free(fdata->name);
+    fdata->name= g_strdup(_table->columns[_draggedRow].name);
+    fdata->value= NULL;
+    
+    [fkColumnList reloadData];
+    return YES;    
+  }
+  else if (tableView == indColumnList)
+  {
+    int cindex;
+    int index= [indexList selectedRow];
+    MYX_DBM_INDEX_COLUMN_DATA *cdata;
+    
+    [self addIndexColumn:nil];
+    cindex= [indColumnList selectedRow];
+    cdata= _table->indices[index].columns+cindex;
+    
+    g_free(cdata->name);
+    cdata->name= g_strdup(_table->columns[_draggedRow].name);
+    
+    [fkColumnList reloadData];
+    return YES;
+  }
+  else
+    return NO;
+}
 
 - (void)tableViewSelectionDidChange:(NSNotification *)aNotification
 {
@@ -738,21 +923,19 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
 
   if (table == columnList)
   {  
-    [colGroup setEnabledRecursive:enabled];
     if (row < 0)
     {
-      enabled= NO;
+      [colGroup setEnabledRecursive:NO];
       [self showColumn:0];
     }
     else
     {
-      enabled= YES;
+      [colGroup setEnabledRecursive:YES];
       [self showColumn:_table->columns+row];
     }
   }
   else if (table == indexList)
   {
-    [indBox setEnabledRecursive:enabled];
     if (row < 0)
     {
       enabled= NO;
@@ -763,10 +946,10 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
       enabled= YES;
       [self showIndex:_table->indices+row];
     }
+	[indBox setEnabledRecursive:enabled];
   }
   else if (table == fkList)
   {
-    [fkBox setEnabledRecursive:enabled];
     if (row < 0)
     {
       enabled= NO;
@@ -777,6 +960,7 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
       enabled= YES;
       [self showFK:_table->fks+row];
     }
+	[fkBox setEnabledRecursive:enabled];
   }  
 }
 
@@ -834,7 +1018,8 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
   else
   {
     _table= g_new0(MYX_DBM_TABLE_DATA, 1);
-    
+
+    _table->schema= g_strdup([_schemaName UTF8String]);
     _table->charset= g_strdup("utf8");
   }
 
@@ -942,7 +1127,6 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
     case MYX_DBM_TE_MYISAM:
       break;
     case MYX_DBM_TE_INNODB:
-      NSLog(@"!!!");
       fkEnabled= YES;
       break;
     case MYX_DBM_TE_MEMORY:
@@ -960,16 +1144,6 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
   [fkContainer setEnabledRecursive:fkEnabled];
 }
 
-- (NSImage*)loadImage: (NSString*)file
-{
-  NSImage *image= [[[NSImage alloc] initWithContentsOfFile: 
-    [[NSBundle bundleForClass:[self class]] pathForResource:file
-                                    ofType:@"png"]] autorelease];
-  if (!image)
-    NSLog(@"Could not load image %@", file);
-  return image;
-}
-
 @end
 
 //===
@@ -1040,7 +1214,7 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
     {
       NSString *tmp= [NSString stringWithUTF8String:script];
       
-      if ([self confirmScript:tmp])
+      if ((tmp= [self confirmScript:tmp]))
       {
         MYX_LIB_ERROR error;
         NSString *msg;
@@ -1050,7 +1224,7 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
         else
           msg= @"Error executing SQL commands to update table.";
         
-        myx_query_execute_direct(_mysql, script, &error);
+        myx_query_execute_direct(_mysql, [tmp UTF8String], &error);
         
         if (error == MYX_SQL_ERROR)
           MXRunAlertPanelWithMySQLError(@"Error", msg, _mysql);
@@ -1065,10 +1239,13 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
             NSRunAlertPanel(@"Edit Table", @"Modifications to Table '%@' Were Saved.",
                             @"OK", nil, nil, [NSString stringWithUTF8String:_table->name]);
           
+		  _tableName= [NSString stringWithUTF8String:_table->name];
+
           // reload newly saved data
           [self revertChanges];
-          
-          _tableName= [NSString stringWithUTF8String:_table->name];
+		  
+		  [[NSNotificationCenter defaultCenter] postNotificationName:MTableEditorTableDidChange
+															  object:self];
         }
       }
     }
@@ -1134,6 +1311,21 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
     g_free(_table->columns[column].collation);
     _table->columns[column].collation= g_strdup(collation);    
   }
+  else if (sender == colDefaultNull)
+  {
+    _table->columns[column].default_value_is_null= [sender state] == NSOnState;
+    if (_table->columns[column].default_value_is_null)
+    {
+      [colDefaultText setEnabled:NO];
+    }
+    else
+    {
+      g_free(_table->columns[column].default_value);
+      _table->columns[column].default_value= g_strdup("");
+      [colDefaultText setEnabled:YES];
+    }
+    [columnList reloadData];
+  }
   else if ([sender isMemberOfClass:[NSMatrix class]])
   {
     _table->columns[column].primary_key= [colPKCheck state] == NSOnState ? 1 : 0;
@@ -1251,7 +1443,7 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
     
     [[NSNotificationCenter defaultCenter] addObserver:self
                                              selector:@selector(handleTextEdited:)
-                                                 name:NSControlTextDidEndEditingNotification
+                                                 name:NSControlTextDidChangeNotification
                                                object:nil];
     [[self window] setDelegate:self];
   }
@@ -1302,7 +1494,7 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
   [button setTarget:self];
   [button setAction:addSel];
   [sv addSubview:button];
-  [sv addAccessoryView:button];
+  [sv addHorizontalAccessoryView:button];
   
   button= [[[NSButton alloc] initWithFrame:NSMakeRect(0,0,size,size)] autorelease];
   [button setButtonType: NSMomentaryChangeButton];
@@ -1313,12 +1505,14 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
   [button setTarget:self];
   [button setAction:delSel];
   [sv addSubview:button];
-  [sv addAccessoryView:button];
+  [sv addHorizontalAccessoryView:button];
 }
 
 
 - (void)awakeFromNib
 {
+  NSBundle *bundle= [NSBundle bundleForClass:[self class]];
+  
   [self setupButtonsForTableView:columnList 
                        addAction:@selector(addColumn:)
                     deleteAction:@selector(deleteColumn:)];
@@ -1336,18 +1530,26 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
                     deleteAction:@selector(deleteIndexColumn:)];
 
   [[columnList tableColumnWithIdentifier:@"nnull"] setHeaderCell:
-    [[NSTableHeaderCell alloc] initImageCell:[self loadImage:@"editor_table_not_null"]]];
+    [[NSTableHeaderCell alloc] initImageCell:MXGetImageFromBundle(bundle,@"editor_table_not_null.png")]];
   [[columnList tableColumnWithIdentifier:@"autoinc"] setHeaderCell:
-    [[NSTableHeaderCell alloc] initImageCell:[self loadImage:@"editor_table_auto_inc"]]];
+    [[NSTableHeaderCell alloc] initImageCell:MXGetImageFromBundle(bundle,@"editor_table_auto_inc.png")]];
 
-  _pkIcon= [[self loadImage:@"column_pk"] retain];
-  _columnIcon= [[self loadImage:@"column"] retain];
-  _blobIcon= [[self loadImage:@"datatype_blob"] retain];
-  _dateIcon= [[self loadImage:@"datatype_datetime"] retain];
-  _numericIcon= [[self loadImage:@"datatype_numeric"] retain];
-  _spatialIcon= [[self loadImage:@"datatype_spatial"] retain];
-  _stringIcon= [[self loadImage:@"datatype_string"] retain];
-  _userdefIcon= [[self loadImage:@"datatype_userdefined"] retain];
+  _pkIcon= [MXGetImageFromBundle(bundle,@"column_pk.png") retain];
+  _columnIcon= [MXGetImageFromBundle(bundle,@"column.png") retain];
+  _blobIcon= [MXGetImageFromBundle(bundle,@"datatype_blob.png") retain];
+  _dateIcon= [MXGetImageFromBundle(bundle,@"datatype_datetime.png") retain];
+  _numericIcon= [MXGetImageFromBundle(bundle,@"datatype_numeric.png") retain];
+  _spatialIcon= [MXGetImageFromBundle(bundle,@"datatype_spatial.png") retain];
+  _stringIcon= [MXGetImageFromBundle(bundle,@"datatype_string.png") retain];
+  _userdefIcon= [MXGetImageFromBundle(bundle,@"datatype_userdefined.png") retain];
+  
+  _nullIcon= [MXGetImageFromBundle(bundle,@"field_overlay_null.png") retain];
+  
+  [columnList registerForDraggedTypes:[NSArray arrayWithObject:MTableRowPboardType]];
+  [fkColumnList registerForDraggedTypes:[NSArray arrayWithObject:MTableRowPboardType]];
+  [indColumnList registerForDraggedTypes:[NSArray arrayWithObject:MTableRowPboardType]];
+  
+  [colDefaultNull setToolTip:@"Make default value NULL."];
 }
 
 
@@ -1397,9 +1599,9 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
     _tableName= nil;
   _catalogName= [catalog retain];
   _schemaName= [schema retain];
-
   // set up the Schema selection popup button
   [schemaPop removeAllItems];
+  
   for (c= 0; c < _catalogs->catalogs_num; c++)
   {
     MYX_CATALOG *cat= _catalogs->catalogs+c;
@@ -1476,9 +1678,10 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
 }
 
 
-- (BOOL)confirmScript:(NSString*)script
+- (NSString*)confirmScript:(NSString*)script
 {
   int rc;
+  NSString *final;
   
   [confirmText setString:script];
   
@@ -1490,9 +1693,14 @@ objectValueForTableColumn:(NSTableColumn *)aTableColumn
 
   rc= [NSApp runModalForWindow:confirmPanel];
   
+  if (rc == 1)
+	final= [[[confirmText string] retain] autorelease];
+  else
+	final= nil;
+  
   [confirmPanel close];
   
-  return rc == 1;
+  return final;
 }
 
 
