@@ -19,6 +19,31 @@
 
 #include "MGCellRendererBlob.h" // data2str()
 
+
+static MYXResultSetCallbacks callbacks= {
+  MGResultSetModel::row_added_callback,
+    MGResultSetModel::row_deleted_callback,
+    NULL
+};
+
+
+
+void MGResultSetModel::row_added_callback(MYXResultSet *rs, unsigned int row)
+{
+  Gtk::TreePath path;
+  path.push_back(row);
+  dynamic_cast<MGResultSetModel*>(rs)->row_inserted(path, dynamic_cast<MGResultSetModel*>(rs)->get_iter(path));
+}
+
+
+void MGResultSetModel::row_deleted_callback(MYXResultSet *rs, unsigned int row)
+{
+  Gtk::TreePath path;
+  path.push_back(row);
+  dynamic_cast<MGResultSetModel*>(rs)->row_deleted(path);
+}
+
+
     //XXX TODO fix the model stuff so that there's only 1 action per cell
     // when it's submited to apply_actions
 
@@ -26,11 +51,10 @@ MGResultSetModel::MGResultSetModel(const Gtk::TreeModelColumnRecord& columns,
                                    MYX_RESULTSET *resultset)
 : Glib::ObjectBase(typeid(MGResultSetModel)), //register a custom GType.
   Glib::Object(), //The custom GType is actually registered here.
+  MYXResultSet(resultset, callbacks),
   _stamp(1), //When the model's stamp != the iterator's stamp then that iterator is invalid and should be ignored. Also, 0=invalid
-  _resultset(resultset),
   _edit_enabled(false),
-  _disable_checks(true),
-  _new_rows(0)
+  _disable_checks(true)
 {
 //  Gtk::TreeModel::add_interface(Glib::Object::get_type());
 
@@ -62,7 +86,7 @@ Gtk::TreeModelFlags MGResultSetModel::get_flags_vfunc()
 
 int MGResultSetModel::get_n_columns_vfunc()
 {
-  return _resultset->columns_num*2+(int)SpecialLast;
+  return _resultset->columns_num_to_display*2+(int)SpecialLast;
 }
 
 
@@ -89,13 +113,8 @@ bool MGResultSetModel::iter_next_vfunc(GtkTreeIter* iter)
   if (_disable_checks || treeiter_valid(iter))
   {
     iter->stamp= _stamp;
-
-    iter->user_data= (gpointer)((int)iter->user_data+1);
-
-//    if ((unsigned)iter->user_data > _resultset->rows_num)
-//      g_message("QEWQWE!!!");
-
-    return (int)iter->user_data < (int)_resultset->rows_num+_new_rows ? true : false;
+    iter->user_data= (gpointer)((unsigned long)iter->user_data+1);
+    return (unsigned long)iter->user_data < get_row_count() ? true : false;
   }
   iter->stamp= 0;
   return false;
@@ -113,7 +132,7 @@ bool MGResultSetModel::iter_children_vfunc(GtkTreeIter* iter, const GtkTreeIter*
 
 bool MGResultSetModel::iter_has_child_vfunc(const GtkTreeIter* iter)
 {
-  if (!iter && _resultset && _resultset->rows_num+_new_rows>0)
+  if (!iter && _resultset && get_row_count() > 0)
   {
     return true;
   }
@@ -124,9 +143,7 @@ bool MGResultSetModel::iter_has_child_vfunc(const GtkTreeIter* iter)
 int MGResultSetModel::iter_n_children_vfunc(const GtkTreeIter* iter)
 {
   if (!iter)
-  {
-    return _resultset->rows_num + _new_rows;
-  }
+    return get_row_count();
   return 0;
 }
 
@@ -139,7 +156,6 @@ bool MGResultSetModel::iter_nth_child_vfunc(GtkTreeIter* iter, const GtkTreeIter
     iter->user_data= (gpointer)n;
     return true;
   }
-  g_message("!!! %p %i",parent,n);
   iter->stamp= 0;
   return false;
 }
@@ -161,7 +177,7 @@ void MGResultSetModel::ref_node_vfunc(GtkTreeIter* iter)
 
 void MGResultSetModel::unref_node_vfunc(GtkTreeIter* iter)
 {
-//  g_message("unrefnode");
+ // g_message("unrefnode");
 }
 
 
@@ -171,7 +187,7 @@ Gtk::TreeModel::Path MGResultSetModel::get_path_vfunc(const Gtk::TreeModel::iter
   {
     Path path;
 
-    path.push_back((int)iter.gobj()->user_data);
+    path.push_back((long)iter.gobj()->user_data);
     
     return path;
   }
@@ -185,7 +201,7 @@ bool MGResultSetModel::get_iter_vfunc(GtkTreeIter* iter, const Gtk::TreeModel::P
 
   if (len != 1)
   {
-    g_warning("bad path");
+    g_message("bad path");
     iter->stamp= 0;
     return false;
   }
@@ -196,7 +212,7 @@ bool MGResultSetModel::get_iter_vfunc(GtkTreeIter* iter, const Gtk::TreeModel::P
   {
     if (row_index >= _resultset->rows_num + _new_rows)
     {
-      iter->stamp= 0;
+      iter->stamp= 111;
       return false;
     }
   }
@@ -207,49 +223,25 @@ bool MGResultSetModel::get_iter_vfunc(GtkTreeIter* iter, const Gtk::TreeModel::P
 }
 
 
-
-bool MGResultSetModel::get_value(const Gtk::TreeModel::iterator& row, int column, gpointer &value, guint &value_length)
+bool MGResultSetModel::get_value(const Gtk::TreeModel::iterator& row, unsigned int column, gpointer &value, guint &value_length)
 {
   const GtkTreeIter *iter= row.gobj();
 
   if (_disable_checks || treeiter_valid(iter))
   {
-    int r= (int)iter->user_data;
+    unsigned long r= (unsigned long)iter->user_data;
 
-    if (column < (int)_resultset->columns_num*2)
+    if (column < _resultset->columns_num*2)
     {
-      // check if there's some changed version of the data
       switch (column % 2)
       {
       case 0: // data
         {
-          const char *svalue;
-          if (!_actions.empty() && find_change(r, column/2, svalue, value_length))
-          {
-            value= (gpointer)svalue;
-            return true;
-          }
-          
-          if ((unsigned int)iter->user_data < _resultset->rows_num)
-          {
-            MYX_RS_ROW *row= _resultset->rows + r;
-            
-            if (!row->fields)
-            {
-              value= NULL;
-              value_length= 0;
-            }
-            else
-            {
-              value= row->fields[(column/2)].value;
-              value_length= row->fields[(column/2)].value_length;
-            }
-          }
-          else
-          {
-            value= NULL;
-            value_length= 0;
-          }
+          char *v;
+          size_t l;
+          get(r, column/2, v, l);
+          value= v;
+          value_length= l;
         }
         break;
       case 1: // color
@@ -257,7 +249,7 @@ bool MGResultSetModel::get_value(const Gtk::TreeModel::iterator& row, int column
           char *color= NULL;
 
           // 1st check diff
-          if (r < (int)_resultset->rows_num)
+          if (r < _resultset->rows_num)
           {
             MYX_RS_ROW *row= _resultset->rows + r;
             switch (row->diff&MYX_RD_MASK)
@@ -277,21 +269,24 @@ bool MGResultSetModel::get_value(const Gtk::TreeModel::iterator& row, int column
           }
 
           // 2nd. changes color overrides diff color
-          if (_actions.find(r)!=_actions.end()) // find out state
+          switch (get_edit_status(r, column/2))
           {
-            MYX_RS_ACTION *action= get_action(*_actions[r].begin());
-            if (action && action->action == MYX_RSA_DELETE)
-            {
-              color= _state_colors[0];
-            }
-            else if (!action || action->action == MYX_RSA_ADD)
-              color= _state_colors[1];
-            else
-              color= _state_colors[2];
-          }
-          else
-          {
-            // nothing here, use default color
+          case MESUnchanged:
+            break;
+          case MESPlaceHolder:
+            color= _state_colors[3];
+            break;
+          case MESAdded:
+            color= _state_colors[1];
+            break;
+          case MESDeleted:
+            color= _state_colors[0];
+            break;
+          case MESChanged:
+            color= _state_colors[2];
+            break;
+          default:
+            g_message("SHIT!");
           }
 
           value= color;
@@ -315,7 +310,7 @@ void MGResultSetModel::get_value_vfunc(const Gtk::TreeModel::iterator& iter_, in
 
   if (_disable_checks || treeiter_valid(iter))
   {
-    int r= (int)iter->user_data;
+    unsigned long r= (unsigned long)iter->user_data;
     
     // from 0 to 2*_resultset->columns_num, the columns correspond
     // to the content (text or blob ptr) and color of the resultset,
@@ -335,15 +330,8 @@ void MGResultSetModel::get_value_vfunc(const Gtk::TreeModel::iterator& iter_, in
           /* this is for the editable flag */
           if (editable)
           {
-            // check whether this particular row can be edited
-            if (_actions.find(r)!=_actions.end())
-            {
-              MYX_RS_ACTION *action= get_action(*_actions[r].begin());
-              if (action && action->action == MYX_RSA_DELETE)
-              {
-                editable= false;
-              }
-            }
+            if (!get_row_editable(r))
+              editable= false;
           }
           g_value_init(value, G_TYPE_BOOLEAN);
           g_value_set_boolean(value, (gboolean)editable);
@@ -361,7 +349,7 @@ void MGResultSetModel::get_value_vfunc(const Gtk::TreeModel::iterator& iter_, in
     {
       bool tmp= _disable_checks;
       gpointer data= NULL;
-      guint size= 0;
+      gsize size= 0;
       bool is_blob;
       switch (_resultset->columns[column/2].column_type)
       {
@@ -380,7 +368,7 @@ void MGResultSetModel::get_value_vfunc(const Gtk::TreeModel::iterator& iter_, in
       {
         if ((column % 2) == 1) // color
         {
-          g_value_set_string(value, (char*)data);
+          g_value_set_string(value, (gchar*)data);
         }
         else
         {
@@ -411,10 +399,12 @@ void MGResultSetModel::set_value_impl(const Gtk::TreeModel::iterator& row, int c
 
 void MGResultSetModel::set_value(const Gtk::TreeModel::iterator& row, int column, const gpointer value, guint value_length)
 {
+  unsigned long row_num= (unsigned long)row.gobj()->user_data;
+  
   switch ((column%2))
   {
   case 0:
-    add_change((unsigned int)row.gobj()->user_data, column/2, value, value_length);
+    set(row_num, column/2, (char*)value, value_length);
     break;
   case 1:
     // color, ignore
@@ -443,7 +433,12 @@ void MGResultSetModel::notify_resultset_changed()
 
 void MGResultSetModel::set_editable(bool flag)
 {
-  _edit_enabled= flag;
+  if (_edit_enabled != flag)
+  {
+    _edit_enabled= flag;
+    
+    set_placeholder_enabled(flag);
+  }
 }
 
 
@@ -481,170 +476,14 @@ void MGResultSetModel::set_diff_colors(const std::vector<Glib::ustring> &colors)
 
 MYX_RS_ROW *MGResultSetModel::get_iter_row(const Gtk::TreeModel::iterator &iter)
 {
-  return _resultset->rows + (int)iter.gobj()->user_data;
-}
-
-
-MYX_RS_ACTION *MGResultSetModel::get_action(int action_index)
-{
-  if (action_index < 0) // -1 = new row marker
-    return NULL;
-  return _resultset->actions->actions+action_index;
-}
-
-
-void MGResultSetModel::add_change(int row, int column, const gpointer data, guint size)
-{
-  MYX_RS_ACTION *action;
-  int index;
-
-  // check whether the row was deleted
-  if (_actions.find(row)!=_actions.end())
-  {
-    action= get_action(*_actions[row].begin());
-    if (action && action->action == MYX_RSA_DELETE)
-    {
-      g_warning("trying to modify a deleted row!");
-      return;
-    }
-  }
-
-  // check whether the new value is the same as the old
-  if (row < (int)_resultset->rows_num)
-  {
-    if (_resultset->rows[row].fields == NULL)
-    {
-      if (size == 0)
-        return;
-    } else if (_resultset->rows[row].fields[column].value_length == size
-               && memcmp(_resultset->rows[row].fields[column].value, data, size) == 0)
-        return; // no change
-  }
-  else
-  {
-    const char *d;
-    unsigned int s;
-
-    // check whether the new value is the same as the old
-    if (find_change(row, column, d, s) && s == size && memcmp(d, data, s)==0)
-      return; // no change
-  }
-
-  action= myx_query_create_action(row >= (int)_resultset->rows_num ? MYX_RSA_ADD : MYX_RSA_UPDATE,
-                                  row, _resultset->columns+column,
-                                  size, (const char*)data);
-
-  index= myx_query_add_action(_resultset, action);
-
-  myx_query_free_action(action);
-
-  _actions[row].push_front(index);
-}
-
-
-bool MGResultSetModel::find_change(int row, int column,
-                                   const char *&data, unsigned int &size)
-{
-  ActionList::iterator rowa= _actions.find(row);
-
-  if (rowa!=_actions.end())
-  {
-    for (std::list<int>::const_iterator iter= rowa->second.begin();
-         iter != rowa->second.end(); ++iter)
-    {
-      MYX_RS_ACTION *action= get_action(*iter);
-      if (!action)
-      {
-        // new row
-        data= "";
-        size= 0;
-        return true;
-      } 
-      else if (action->action == MYX_RSA_DELETE)
-        return false;
-      else if (action->column - _resultset->columns == column)
-      {
-        data= action->new_value;
-        size= action->new_value_length;
-        return true;
-      }
-    }
-  }
-  return false;
-}
-
-
-void MGResultSetModel::apply_edits()
-{
-  Gtk::TreeModel::Path path;
-
-  for (ActionList::iterator iter= _actions.begin();
-       iter != _actions.end(); ++iter)
-  {
-    MYX_RS_ACTION *action;
-    path.clear();
-    path.push_back(iter->first);
-    action= get_action(*iter->second.begin());
-    if (action && action->action == MYX_RSA_DELETE && action->status == MYX_RSAS_APPLIED)
-    {
-      row_deleted(path);
-    }
-  }
-  _actions.clear();
-  _new_rows= 0;
-
-  myx_query_update_resultset(_resultset);
-
-  if (_resultset->actions)
-  {
-    for (unsigned int i= 0; i < _resultset->actions->actions_num; i++)
-    {
-      MYX_RS_ACTION *action= _resultset->actions->actions+i;
-
-      _actions[action->row].push_front(i);
-      if (action->row >= _resultset->rows_num)
-        _new_rows= std::max((unsigned int)_new_rows, action->row - _resultset->rows_num + 1);
-    }
-  }
-}
-
-
-void MGResultSetModel::discard_edits()
-{
-  Gtk::TreeModel::Path path;
-
-  for (ActionList::const_iterator iter= _actions.begin();
-       iter != _actions.end(); ++iter)
-  {
-    MYX_RS_ACTION *action;
-    path.clear();
-    path.push_back(iter->first);
-    action= get_action(*iter->second.begin());
-    if (!action || action->action == MYX_RSA_ADD)
-      row_deleted(path);
-    else
-      row_changed(path, get_iter(path));
-  }
-
-  _actions.clear();
-  myx_query_discard_actions(_resultset);
-  _new_rows= 0;
+  return _resultset->rows + (long)iter.gobj()->user_data;
 }
 
 
 Gtk::TreeModel::iterator MGResultSetModel::append()
 {
   Gtk::TreeModel::iterator iter(this);
-  int new_row;
-
-  new_row= _resultset->rows_num + _new_rows;
-  _new_rows++;
-
-  if (_actions.find(new_row)!=_actions.end())
-    g_warning("trying to add row that already exists?");
-
-  // insert new row marker
-  _actions[new_row].push_front(-1);
+  unsigned int new_row= add_row();
 
   // make new iterator
   iter.gobj()->stamp= _stamp;
@@ -659,68 +498,7 @@ Gtk::TreeModel::iterator MGResultSetModel::append()
 
 void MGResultSetModel::erase(const Gtk::TreeModel::iterator &iter)
 {
-  unsigned int row= (unsigned int)iter.gobj()->user_data;
-  MYX_RS_ACTION *action;
-  int index;
+  unsigned long row= (unsigned long)iter.gobj()->user_data;
 
-  action= myx_query_create_action(MYX_RSA_DELETE, row, NULL, 0, NULL);
-
-  index= myx_query_add_action(_resultset, action);
-
-  myx_query_free_action(action);
-
-  _actions[row].push_front(index);
+  delete_row(row);
 }
-
-
-void MGResultSetModel::undo(const Gtk::TreeModel::iterator &iter)
-{
-  int row= (int)iter.gobj()->user_data;
-
-  if (_actions.find(row)==_actions.end())
-  {
-    g_warning("trying to undo when there's nothing to undo");
-    return;
-  }
-
-  if (*_actions[row].begin() >= 0)
-    myx_query_delete_action(_resultset, *_actions[row].begin());
-
-  int index= *_actions[row].begin();
-  
-  _actions[row].erase(_actions[row].begin());
-  if (_actions[row].empty())
-    _actions.erase(_actions.find(row));
-
-  // now this sucks... but we have to update all indexes > index
-  // since they were shifted in _resultset->actions
-  for (ActionList::iterator it= _actions.begin(); it != _actions.end(); ++it)
-  {
-    for (std::list<int>::iterator jter= it->second.begin();
-         jter != it->second.end(); ++jter)
-    {
-      if (*jter > index)
-        (*jter)--;
-    }
-  }
-
-  if (row > (int)_resultset->rows_num && _actions.find(row)==_actions.end())
-  {
-    // emit deleted signal if this was a new row
-    row_deleted(get_path(iter));
-  }
-  else
-  {
-    row_changed(get_path(iter), iter);
-  }
-}
-
-
-bool MGResultSetModel::row_undoable(const Gtk::TreeModel::iterator &iter)
-{
-  int row= (int)iter.gobj()->user_data;
-  if (_actions.find(row)==_actions.end())
-    return false;
-  return true;
-}
-
