@@ -57,7 +57,7 @@ void MQFunctionBrowser::refresh_list(const Glib::ustring &filter)
     {      
       MYX_SQL_FUNCTION *func= group->functions+j;
       
-      if (filter.empty() || myx_match_pattern(func->caption, filter.c_str(), 0)>0)
+      if (filter.empty() || myx_match_pattern(func->caption, filter.c_str(), 0, 0)>0)
       {
         Gtk::TreeIter iter= _store->append(grow.children());
         Gtk::TreeRow row= *iter;
