@@ -206,7 +206,7 @@ void MQHistoryBrowser::refresh_list(const Glib::ustring &filter)
 
     for (unsigned int j= 0; j < tree->history_intervals_num; j++)
       if (tree->history_intervals[j].interval_type < sizeof(order)/sizeof(MYX_HISTORY_INTERVAL_TYPE)
-          && tree->history_intervals[j].interval_type == (int)order[i])
+          && tree->history_intervals[j].interval_type == (unsigned int)order[i])
       {
         interv= tree->history_intervals+j;
         break;
