@@ -24,9 +24,10 @@
 
 #include <sigc++/class_slot.h>
 
+#include <gtkmm/main.h>
 
 MQQueryDispatcher::Request::Request()
-  : keep_open(false), tmp_mysql(0)
+  : keep_open(false), tmp_mysql(0), warnings(0)
 {
 }
 
@@ -40,13 +41,13 @@ void MQQueryDispatcher::QueryRequest::finish()
 {
   if (got_error)
   {
-    error_cb(result,mysql_error,mysql_errno);
+    error_cb(result,warnings);
     if (tmp_mysql)
       myx_mysql_close(tmp_mysql);
   }
   else
   {
-    finish_cb(result);
+    finish_cb(result,warnings);
     if (!result)
     {
 //XXX verificar isso (obsoleto depois do callback?)
@@ -70,13 +71,12 @@ void MQQueryDispatcher::QueryRequest::execute(MYSQL *mysql)
                             MQQueryDispatcher::before_realloc,
                             MQQueryDispatcher::after_realloc);
 
-  if (error != MYX_NO_ERROR)
-  {
-    mysql_error= myx_mysql_error(mysql);
-    mysql_errno= myx_mysql_errno(mysql);
+  warnings= myx_mysql_error_msgs_fetch(mysql);
 
+  if (error != MYX_NO_ERROR && error != MYX_STOP_EXECUTION)
     got_error= true;
-  }
+  else
+    got_error= false;
 }
 
 
@@ -105,6 +105,50 @@ void MQQueryDispatcher::SaveRequest::finish()
 }
 
 
+
+class TableFetchRequest : public MQQueryDispatcher::Request {
+    Glib::ustring catalog;
+    Glib::ustring schema;
+  public:
+    TableFetchRequest(const Glib::ustring &catalog,
+                      const Glib::ustring &schema,
+                      MYX_SCHEMA_TABLES **tbls)
+      : catalog(catalog), schema(schema), tables(tbls) {};
+    MYX_SCHEMA_TABLES **tables;
+
+    virtual void execute(MYSQL *mysql)
+    {
+      *tables= myx_get_schema_tables(mysql, catalog.c_str(), schema.c_str());
+    }
+
+    virtual void finish()
+    {
+      Gtk::Main::instance()->quit();
+    }
+};
+
+
+class SPFetchRequest : public MQQueryDispatcher::Request {
+    Glib::ustring catalog;
+    Glib::ustring schema;
+  public:
+    SPFetchRequest(const Glib::ustring &catalog,
+                   const Glib::ustring &schema,
+                   MYX_SCHEMA_STORED_PROCEDURES **sps)
+      : catalog(catalog), schema(schema), procedures(sps) {};
+    MYX_SCHEMA_STORED_PROCEDURES **procedures;
+
+    virtual void execute(MYSQL *mysql)
+    {
+      *procedures= myx_get_schema_sps(mysql, catalog.c_str(), schema.c_str());
+    }
+
+    virtual void finish()
+    {
+      Gtk::Main::instance()->quit();
+    }
+};
+
 //----------------------------------------------------------------------
 
 
@@ -204,7 +248,7 @@ void MQQueryDispatcher::start_query(const Glib::ustring &query,
 }
 
 
-bool MQQueryDispatcher::execute(const Glib::ustring &command)
+bool MQQueryDispatcher::execute(const Glib::ustring &command, bool report_errors)
 {
   MYX_LIB_ERROR error;
   
@@ -219,9 +263,10 @@ bool MQQueryDispatcher::execute(const Glib::ustring &command)
   }
   else
   {
-    myg_show_mysql_error(*static_cast<Gtk::Window*>(_owner),
-                         ufmt(_("Error while execeuting query: %s"),
-                              command.c_str()), _mysql);
+    if (report_errors)
+      myg_show_mysql_error(*static_cast<Gtk::Window*>(_owner),
+                           ufmt(_("Error while execeuting query: %s"),
+                                command.c_str()), _mysql);
 
     _mysql_mutex.unlock();
 
@@ -275,6 +320,8 @@ int MQQueryDispatcher::progress_row_fetch(unsigned long cur_count,
 
 void MQQueryDispatcher::normal_thread(Request *req)
 {
+  mysql_thread_init();
+
   MYSQL *mysql= req->tmp_mysql ? req->tmp_mysql : myx_mysql_init();
 
   MYX_USER_CONNECTION conn;
@@ -286,8 +333,7 @@ void MQQueryDispatcher::normal_thread(Request *req)
   }
   else
   {
-    req->mysql_error= myx_mysql_error(mysql);
-    req->mysql_errno= myx_mysql_errno(mysql);
+    req->warnings= myx_mysql_error_msgs_fetch(mysql);
 
     req->got_error= true;
   }
@@ -302,11 +348,14 @@ void MQQueryDispatcher::normal_thread(Request *req)
   _mysql_queue_result_mutex.unlock();
 
   _thread_dispatcher.emit();
+
+  mysql_thread_end();
 }
 
 
 void MQQueryDispatcher::mysql_query_thread()
 {
+  mysql_thread_init();
   for (;;) 
   {
     Request *req;
@@ -335,6 +384,7 @@ void MQQueryDispatcher::mysql_query_thread()
     // notify result ready
     _thread_dispatcher.emit();
   }
+  mysql_thread_end();
 }
 
 
@@ -343,7 +393,8 @@ void MQQueryDispatcher::pre_query_hook(MYSQL *mysql, void *cdata, const char *qu
   MQQueryDispatcher *disp= (MQQueryDispatcher*)cdata;
 
   disp->_hook_queue_mutex.lock();
-  disp->_hook_queue.push(std::pair<MYSQL*,Glib::ustring>(mysql, Glib::ustring(query, length)));
+  // we have to convert in 2 steps, because length is in bytes and ustring takes char. count
+  disp->_hook_queue.push(std::pair<MYSQL*,Glib::ustring>(mysql, Glib::ustring(std::string(query, length))));
   disp->_hook_queue_mutex.unlock();
 
   disp->_hook_queue_ready.emit();
@@ -466,24 +517,35 @@ MYX_CATALOGS *MQQueryDispatcher::get_catalogs()
 MYX_SCHEMA_TABLES *MQQueryDispatcher::get_tables(const Glib::ustring &catalog,
                                                  const Glib::ustring &schema)
 {
-  MYX_SCHEMA_TABLES *tables;
-  
-  _mysql_mutex.lock();
-  tables= myx_get_schema_tables(_mysql, catalog.c_str(), schema.c_str());
-  _mysql_mutex.unlock();
+  MYX_SCHEMA_TABLES *tables= 0;
+  TableFetchRequest *req= new TableFetchRequest(catalog, schema, &tables);
 
+  _mysql_queue_mutex.lock();
+  _mysql_queue.push(req);
+  _mysql_queue_mutex.unlock();
+
+  _mysql_queue_ready.signal();
+
+  Gtk::Main::instance()->run();
+  
   return tables;
 }
 
+
 MYX_SCHEMA_STORED_PROCEDURES *MQQueryDispatcher::get_sps(const Glib::ustring &catalog,
                                                          const Glib::ustring &schema)
 {
-  MYX_SCHEMA_STORED_PROCEDURES *sps;
+  MYX_SCHEMA_STORED_PROCEDURES *sps= 0;
+  SPFetchRequest *req= new SPFetchRequest(catalog, schema, &sps);
 
-  _mysql_mutex.lock();
-  sps= myx_get_schema_sps(_mysql, catalog.c_str(), schema.c_str());
-  _mysql_mutex.unlock();
+  _mysql_queue_mutex.lock();
+  _mysql_queue.push(req);
+  _mysql_queue_mutex.unlock();
 
+  _mysql_queue_ready.signal();
+
+  Gtk::Main::instance()->run();
+  
   return sps;
 }
 
