@@ -54,7 +54,7 @@ bool MQResultSetView::PopupTreeView::on_button_press_event(GdkEventButton *ev)
     Gtk::TreePath path;
     Gtk::TreeViewColumn *column;
     int cx, cy;
-    
+
     if (get_path_at_pos((gint)ev->x, (gint)ev->y,
                     path, column,
                     cx, cy))
@@ -74,11 +74,12 @@ bool MQResultSetView::PopupTreeView::on_button_press_event(GdkEventButton *ev)
 
 
 bool MQResultSetView::show_menu(GdkEventButton *ev)
-{
-  update_menu();
-  
+{  
   if ((ev->type == GDK_BUTTON_PRESS) && (ev->button == 3))
+  {
+    update_menu();
     _edit_menu.popup(ev->button, ev->time);
+  }
   else if ((ev->type == GDK_BUTTON_PRESS) && (ev->button == 1))
     set_active(true);
   
@@ -86,6 +87,69 @@ bool MQResultSetView::show_menu(GdkEventButton *ev)
 }
 
 
+bool MQResultSetView::tree_key_released(GdkEventKey *event)
+{
+  Gtk::TreeViewColumn *column= 0, *previous= 0;
+  Gtk::TreePath path;
+
+  _tree->get_cursor(path, column);
+
+  if (event->keyval == GDK_Tab && column)
+  {
+    std::list<Gtk::TreeViewColumn*> columns= _tree->get_columns();
+    bool found= false;
+
+    if (event->state & GDK_SHIFT_MASK)
+    {
+      std::list<Gtk::TreeViewColumn*>::const_iterator iter= columns.begin();
+      ++iter;
+
+      for (; iter != columns.end(); ++iter)
+      {
+        if (*iter == column)
+        {
+          found= true;
+          break;
+        }
+        previous= *iter;
+      }
+      column= previous;
+      
+      if (!found)
+        path.prev();
+      _tree->set_cursor(path, *column, true);
+    }
+    else
+    {
+      for (std::list<Gtk::TreeViewColumn*>::const_iterator iter= columns.begin();
+           iter != columns.end(); ++iter)
+      {
+        if (previous == column)
+        {
+          found= true;
+          column= *iter;
+          break;
+        }
+        previous= *iter;
+      }
+
+      if (!found)
+      {
+        path.next();
+        _tree->set_cursor(path, *_tree->get_column(1), true);
+        _tree->scroll_to_cell(path, *_tree->get_column(1));
+      }
+      else
+      {
+        _tree->set_cursor(path, *column, true);
+        _tree->scroll_to_cell(path, *column);
+      }
+    }
+    return true;
+  }
+  return false;
+}
+
 MQResultSetView::MQResultSetView(MQQueryDispatcher *disp,
                                  MQHistory *history,
                                  MQGlobalQueryParameters *params)
@@ -114,6 +178,15 @@ MQResultSetView::MQResultSetView(MQQueryDispatcher *disp,
   _scroll.set_shadow_type(Gtk::SHADOW_IN);
   _scroll.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
 
+  {
+    Gdk::Color color1("white"), color2("#f0f0f0");
+  
+    get_colormap()->alloc_color(color1);
+    get_colormap()->alloc_color(color2);
+
+    _ebox.modify_bg(Gtk::STATE_NORMAL, color1);
+    _ebox.modify_bg(Gtk::STATE_INSENSITIVE, color2);
+  }
   _status_label.set_padding(4, 0);
 
   _status_label.set_text("");
@@ -165,7 +238,7 @@ MQResultSetView::MQResultSetView(MQQueryDispatcher *disp,
   _search_btn.set_relief(Gtk::RELIEF_NONE);
   myg_make_image_button(_search_btn, PIXCACHE->load("rs_search.png"),
                         _("Search"));
-
+  
   set_sensitive(false);
   
   show_all();
@@ -253,7 +326,7 @@ void MQResultSetView::append_error(int num, const Glib::ustring &msg,
   Gtk::TreeIter iter= _msgstore->append();
   Gtk::TreeRow row= *iter;
   Glib::ustring errmsg= msg;
-    
+
   if (num == 1046)
   {
     errmsg+=_(". Please select one in File->Select Schema");
@@ -342,6 +415,7 @@ void MQResultSetView::display_result(MYX_RESULTSET *result)
 
     _tree= new PopupTreeView(&_edit_menu);
     
+    _tree->signal_key_release_event().connect(SigC::slot(*this,&MQResultSetView::tree_key_released));
     _tree->set_data("owner", this);
 
     _model= MGResultSetModel::create(*_columns, result);
@@ -349,6 +423,7 @@ void MQResultSetView::display_result(MYX_RESULTSET *result)
     colors.push_back("#cc5555"); // deleted
     colors.push_back("#55bb55"); // added
     colors.push_back("#cccc55"); // modified
+    colors.push_back("#eeffee"); // placeholder for new row
     _model->set_state_colors(colors);
 
     colors.clear();
@@ -398,7 +473,8 @@ void MQResultSetView::display_result(MYX_RESULTSET *result)
                                 PIXCACHE->load("field_overlay_view.png"));
         rend->set_blob_icon(PIXCACHE->load("blob_icon.png"));
         rend->set_max_text_width(prefs.max_blob_length);
-        rend->signal_edited().connect(SigC::bind<int>(SigC::slot(*this,&MQResultSetView::blob_data_edited), (i)*2));
+        rend->property_column()= i;
+        rend->set_delegate(this);
 
         _tree->append_column("", *rend);
         col= _tree->get_column(i+1);
@@ -457,6 +533,8 @@ void MQResultSetView::display_result(MYX_RESULTSET *result)
     }
     _tree->get_selection()->set_mode(Gtk::SELECTION_BROWSE);
     _tree->get_selection()->signal_changed().connect(SigC::slot(*this,&MQResultSetView::selected_row));
+    go_first();
+    selected_row();
 
     _tree->signal_will_popup().connect(SigC::slot(*this,&MQResultSetView::update_menu));
     
@@ -569,6 +647,8 @@ bool MQResultSetView::explain_query()
   
   _query_finished_signal.emit(this);
 
+  _is_busy= false;
+  
   set_sensitive(true);
 
   if (result)
@@ -721,6 +801,13 @@ int MQResultSetView::get_history_index()
 
 void MQResultSetView::update_menu()
 {
+  bool is_field;
+  Gtk::TreePath path;
+  int column;
+
+  if (_tree)
+    is_field= get_selected_field(path, column);
+  
   if (_model && _model->get_editable())
   {
     Gtk::TreeIter iter= _tree->get_selection()->get_selected();
@@ -740,7 +827,7 @@ void MQResultSetView::update_menu()
       myg_menu_set_sensitive(_edit_menu, "undo", true);
      */
 
-    if (iter)
+    if (iter && is_field)
       flag= true;
     else
       flag= false;
@@ -759,7 +846,7 @@ void MQResultSetView::update_menu()
     myg_menu_set_sensitive(_edit_menu, "load_field", false);
     
     bool flag;
-    if (_model && _tree->get_selection()->get_selected())
+    if (_model && _tree->get_selection()->get_selected() && is_field)
       flag= true;
     else
       flag= false;
@@ -795,7 +882,7 @@ void MQResultSetView::selected_error()
   {
     Gtk::TreeRow row= *iter;
     int rowi= row[_msg_columns.row];
-    if (_tree)
+    if (_tree && rowi >= 0)
     {
       Gtk::TreePath path;
       path.push_back(rowi);
@@ -805,7 +892,7 @@ void MQResultSetView::selected_error()
 }
 
 
-void MQResultSetView::query_finished(MYX_RESULTSET *result)
+void MQResultSetView::query_finished(MYX_RESULTSET *result, MYX_MYSQL_ERROR_MSGS *warnings)
 {
   char *delay;
   struct timeval tv;
@@ -836,6 +923,32 @@ void MQResultSetView::query_finished(MYX_RESULTSET *result)
     _status_label.set_text(ufmt(_("Query executed in %s"), delay));
   g_free(delay);
 
+  if (warnings != 0)
+  {
+    for (unsigned int i= 0; i < warnings->errors_num; i++)
+    {
+      switch (warnings->errors[i].level)
+      {
+      case MYX_QEL_ERROR:
+        append_error(warnings->errors[i].error,
+                     warnings->errors[i].text, 
+                     PIXCACHE->load("rs_error.png"));
+        break;
+      case MYX_QEL_WARNING:
+        append_error(warnings->errors[i].error,
+                     warnings->errors[i].text, 
+                     PIXCACHE->load("rs_warning.png"));
+        break;
+      case MYX_QEL_NOTE:
+        append_error(warnings->errors[i].error,
+                     warnings->errors[i].text, 
+                     PIXCACHE->load("rs_notice.png"));
+        break;
+      }
+    }
+    myx_mysql_error_msgs_free(warnings);
+  }
+  
   display_result(result);
 
   _query_finished_signal.emit(this);
@@ -862,9 +975,9 @@ bool MQResultSetView::query_progress(MYX_RESULTSET *result,
 }
 
 
-void MQResultSetView::query_error(MYX_RESULTSET *result, const Glib::ustring &error, int errn)
+void MQResultSetView::query_error(MYX_RESULTSET *result, MYX_MYSQL_ERROR_MSGS *errors)
 {
-  if (errn == 0)
+  if (!errors)
   {
     if (result)
       _status_label.set_text(ufmt(_("Query cancelled. %i rows were fetched."),
@@ -882,9 +995,30 @@ void MQResultSetView::query_error(MYX_RESULTSET *result, const Glib::ustring &er
 
   _query_finished_signal.emit(this);
 
-  if (errn != 0)
+  if (errors != 0)
   {
-    append_error(errn, error, PIXCACHE->load("rs_error.png"));
+    for (unsigned int i= 0; i < errors->errors_num; i++)
+    {
+      switch (errors->errors[i].level)
+      {
+      case MYX_QEL_ERROR:
+        append_error(errors->errors[i].error,
+                     errors->errors[i].text, 
+                     PIXCACHE->load("rs_error.png"));
+        break;
+      case MYX_QEL_WARNING:
+        append_error(errors->errors[i].error,
+                     errors->errors[i].text, 
+                     PIXCACHE->load("rs_warning.png"));
+        break;
+      case MYX_QEL_NOTE:
+        append_error(errors->errors[i].error,
+                     errors->errors[i].text, 
+                     PIXCACHE->load("rs_notice.png"));
+        break;
+      }
+    }
+    myx_mysql_error_msgs_free(errors);
   }
 }
 
@@ -914,16 +1048,31 @@ void MQResultSetView::save_finished(MYX_RS_ACTION_ERRORS *errors)
   
   if (errors || has_failed_actions)
   {
-    _model->apply_edits();
+    _model->post_commit(true);
 
     _msgstore->clear();
     if (errors)
     {
       for (unsigned int i= 0; i < errors->errors_num; i++)
       {
-        append_error(errors->errors[i].error,
-                     _("Error saving changed query. ")+Glib::ustring(errors->errors[i].error_text),
-                     PIXCACHE->load("rs_error.png"), errors->errors[i].action->row);
+        switch (errors->errors[i].level)
+        {
+        case MYX_QEL_ERROR:
+          append_error(errors->errors[i].error,
+                       _("Error saving changed query. ")+Glib::ustring(errors->errors[i].error_text),
+                       PIXCACHE->load("rs_error.png"), errors->errors[i].action->row);
+          break;
+        case MYX_QEL_WARNING:
+          append_error(errors->errors[i].error,
+                       Glib::ustring(errors->errors[i].error_text),
+                       PIXCACHE->load("rs_warning.png"), errors->errors[i].action->row);
+          break;
+        case MYX_QEL_NOTE:
+          append_error(errors->errors[i].error,
+                       Glib::ustring(errors->errors[i].error_text),
+                       PIXCACHE->load("rs_notice.png"), errors->errors[i].action->row);
+          break;
+        }
       }
       Gtk::TreePath path;
       path.push_back(_msgstore->children().size()-1);
@@ -942,7 +1091,8 @@ void MQResultSetView::save_finished(MYX_RS_ACTION_ERRORS *errors)
   }
   else
   {
-    _model->apply_edits();
+    _model->post_commit(false);
+    _model->set_editable(false);
 
     // refresh
     _tree->queue_draw();
@@ -951,6 +1101,9 @@ void MQResultSetView::save_finished(MYX_RS_ACTION_ERRORS *errors)
     
     _msgscroll.hide();
   }
+  
+  _tree->set_model(Gtk::ListStore::create(_msg_columns));
+  _tree->set_model(_model);
 }
 
 
@@ -1035,8 +1188,11 @@ void MQResultSetView::edit_start()
   else
   {
     _model->set_editable(false);
-    _model->discard_edits();
+    _model->discard_changes();
     update_edit_button_sensitivity();
+
+    _tree->set_model(Gtk::ListStore::create(_msg_columns));
+    _tree->set_model(_model);
     
     _tree->queue_draw();
     
@@ -1044,13 +1200,15 @@ void MQResultSetView::edit_start()
   }
 }
 
-
 void MQResultSetView::edit_apply()
 {
   _model->set_editable(false);
-  
+  _model->pre_commit();
   update_edit_button_sensitivity();
-  
+
+  _tree->set_model(Gtk::ListStore::create(_msg_columns));
+  _tree->set_model(_model);
+
   if (_result->actions && _result->actions->actions_num > 0)
   {
     _query_started_signal.emit(this, true);
@@ -1058,6 +1216,12 @@ void MQResultSetView::edit_apply()
     _dispatcher->save_edits(_result,
                             SigC::slot(*this,&MQResultSetView::save_finished));
   }
+  else
+  {
+    _query_started_signal.emit(this, true);
+    
+    save_finished(NULL);
+  }
   _editable_change_signal.emit();
 }
 
@@ -1109,7 +1273,8 @@ void MQResultSetView::set_active(bool flag)
 
     _scroll.queue_draw();
   }
-  _status_label.set_sensitive(flag);
+
+  _ebox.set_sensitive(flag);
 
   if (flag)
     _activate_signal.emit(this);
@@ -1150,26 +1315,49 @@ void MQResultSetView::save_cell_change(MGBlobEditor *editor,
 }
 
 
-void MQResultSetView::column_action_mi(FieldAction action)
+bool MQResultSetView::get_selected_field(Gtk::TreePath &path, int &column)
 {
-  Gtk::TreePath path;
-  Gtk::TreeViewColumn *column;
-  unsigned int column_index= 0;
-  bool is_blob= false;
+  Gtk::TreeViewColumn *col= 0;
+
+  if (!_tree->get_selection()->get_selected())
+    return false;
   
-  _tree->get_cursor(path, column);
+  _tree->get_cursor(path, col);
+
+  if (!col || path.empty())
+    return false;
+
+  column= -1;
   
   for (unsigned int i= 0; ; i++)
   {
     Gtk::TreeViewColumn *tmp= _tree->get_column(i);
     if (!tmp)
       break;
-    if (column == tmp)
+    if (col == tmp)
     {
-      column_index= i-1;
+      column= i-1;
       break;
     }
   }
+  if (column < 0)
+    return false;
+
+  return true;
+}
+
+
+void MQResultSetView::column_action_mi(FieldAction action)
+{
+  Gtk::TreePath path;
+  int column_index= 0;
+  bool is_blob= false;
+  Gtk::TreeViewColumn *column;
+
+  if (!get_selected_field(path, column_index))
+    return;
+  
+  column= _tree->get_column(column_index+1);
 
   if (_result->columns[column_index].column_type == MYX_RSCT_BLOB
       || _result->columns[column_index].column_type == MYX_RSCT_TEXT)
@@ -1285,7 +1473,7 @@ void MQResultSetView::column_action_mi(FieldAction action)
     if (is_blob)
     {
       ((CellRendererBlob*)column->get_first_cell_renderer())->view_clicked(get_parent(),
-                                                                             path.to_string());
+                                                                           path.to_string());
     }
     else
     {
@@ -1386,7 +1574,7 @@ void MQResultSetView::undo_row_mi()
 
   if (iter)
   {
-    _model->undo(iter);
+//    _model->undo(iter);
   }
 }
 
@@ -1645,6 +1833,157 @@ void MQResultSetView::set_compact_editbar(bool flag)
 }
 
 
+
+void MQResultSetView::editor_save(MGBlobEditor *editor)
+{
+  gpointer buffer;
+  gsize size;
+  Glib::ustring path;
+
+  editor->get_data(buffer, size);
+
+  path= (char*)((Glib::Object*)editor)->get_data("path");
+
+  g_free(buffer);
+}
+
+
+static MGBlobEditor::ViewType get_flags_for_data(char *fdata, gsize fsize,
+                                                 bool binary)
+{
+  MGBlobEditor::ViewType flags;
+  
+  if (fsize > 0 && myx_guess_image_format(fdata, fsize)!=MYX_IMG_UNKNOWN)
+    flags= MGBlobEditor::ViewType(MGBlobEditor::VImage|MGBlobEditor::VBinary);
+  else if (binary)
+    flags= MGBlobEditor::VBinary;
+  else
+    flags= MGBlobEditor::ViewType(MGBlobEditor::VText|MGBlobEditor::VBinary);
+
+  return flags;
+}
+
+void MQResultSetView::blob_edit(CellRendererBlob *cell, Gtk::Widget *widget, const Glib::ustring &path, int column)
+{
+  gpointer fdata; 
+  gsize fsize; 
+  Gtk::TreeIter iter= _model->get_iter(Gtk::TreePath(path));
+
+  if (_model->get_value(iter, column, fdata, fsize))
+  {
+    MGBlobEditor *editor= new MGBlobEditor(get_flags_for_data((char*)fdata, fsize, cell->is_binary()), true);
+    editor->set_data(fdata, fsize);
+    editor->resize(400, 300);
+    editor->set_delete_on_close();
+    
+    ((Glib::Object*)editor)->set_data("path", g_strdup(path.c_str()), g_free);
+    
+    editor->set_position(Gtk::WIN_POS_CENTER_ON_PARENT);
+    editor->set_transient_for(*(Gtk::Window*)widget->get_toplevel());
+    editor->property_destroy_with_parent()=true;
+    editor->signal_save().connect(SigC::bind<MGBlobEditor*>(SigC::slot(*this,&MQResultSetView::editor_save),editor));
+    
+    editor->show();
+  }
+}
+
+
+void MQResultSetView::blob_save(CellRendererBlob *cell, Gtk::Widget *widget, const Glib::ustring &path, int column)
+{
+  gpointer fdata; 
+  gsize fsize;
+  Gtk::TreeIter iter= _model->get_iter(Gtk::TreePath(path));
+
+  if (_model->get_value(iter, column, fdata, fsize))
+  {
+    Gtk::FileSelection fsel(_("Save Field Data"));
+    
+    if (fsel.run() == Gtk::RESPONSE_OK)
+    {
+      gsize written;
+      
+      try 
+      {
+        Glib::RefPtr<Glib::IOChannel> channel= Glib::IOChannel::create_from_file(fsel.get_filename(),"w+");
+        channel->set_encoding();
+        channel->write((char*)fdata, fsize, written);
+        channel->close();
+      } 
+      catch (Glib::FileError &exc)
+      {
+        myg_show_error(ufmt(_("Could not save data to file '%s'.\n"), fsel.get_filename().c_str())+exc.what());
+      }
+    }
+  }
+}
+
+
+void MQResultSetView::blob_load(CellRendererBlob *cell, Gtk::Widget *widget, const Glib::ustring &path, int column)
+{
+  Gtk::FileSelection fsel(_("Load Field Data"));
+
+  if (fsel.run() == Gtk::RESPONSE_OK)
+  {
+    Glib::RefPtr<Glib::IOChannel> channel;
+    try 
+    {
+      channel= Glib::IOChannel::create_from_file(fsel.get_filename(),"r");
+      channel->set_encoding();
+    } 
+    catch (Glib::Error &exc)
+    {
+      myg_show_error(ufmt(_("Could not open file '%s'"), fsel.get_filename().c_str())+"\n"+exc.what());
+      return;
+    }
+
+    try
+    {
+      gsize size= get_file_size(fsel.get_filename().c_str());
+      gchar *buffer= g_new0(gchar, size);
+
+      channel->read(buffer, size, size);
+
+      blob_data_edited(path, buffer, size, column);
+      
+      g_free(buffer);
+    }
+    catch (Glib::Error &exc)
+    {
+      myg_show_error(ufmt(_("Could not read data from file '%s'"), fsel.get_filename().c_str())+"\n"+exc.what());
+    }
+    channel->close();
+  }
+}
+
+
+void MQResultSetView::blob_clear(CellRendererBlob *cell, Gtk::Widget *widget, const Glib::ustring &path, int column)
+{
+  blob_data_edited(path, NULL, 0, column);
+}
+
+
+void MQResultSetView::blob_view(CellRendererBlob *cell, Gtk::Widget *widget, const Glib::ustring &path, int column)
+{
+  gpointer fdata; 
+  gsize fsize; 
+  Gtk::TreeIter iter= _model->get_iter(Gtk::TreePath(path));
+
+  if (_model->get_value(iter, column, fdata, fsize))
+  {
+    MGBlobEditor *editor= new MGBlobEditor(get_flags_for_data((char*)fdata, fsize, cell->is_binary()));
+    editor->set_data(fdata, fsize);
+    editor->resize(400, 300);
+    editor->set_delete_on_close();
+    
+    editor->set_position(Gtk::WIN_POS_CENTER_ON_PARENT);
+    editor->set_transient_for(*(Gtk::Window*)widget->get_toplevel());
+    editor->property_destroy_with_parent()=true;
+
+    editor->show();
+  }
+}
+
+
 void MQResultSetView::blob_data_edited(const Glib::ustring& path, gpointer data, gsize size, int column)
 {
   _model->set_value(_model->get_iter(path), column, data, size);
