@@ -69,11 +69,17 @@ bool MQResultTab::tab_will_close()
 
 void MQResultTab::set_top_widget(Gtk::Widget *w)
 {
-  if (_topPaned.get_child1())
+  if (_topPaned.get_child1() && _topPaned.get_child1()!=w)
     _topPaned.remove(*_topPaned.get_child1());
 
   if (w)
+  {
+    w->reference();
+    if (w->get_parent())
+      ((Gtk::Container*)w->get_parent())->remove(*w);
     _topPaned.pack1(*w, true, true);
+    w->unreference();
+  }
 }
 
 
@@ -208,8 +214,6 @@ void MQResultTab::add_resultset(MQResultSetView *rsv, bool vertical)
     rsv->set_compact_editbar(true);
 
   rsv->show();
-  rsv->set_active(true);
-  
 
   rsv->signal_editable_changed().connect(SigC::slot(*this,&MQResultTab::update_cmp_toolbar));
   rsv->signal_row_changed().connect(SigC::slot(*this,&MQResultTab::row_changed));
